# Auto-generated ARN patterns for matching
# Patterns are ordered: most specific first
import re

ARN_PATTERNS = {
    'a4b': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):a4b:(?P<Region>[\w-]*):(?P<Account>\d{12}):address-book/(?P<ResourceId>.+?)$"), ['addressbook']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):a4b:(?P<Region>[\w-]*):(?P<Account>\d{12}):conference-provider/(?P<ResourceId>.+?)$"), ['conferenceprovider']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):a4b:(?P<Region>[\w-]*):(?P<Account>\d{12}):contact/(?P<ResourceId>.+?)$"), ['contact']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):a4b:(?P<Region>[\w-]*):(?P<Account>\d{12}):device/(?P<ResourceId>.+?)$"), ['device']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):a4b:(?P<Region>[\w-]*):(?P<Account>\d{12}):gateway/(?P<ResourceId>.+?)$"), ['gateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):a4b:(?P<Region>[\w-]*):(?P<Account>\d{12}):gateway-group/(?P<ResourceId>.+?)$"), ['gatewaygroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):a4b:(?P<Region>[\w-]*):(?P<Account>\d{12}):network-profile/(?P<ResourceId>.+?)$"), ['networkprofile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):a4b:(?P<Region>[\w-]*):(?P<Account>\d{12}):profile/(?P<ResourceId>.+?)$"), ['profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):a4b:(?P<Region>[\w-]*):(?P<Account>\d{12}):room/(?P<ResourceId>.+?)$"), ['room']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):a4b:(?P<Region>[\w-]*):(?P<Account>\d{12}):schedule/(?P<ResourceId>.+?)$"), ['schedule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):a4b:(?P<Region>[\w-]*):(?P<Account>\d{12}):skill-group/(?P<ResourceId>.+?)$"), ['skillgroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):a4b:(?P<Region>[\w-]*):(?P<Account>\d{12}):user/(?P<ResourceId>.+?)$"), ['user']),
    ],
    'access-analyzer': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):access-analyzer:(?P<Region>[\w-]*):(?P<Account>\d{12}):analyzer/(?P<AnalyzerName>.+?)/archive-rule/(?P<RuleName>.+?)$"), ['ArchiveRule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):access-analyzer:(?P<Region>[\w-]*):(?P<Account>\d{12}):analyzer/(?P<AnalyzerName>.+?)$"), ['Analyzer']),
    ],
    'account': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):account::(?P<ManagementAccountId>.+?):account/o-(?P<OrganizationId>.+?)/(?P<MemberAccountId>.+?)$"), ['accountInOrganization']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):account::(?P<Account>\d{12}):account$"), ['account']),
    ],
    'acm': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):acm:(?P<Region>[\w-]*):(?P<Account>\d{12}):certificate/(?P<CertificateId>.+?)$"), ['certificate']),
    ],
    'acm-pca': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):acm-pca:(?P<Region>[\w-]*):(?P<Account>\d{12}):certificate-authority/(?P<CertificateAuthorityId>.+?)$"), ['certificate-authority']),
    ],
    'aidevops': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):aidevops:(?P<Region>[\w-]*):(?P<Account>\d{12}):agentspace/(?P<AgentSpaceId>.+?)/associations/(?P<AssociationId>.+?)$"), ['AssociationResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):aidevops:(?P<Region>[\w-]*):(?P<Account>\d{12}):agentspace/(?P<AgentSpaceId>.+?)$"), ['AgentSpaceResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):aidevops:(?P<Region>[\w-]*):(?P<Account>\d{12}):service/(?P<ServiceId>.+?)$"), ['ServiceResource']),
    ],
    'aiops': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):aiops:(?P<Region>[\w-]*):(?P<Account>\d{12}):investigation-group/(?P<InvestigationGroupId>.+?)$"), ['investigation-group']),
    ],
    'airflow': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):airflow:(?P<Region>[\w-]*):(?P<Account>\d{12}):role/(?P<EnvironmentName>.+?)/(?P<RoleName>.+?)$"), ['rbac-role']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):airflow:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment/(?P<EnvironmentName>.+?)$"), ['environment']),
    ],
    'airflow-serverless': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):airflow-serverless:(?P<Region>[\w-]*):(?P<Account>\d{12}):workflow/(?P<WorkflowId>.+?)$"), ['Workflow']),
    ],
    'amplify': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplify:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/branches/(?P<BranchName>.+?)/jobs/(?P<JobId>.+?)$"), ['jobs']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplify:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/branches/(?P<BranchName>.+?)$"), ['branches']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplify:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/domains/(?P<DomainName>.+?)$"), ['domains']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplify:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)$"), ['apps']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplify:(?P<Region>[\w-]*):(?P<Account>\d{12}):webhooks/(?P<WebhookId>.+?)$"), ['webhooks']),
    ],
    'amplifybackend': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplifybackend:(?P<Region>[\w-]*):(?P<Account>\d{12}):/backend/(?P<AppId>.+?)/api/(?P<ApiId>.+?)$"), ['api']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplifybackend:(?P<Region>[\w-]*):(?P<Account>\d{12}):/backend/(?P<AppId>.+?)/auth/(?P<AuthId>.+?)$"), ['auth']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplifybackend:(?P<Region>[\w-]*):(?P<Account>\d{12}):/backend/(?P<AppId>.+?)/challenge/(?P<ChallengeId>.+?)$"), ['token']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplifybackend:(?P<Region>[\w-]*):(?P<Account>\d{12}):/backend/(?P<AppId>.+?)/config/(?P<ConfigId>.+?)$"), ['config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplifybackend:(?P<Region>[\w-]*):(?P<Account>\d{12}):/backend/(?P<AppId>.+?)/environments/(?P<EnvironmentId>.+?)$"), ['environment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplifybackend:(?P<Region>[\w-]*):(?P<Account>\d{12}):/backend/(?P<AppId>.+?)/job/(?P<JobId>.+?)$"), ['job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplifybackend:(?P<Region>[\w-]*):(?P<Account>\d{12}):/backend/(?P<AppId>.+?)/storage/(?P<StorageId>.+?)$"), ['storage']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplifybackend:(?P<Region>[\w-]*):(?P<Account>\d{12}):/backend/(?P<AppId>.+?)/(?P<SubResourceId>.+?)$"), ['backend']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplifybackend:(?P<Region>[\w-]*):(?P<Account>\d{12}):/backend/(?P<BackendId>.+?)$"), ['created-backend']),
    ],
    'amplifyuibuilder': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplifyuibuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):app/(?P<AppId>.+?)/environment/(?P<EnvironmentName>.+?)/codegen-jobs/(?P<Id>.+?)$"), ['CodegenJobResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplifyuibuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):app/(?P<AppId>.+?)/environment/(?P<EnvironmentName>.+?)/components/(?P<Id>.+?)$"), ['ComponentResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplifyuibuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):app/(?P<AppId>.+?)/environment/(?P<EnvironmentName>.+?)/forms/(?P<Id>.+?)$"), ['FormResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):amplifyuibuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):app/(?P<AppId>.+?)/environment/(?P<EnvironmentName>.+?)/themes/(?P<Id>.+?)$"), ['ThemeResource']),
    ],
    'aoss': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):aoss:(?P<Region>[\w-]*):(?P<Account>\d{12}):collection/(?P<CollectionId>.+?)$"), ['Collection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):aoss:(?P<Region>[\w-]*):(?P<Account>\d{12}):collection-group/(?P<CollectionGroupId>.+?)$"), ['CollectionGroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):aoss:(?P<Region>[\w-]*):(?P<Account>\d{12}):dashboards/default$"), ['Dashboards']),
    ],
    'apigateway': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/resources/(?P<ResourceId>.+?)/methods/(?P<HttpMethodType>.+?)/integration/responses/(?P<StatusCode>.+?)$"), ['IntegrationResponse']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/resources/(?P<ResourceId>.+?)/methods/(?P<HttpMethodType>.+?)/responses/(?P<StatusCode>.+?)$"), ['MethodResponse']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/resources/(?P<ResourceId>.+?)/methods/(?P<HttpMethodType>.+?)/integration$"), ['Integration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/integrations/(?P<IntegrationId>.+?)/integrationresponses/(?P<IntegrationResponseId>.+?)$"), ['IntegrationResponse']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/routes/(?P<RouteId>.+?)/requestparameters/(?P<RequestParameterKey>.+?)$"), ['RouteRequestParameter']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/routes/(?P<RouteId>.+?)/routeresponses/(?P<RouteResponseId>.+?)$"), ['RouteResponse']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/stages/(?P<StageName>.+?)/cache/authorizers$"), ['AuthorizersCache']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/stages/(?P<StageName>.+?)/routesettings/(?P<RouteKey>.+?)$"), ['RouteSettings']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/domainnames/(?P<DomainName>.+?)\+(?P<DomainIdentifier>.+?)/basepathmappings/(?P<BasePath>.+?)$"), ['PrivateBasePathMapping']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/resources/(?P<ResourceId>.+?)/methods/(?P<HttpMethodType>.+?)$"), ['Method']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/stages/(?P<StageName>.+?)/sdks/(?P<SdkType>.+?)$"), ['Sdk']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/integrations/(?P<IntegrationId>.+?)/integrationresponses$"), ['IntegrationResponses']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/models/(?P<ModelId>.+?)/template$"), ['ModelTemplate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/routes/(?P<RouteId>.+?)/routeresponses$"), ['RouteResponses']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/stages/(?P<StageName>.+?)/accesslogsettings$"), ['AccessLogSettings']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/domainnames/(?P<DomainName>.+?)\+(?P<DomainIdentifier>.+?)/basepathmappings$"), ['PrivateBasePathMappings']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/documentation/parts/(?P<DocumentationPartId>.+?)$"), ['DocumentationPart']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/documentation/versions/(?P<DocumentationVersionId>.+?)$"), ['DocumentationVersion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/authorizers/(?P<AuthorizerId>.+?)$"), ['Authorizer']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/deployments/(?P<DeploymentId>.+?)$"), ['Deployment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/exports/(?P<Specification>.+?)$"), ['ExportedAPI']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/integrations/(?P<IntegrationId>.+?)$"), ['Integration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/models/(?P<ModelId>.+?)$"), ['Model']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/routes/(?P<RouteId>.+?)$"), ['Route']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/stages/(?P<StageName>.+?)$"), ['Stage']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/domainnames/(?P<DomainName>.+?)/apimappings/(?P<ApiMappingId>.+?)$"), ['ApiMapping']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/domainnames/(?P<DomainName>.+?)/basepathmappings/(?P<BasePath>.+?)$"), ['BasePathMapping']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/models/(?P<ModelName>.+?)/template$"), ['Template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/authorizers/(?P<AuthorizerId>.+?)$"), ['Authorizer']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/deployments/(?P<DeploymentId>.+?)$"), ['Deployment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/documentation/parts$"), ['DocumentationParts']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/documentation/versions$"), ['DocumentationVersions']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/gatewayresponses/(?P<ResponseType>.+?)$"), ['GatewayResponse']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/models/(?P<ModelName>.+?)$"), ['Model']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/requestvalidators/(?P<RequestValidatorId>.+?)$"), ['RequestValidator']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/resources/(?P<ResourceId>.+?)$"), ['Resource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/stages/(?P<StageName>.+?)$"), ['Stage']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<ApiId>.+?)/stages/(?P<StageName>.+?)$"), ['apigateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/usageplans/(?P<UsagePlanId>.+?)/keys/(?P<Id>.+?)$"), ['UsagePlanKey']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/authorizers$"), ['Authorizers']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/cors$"), ['Cors']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/deployments$"), ['Deployments']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/integrations$"), ['Integrations']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/models$"), ['Models']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/routes$"), ['Routes']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)/stages$"), ['Stages']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/domainnames/(?P<DomainName>.+?)/apimappings$"), ['ApiMappings']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/domainnames/(?P<DomainName>.+?)/basepathmappings$"), ['BasePathMappings']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/authorizers$"), ['Authorizers']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/deployments$"), ['Deployments']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/gatewayresponses$"), ['GatewayResponses']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/models$"), ['Models']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/requestvalidators$"), ['RequestValidators']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/resources$"), ['Resources']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)/stages$"), ['Stages']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/usageplans/(?P<UsagePlanId>.+?)/keys$"), ['UsagePlanKeys']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apikeys/(?P<ApiKeyId>.+?)$"), ['ApiKey']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis/(?P<ApiId>.+?)$"), ['Api']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/clientcertificates/(?P<ClientCertificateId>.+?)$"), ['ClientCertificate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/domainnames/(?P<DomainName>.+?)$"), ['DomainName']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis/(?P<RestApiId>.+?)$"), ['RestApi']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/tags/(?P<UrlEncodedResourceARN>.+?)$"), ['Tags']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/usageplans/(?P<UsagePlanId>.+?)$"), ['UsagePlan']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/vpclinks/(?P<VpcLinkId>.+?)$"), ['VpcLink']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/account$"), ['Account']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apikeys$"), ['ApiKeys']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/apis$"), ['Apis']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/clientcertificates$"), ['ClientCertificates']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/domainnames$"), ['DomainNames']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/restapis$"), ['RestApis']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/usageplans$"), ['UsagePlans']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*)::/vpclinks$"), ['VpcLinks']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):/domainnameaccessassociations/domainname/(?P<DomainName>.+?)/(?P<SourceType>.+?)/(?P<SourceId>.+?)$"), ['DomainNameAccessAssociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):/domainnames/(?P<DomainName>.+?)\+(?P<DomainIdentifier>.+?)$"), ['PrivateDomainName']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):/domainnames/(?P<DomainName>.+?)/routingrules/(?P<RoutingRuleId>.+?)$"), ['RoutingRule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):/portalproducts/(?P<PortalProductId>.+?)/productpages/(?P<ProductPageId>.+?)$"), ['ProductPage']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):/portalproducts/(?P<PortalProductId>.+?)/productrestendpointpages/(?P<ProductRestEndpointPageId>.+?)$"), ['ProductRestEndpointPage']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):/portalproducts/(?P<PortalProductId>.+?)$"), ['PortalProduct']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):/portals/(?P<PortalId>.+?)$"), ['Portal']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apigateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):/domainnameaccessassociations$"), ['DomainNameAccessAssociations']),
    ],
    'app-integrations': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):app-integrations:(?P<Region>[\w-]*):(?P<Account>\d{12}):application-association/(?P<ApplicationId>.+?)/(?P<ApplicationAssociationId>.+?)$"), ['application-association']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):app-integrations:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-integration-association/(?P<DataIntegrationId>.+?)/(?P<ResourceId>.+?)$"), ['data-integration-association']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):app-integrations:(?P<Region>[\w-]*):(?P<Account>\d{12}):event-integration-association/(?P<EventIntegrationName>.+?)/(?P<ResourceId>.+?)$"), ['event-integration-association']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):app-integrations:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)$"), ['application']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):app-integrations:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-integration/(?P<DataIntegrationId>.+?)$"), ['data-integration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):app-integrations:(?P<Region>[\w-]*):(?P<Account>\d{12}):event-integration/(?P<EventIntegrationName>.+?)$"), ['event-integration']),
    ],
    'appconfig': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appconfig:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/configurationprofile/(?P<ConfigurationProfileId>.+?)/hostedconfigurationversion/(?P<VersionNumber>.+?)$"), ['hostedconfigurationversion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appconfig:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/environment/(?P<EnvironmentId>.+?)/configuration/(?P<ConfigurationProfileId>.+?)$"), ['configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appconfig:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/environment/(?P<EnvironmentId>.+?)/deployment/(?P<DeploymentNumber>.+?)$"), ['deployment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appconfig:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/configurationprofile/(?P<ConfigurationProfileId>.+?)$"), ['configurationprofile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appconfig:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/environment/(?P<EnvironmentId>.+?)$"), ['environment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appconfig:(?P<Region>[\w-]*):(?P<Account>\d{12}):extension/(?P<ExtensionId>.+?)/(?P<ExtensionVersionNumber>.+?)$"), ['extension']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appconfig:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)$"), ['application']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appconfig:(?P<Region>[\w-]*):(?P<Account>\d{12}):deploymentstrategy/(?P<DeploymentStrategyId>.+?)$"), ['deploymentstrategy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appconfig:(?P<Region>[\w-]*):(?P<Account>\d{12}):extensionassociation/(?P<ExtensionAssociationId>.+?)$"), ['extensionassociation']),
    ],
    'appfabric': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appfabric:(?P<Region>[\w-]*):(?P<Account>\d{12}):appbundle/(?P<AppbundleId>.+?)/ingestion/(?P<IngestionIdentifier>.+?)/ingestiondestination/(?P<IngestionDestinationIdentifier>.+?)$"), ['ingestiondestination']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appfabric:(?P<Region>[\w-]*):(?P<Account>\d{12}):appbundle/(?P<AppbundleId>.+?)/appauthorization/(?P<AppAuthorizationIdentifier>.+?)$"), ['appauthorization']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appfabric:(?P<Region>[\w-]*):(?P<Account>\d{12}):appbundle/(?P<AppbundleId>.+?)/ingestion/(?P<IngestionIdentifier>.+?)$"), ['ingestion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appfabric:(?P<Region>[\w-]*):(?P<Account>\d{12}):appbundle/(?P<AppBundleIdentifier>.+?)$"), ['appbundle']),
    ],
    'appflow': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appflow:(?P<Region>[\w-]*):(?P<Account>\d{12}):connector/(?P<ConnectorLabel>.+?)$"), ['connector']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appflow:(?P<Region>[\w-]*):(?P<Account>\d{12}):connectorprofile/(?P<ProfileName>.+?)$"), ['connectorprofile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appflow:(?P<Region>[\w-]*):(?P<Account>\d{12}):flow/(?P<FlowName>.+?)$"), ['flow']),
    ],
    'application-autoscaling': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):application-autoscaling:(?P<Region>[\w-]*):(?P<Account>\d{12}):scalable-target/(?P<ResourceId>.+?)$"), ['ScalableTarget']),
    ],
    'application-signals': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):application-signals:(?P<Region>[\w-]*):(?P<Account>\d{12}):slo/(?P<SloName>.+?)$"), ['slo']),
    ],
    'appmesh': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appmesh:(?P<Region>[\w-]*):(?P<Account>\d{12}):mesh/(?P<MeshName>.+?)/virtualGateway/(?P<VirtualGatewayName>.+?)/gatewayRoute/(?P<GatewayRouteName>.+?)$"), ['gatewayRoute']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appmesh:(?P<Region>[\w-]*):(?P<Account>\d{12}):mesh/(?P<MeshName>.+?)/virtualRouter/(?P<VirtualRouterName>.+?)/route/(?P<RouteName>.+?)$"), ['route']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appmesh:(?P<Region>[\w-]*):(?P<Account>\d{12}):mesh/(?P<MeshName>.+?)/virtualGateway/(?P<VirtualGatewayName>.+?)$"), ['virtualGateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appmesh:(?P<Region>[\w-]*):(?P<Account>\d{12}):mesh/(?P<MeshName>.+?)/virtualNode/(?P<VirtualNodeName>.+?)$"), ['virtualNode']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appmesh:(?P<Region>[\w-]*):(?P<Account>\d{12}):mesh/(?P<MeshName>.+?)/virtualRouter/(?P<VirtualRouterName>.+?)$"), ['virtualRouter']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appmesh:(?P<Region>[\w-]*):(?P<Account>\d{12}):mesh/(?P<MeshName>.+?)/virtualService/(?P<VirtualServiceName>.+?)$"), ['virtualService']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appmesh:(?P<Region>[\w-]*):(?P<Account>\d{12}):mesh/(?P<MeshName>.+?)$"), ['mesh']),
    ],
    'appmesh-preview': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appmesh-preview:(?P<Region>[\w-]*):(?P<Account>\d{12}):mesh/(?P<MeshName>.+?)/virtualGateway/(?P<VirtualGatewayName>.+?)/gatewayRoute/(?P<GatewayRouteName>.+?)$"), ['gatewayRoute']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appmesh-preview:(?P<Region>[\w-]*):(?P<Account>\d{12}):mesh/(?P<MeshName>.+?)/virtualRouter/(?P<VirtualRouterName>.+?)/route/(?P<RouteName>.+?)$"), ['route']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appmesh-preview:(?P<Region>[\w-]*):(?P<Account>\d{12}):mesh/(?P<MeshName>.+?)/virtualGateway/(?P<VirtualGatewayName>.+?)$"), ['virtualGateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appmesh-preview:(?P<Region>[\w-]*):(?P<Account>\d{12}):mesh/(?P<MeshName>.+?)/virtualNode/(?P<VirtualNodeName>.+?)$"), ['virtualNode']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appmesh-preview:(?P<Region>[\w-]*):(?P<Account>\d{12}):mesh/(?P<MeshName>.+?)/virtualRouter/(?P<VirtualRouterName>.+?)$"), ['virtualRouter']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appmesh-preview:(?P<Region>[\w-]*):(?P<Account>\d{12}):mesh/(?P<MeshName>.+?)/virtualService/(?P<VirtualServiceName>.+?)$"), ['virtualService']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appmesh-preview:(?P<Region>[\w-]*):(?P<Account>\d{12}):mesh/(?P<MeshName>.+?)$"), ['mesh']),
    ],
    'apprunner': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apprunner:(?P<Region>[\w-]*):(?P<Account>\d{12}):autoscalingconfiguration/(?P<AutoscalingConfigurationName>.+?)/(?P<AutoscalingConfigurationVersion>.+?)/(?P<AutoscalingConfigurationId>.+?)$"), ['autoscalingconfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apprunner:(?P<Region>[\w-]*):(?P<Account>\d{12}):observabilityconfiguration/(?P<ObservabilityConfigurationName>.+?)/(?P<ObservabilityConfigurationVersion>.+?)/(?P<ObservabilityConfigurationId>.+?)$"), ['observabilityconfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apprunner:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpcconnector/(?P<VpcConnectorName>.+?)/(?P<VpcConnectorVersion>.+?)/(?P<VpcConnectorId>.+?)$"), ['vpcconnector']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apprunner:(?P<Region>[\w-]*):(?P<Account>\d{12}):connection/(?P<ConnectionName>.+?)/(?P<ConnectionId>.+?)$"), ['connection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apprunner:(?P<Region>[\w-]*):(?P<Account>\d{12}):service/(?P<ServiceName>.+?)/(?P<ServiceId>.+?)$"), ['service']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apprunner:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpcingressconnection/(?P<VpcIngressConnectionName>.+?)/(?P<VpcIngressConnectionId>.+?)$"), ['vpcingressconnection']),
    ],
    'appstream': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appstream:(?P<Region>[\w-]*):(?P<Account>\d{12}):app-block/(?P<AppBlockName>.+?)$"), ['app-block']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appstream:(?P<Region>[\w-]*):(?P<Account>\d{12}):app-block-builder/(?P<AppBlockBuilderName>.+?)$"), ['app-block-builder']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appstream:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationName>.+?)$"), ['application']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appstream:(?P<Region>[\w-]*):(?P<Account>\d{12}):fleet/(?P<FleetName>.+?)$"), ['fleet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appstream:(?P<Region>[\w-]*):(?P<Account>\d{12}):image/(?P<ImageName>.+?)$"), ['image']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appstream:(?P<Region>[\w-]*):(?P<Account>\d{12}):image-builder/(?P<ImageBuilderName>.+?)$"), ['image-builder']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appstream:(?P<Region>[\w-]*):(?P<Account>\d{12}):stack/(?P<StackName>.+?)$"), ['stack']),
    ],
    'appstudio': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appstudio:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)$"), ['application']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appstudio:(?P<Region>[\w-]*):(?P<Account>\d{12}):connector/(?P<ConnectionId>.+?)$"), ['connector']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appstudio:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)$"), ['instance']),
    ],
    'appsync': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appsync:(?P<Region>[\w-]*):(?P<Account>\d{12}):apis/(?P<GraphQLAPIId>.+?)/types/(?P<TypeName>.+?)/fields/(?P<FieldName>.+?)$"), ['field']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appsync:(?P<Region>[\w-]*):(?P<Account>\d{12}):apis/(?P<ApiId>.+?)/channelNamespace/(?P<ChannelNamespaceName>.+?)$"), ['channelNamespace']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appsync:(?P<Region>[\w-]*):(?P<Account>\d{12}):apis/(?P<GraphQLAPIId>.+?)/datasources/(?P<DatasourceName>.+?)$"), ['datasource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appsync:(?P<Region>[\w-]*):(?P<Account>\d{12}):apis/(?P<GraphQLAPIId>.+?)/functions/(?P<FunctionId>.+?)$"), ['function']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appsync:(?P<Region>[\w-]*):(?P<Account>\d{12}):apis/(?P<SourceGraphQLAPIId>.+?)/mergedApiAssociations/(?P<Associationid>.+?)$"), ['mergedApiAssociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appsync:(?P<Region>[\w-]*):(?P<Account>\d{12}):apis/(?P<MergedGraphQLAPIId>.+?)/sourceApiAssociations/(?P<Associationid>.+?)$"), ['sourceApiAssociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appsync:(?P<Region>[\w-]*):(?P<Account>\d{12}):apis/(?P<GraphQLAPIId>.+?)/types/(?P<TypeName>.+?)$"), ['type']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appsync:(?P<Region>[\w-]*):(?P<Account>\d{12}):apis/(?P<GraphQLAPIId>.+?)$"), ['graphqlapi']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appsync:(?P<Region>[\w-]*):(?P<Account>\d{12}):apis/(?P<ApiId>.+?)$"), ['api']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):appsync:(?P<Region>[\w-]*):(?P<Account>\d{12}):domainnames/(?P<DomainName>.+?)$"), ['domain']),
    ],
    'apptest': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apptest:(?P<Region>[\w-]*):(?P<Account>\d{12}):testcase/(?P<TestCaseId>.+?)$"), ['TestCase']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apptest:(?P<Region>[\w-]*):(?P<Account>\d{12}):testconfiguration/(?P<TestConfigurationId>.+?)$"), ['TestConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apptest:(?P<Region>[\w-]*):(?P<Account>\d{12}):testrun/(?P<TestRunId>.+?)$"), ['TestRun']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):apptest:(?P<Region>[\w-]*):(?P<Account>\d{12}):testsuite/(?P<TestSuiteId>.+?)$"), ['TestSuite']),
    ],
    'aps': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):aps:(?P<Region>[\w-]*):(?P<Account>\d{12}):anomalydetector/(?P<WorkspaceId>.+?)/(?P<AnomalyDetectorId>.+?)$"), ['anomalydetector']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):aps:(?P<Region>[\w-]*):(?P<Account>\d{12}):rulegroupsnamespace/(?P<WorkspaceId>.+?)/(?P<Namespace>.+?)$"), ['rulegroupsnamespace']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):aps:(?P<Region>[\w-]*):(?P<Account>\d{12}):scraper/(?P<ScraperId>.+?)$"), ['scraper']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):aps:(?P<Region>[\w-]*):(?P<Account>\d{12}):workspace/(?P<WorkspaceId>.+?)$"), ['workspace']),
    ],
    'arc-region-switch': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):arc-region-switch::(?P<Account>\d{12}):plan/(?P<ResourceId>.+?)$"), ['plan']),
    ],
    'artifact': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):artifact:::agreement/(?P<AgreementId>.+?)$"), ['agreement']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):artifact::(?P<Account>\d{12}):customer-agreement/(?P<CustomerAgreementId>.+?)$"), ['customer-agreement']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):artifact:(?P<Region>[\w-]*)::report/(?P<ReportId>.+?):(?P<Version>.+?)$"), ['report']),
    ],
    'athena': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):athena:(?P<Region>[\w-]*):(?P<Account>\d{12}):workgroup/(?P<WorkGroupName>.+?)/session/(?P<SessionId>.+?)$"), ['session']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):athena:(?P<Region>[\w-]*):(?P<Account>\d{12}):capacity-reservation/(?P<CapacityReservationName>.+?)$"), ['capacity-reservation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):athena:(?P<Region>[\w-]*):(?P<Account>\d{12}):datacatalog/(?P<DataCatalogName>.+?)$"), ['datacatalog']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):athena:(?P<Region>[\w-]*):(?P<Account>\d{12}):workgroup/(?P<WorkGroupName>.+?)$"), ['workgroup']),
    ],
    'auditmanager': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):auditmanager:(?P<Region>[\w-]*):(?P<Account>\d{12}):assessment/(?P<AssessmentId>.+?)/controlSet/(?P<ControlSetId>.+?)$"), ['assessmentControlSet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):auditmanager:(?P<Region>[\w-]*):(?P<Account>\d{12}):assessment/(?P<AssessmentId>.+?)$"), ['assessment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):auditmanager:(?P<Region>[\w-]*):(?P<Account>\d{12}):assessmentFramework/(?P<AssessmentFrameworkId>.+?)$"), ['assessmentFramework']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):auditmanager:(?P<Region>[\w-]*):(?P<Account>\d{12}):control/(?P<ControlId>.+?)$"), ['control']),
    ],
    'autoscaling': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):autoscaling:(?P<Region>[\w-]*):(?P<Account>\d{12}):autoScalingGroup:(?P<GroupId>.+?):autoScalingGroupName/(?P<GroupFriendlyName>.+?)$"), ['autoScalingGroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):autoscaling:(?P<Region>[\w-]*):(?P<Account>\d{12}):launchConfiguration:(?P<Id>.+?):launchConfigurationName/(?P<LaunchConfigurationName>.+?)$"), ['launchConfiguration']),
    ],
    'aws-marketplace': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):aws-marketplace:(?P<Region>[\w-]*):(?P<Account>\d{12}):DeploymentParameter:catalogs/(?P<CatalogName>.+?)/products/(?P<ProductId>.+?)/(?P<ResourceId>.+?)$"), ['DeploymentParameter']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):aws-marketplace:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<Catalog>.+?)/ChangeSet/(?P<ResourceId>.+?)$"), ['ChangeSet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):aws-marketplace:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<Catalog>.+?)/(?P<EntityType>.+?)/(?P<ResourceId>.+?)$"), ['Entity']),
    ],
    'b2bi': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):b2bi:(?P<Region>[\w-]*):(?P<Account>\d{12}):capability/(?P<ResourceId>.+?)$"), ['capability']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):b2bi:(?P<Region>[\w-]*):(?P<Account>\d{12}):partnership/(?P<ResourceId>.+?)$"), ['partnership']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):b2bi:(?P<Region>[\w-]*):(?P<Account>\d{12}):profile/(?P<ResourceId>.+?)$"), ['profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):b2bi:(?P<Region>[\w-]*):(?P<Account>\d{12}):transformer/(?P<ResourceId>.+?)$"), ['transformer']),
    ],
    'backup': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):backup:(?P<Region>[\w-]*):(?P<Account>\d{12}):framework:(?P<FrameworkName>.+?)-(?P<FrameworkId>.+?)$"), ['framework']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):backup:(?P<Region>[\w-]*):(?P<Account>\d{12}):report-plan:(?P<ReportPlanName>.+?)-(?P<ReportPlanId>.+?)$"), ['reportPlan']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):backup:(?P<Region>[\w-]*):(?P<Account>\d{12}):restore-testing-plan:(?P<RestoreTestingPlanName>.+?)-(?P<RestoreTestingPlanId>.+?)$"), ['restoreTestingPlan']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):backup:(?P<Region>[\w-]*):(?P<Account>\d{12}):tiering-configuration:(?P<TieringConfigurationName>.+?)-(?P<TieringConfigurationId>.+?)$"), ['tieringConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):backup:(?P<Region>[\w-]*):(?P<Account>\d{12}):backup-plan:(?P<BackupPlanId>.+?)$"), ['backupPlan', 'backup-plan']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):backup:(?P<Region>[\w-]*):(?P<Account>\d{12}):backup-vault:(?P<BackupVaultName>.+?)$"), ['backupVault', 'backup-vault']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):backup:(?P<Region>[\w-]*):(?P<Account>\d{12}):legal-hold:(?P<LegalHoldId>.+?)$"), ['legalHold']),
    ],
    'backup-gateway': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):backup-gateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):gateway/(?P<GatewayId>.+?)$"), ['gateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):backup-gateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):hypervisor/(?P<HypervisorId>.+?)$"), ['hypervisor']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):backup-gateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):vm/(?P<VirtualmachineId>.+?)$"), ['virtualmachine']),
    ],
    'backup-search': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):backup-search:(?P<Region>[\w-]*):(?P<Account>\d{12}):search-export-job/(?P<ResourceId>.+?)$"), ['searchExportJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):backup-search:(?P<Region>[\w-]*):(?P<Account>\d{12}):search-job/(?P<ResourceId>.+?)$"), ['searchJob']),
    ],
    'batch': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):batch:(?P<Region>[\w-]*):(?P<Account>\d{12}):job-definition/(?P<JobDefinitionName>.+?):(?P<Revision>.+?)$"), ['job-definition-revision']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):batch:(?P<Region>[\w-]*):(?P<Account>\d{12}):compute-environment/(?P<ComputeEnvironmentName>.+?)$"), ['compute-environment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):batch:(?P<Region>[\w-]*):(?P<Account>\d{12}):consumable-resource/(?P<ConsumableResourceName>.+?)$"), ['consumable-resource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):batch:(?P<Region>[\w-]*):(?P<Account>\d{12}):job/(?P<JobId>.+?)$"), ['job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):batch:(?P<Region>[\w-]*):(?P<Account>\d{12}):job-definition/(?P<JobDefinitionName>.+?)$"), ['job-definition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):batch:(?P<Region>[\w-]*):(?P<Account>\d{12}):job-queue/(?P<JobQueueName>.+?)$"), ['job-queue']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):batch:(?P<Region>[\w-]*):(?P<Account>\d{12}):scheduling-policy/(?P<SchedulingPolicyName>.+?)$"), ['scheduling-policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):batch:(?P<Region>[\w-]*):(?P<Account>\d{12}):service-environment/(?P<ServiceEnvironmentName>.+?)$"), ['service-environment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):batch:(?P<Region>[\w-]*):(?P<Account>\d{12}):service-job/(?P<JobId>.+?)$"), ['service-job']),
    ],
    'bcm-data-exports': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bcm-data-exports:(?P<Region>[\w-]*):(?P<Account>\d{12}):export/(?P<Identifier>.+?)$"), ['export']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bcm-data-exports:(?P<Region>[\w-]*):(?P<Account>\d{12}):table/(?P<Identifier>.+?)$"), ['table']),
    ],
    'bcm-pricing-calculator': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bcm-pricing-calculator:(?P<Region>[\w-]*):(?P<Account>\d{12}):bill-estimate/(?P<BillEstimateId>.+?)$"), ['bill-estimate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bcm-pricing-calculator:(?P<Region>[\w-]*):(?P<Account>\d{12}):bill-scenario/(?P<BillScenarioId>.+?)$"), ['bill-scenario']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bcm-pricing-calculator:(?P<Region>[\w-]*):(?P<Account>\d{12}):workload-estimate/(?P<WorkloadEstimateId>.+?)$"), ['workload-estimate']),
    ],
    'bedrock': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock::(?P<Account>\d{12}):system-tool/(?P<ResourceId>.+?)$"), ['system-tool']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*)::foundation-model/(?P<ResourceId>.+?)$"), ['foundation-model']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):flow/(?P<FlowId>.+?)/alias/(?P<FlowAliasId>.+?)/execution/(?P<FlowExecutionId>.+?)$"), ['flow-execution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):flow/(?P<FlowId>.+?)/alias/(?P<FlowAliasId>.+?)$"), ['flow-alias']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):agent-alias/(?P<AgentId>.+?)/(?P<AgentAliasId>.+?)$"), ['agent-alias']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):automated-reasoning-policy/(?P<AutomatedReasoningPolicyId>.+?):(?P<AutomatedReasoningPolicyVersion>.+?)$"), ['automated-reasoning-policy-version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):marketplace/model-endpoint/all-access$"), ['bedrock-marketplace-model-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):prompt/(?P<PromptId>.+?):(?P<PromptVersion>.+?)$"), ['prompt-version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):agent/(?P<AgentId>.+?)$"), ['agent']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):application-inference-profile/(?P<ResourceId>.+?)$"), ['application-inference-profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):async-invoke/(?P<ResourceId>.+?)$"), ['async-invoke']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):automated-reasoning-policy/(?P<AutomatedReasoningPolicyId>.+?)$"), ['automated-reasoning-policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):blueprint/(?P<BlueprintId>.+?)$"), ['blueprint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):custom-model/(?P<ResourceId>.+?)$"), ['custom-model']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):custom-model-deployment/(?P<ResourceId>.+?)$"), ['custom-model-deployment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-automation-invocation/(?P<JobId>.+?)$"), ['data-automation-invocation-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-automation-profile/(?P<ProfileId>.+?)$"), ['data-automation-profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-automation-project/(?P<ProjectId>.+?)$"), ['data-automation-project']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):default-prompt-router/(?P<ResourceId>.+?)$"), ['default-prompt-router']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):evaluation-job/(?P<ResourceId>.+?)$"), ['evaluation-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):flow/(?P<FlowId>.+?)$"), ['flow']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):guardrail/(?P<GuardrailId>.+?)$"), ['guardrail']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):guardrail-profile/(?P<ResourceId>.+?)$"), ['guardrail-profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):imported-model/(?P<ResourceId>.+?)$"), ['imported-model']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):inference-profile/(?P<ResourceId>.+?)$"), ['inference-profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):knowledge-base/(?P<KnowledgeBaseId>.+?)$"), ['knowledge-base']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):model-copy-job/(?P<ResourceId>.+?)$"), ['model-copy-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):model-customization-job/(?P<ResourceId>.+?)$"), ['model-customization-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):model-evaluation-job/(?P<ResourceId>.+?)$"), ['model-evaluation-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):model-import-job/(?P<ResourceId>.+?)$"), ['model-import-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):model-invocation-job/(?P<JobIdentifier>.+?)$"), ['model-invocation-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):prompt/(?P<PromptId>.+?)$"), ['prompt']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):prompt-router/(?P<ResourceId>.+?)$"), ['prompt-router']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):provisioned-model/(?P<ResourceId>.+?)$"), ['provisioned-model']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock:(?P<Region>[\w-]*):(?P<Account>\d{12}):session/(?P<SessionId>.+?)$"), ['session']),
    ],
    'bedrock-agentcore': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):aws:browser/(?P<BrowserId>.+?)$"), ['browser']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):aws:code-interpreter/(?P<CodeInterpreterId>.+?)$"), ['code-interpreter']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):policy-engine/(?P<PolicyEngineId>.+?)/policy/(?P<PolicyId>.+?)$"), ['policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):policy-engine/(?P<PolicyEngineId>.+?)/policy-generation/(?P<PolicyGenerationId>.+?)$"), ['policy-generation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):runtime/(?P<RuntimeId>.+?)/runtime-endpoint/(?P<Name>.+?)$"), ['runtime-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):token-vault/(?P<TokenVaultId>.+?)/apikeycredentialprovider/(?P<Name>.+?)$"), ['apikeycredentialprovider']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):token-vault/(?P<TokenVaultId>.+?)/oauth2credentialprovider/(?P<Name>.+?)$"), ['oauth2credentialprovider']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):workload-identity-directory/(?P<DirectoryId>.+?)/workload-identity/(?P<WorkloadIdentityName>.+?)$"), ['workload-identity']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):browser-custom/(?P<BrowserId>.+?)$"), ['browser-custom']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):code-interpreter-custom/(?P<CodeInterpreterId>.+?)$"), ['code-interpreter-custom']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):evaluator/(?P<EvaluatorId>.+?)$"), ['evaluator']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):gateway/(?P<GatewayId>.+?)$"), ['gateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):memory/(?P<MemoryId>.+?)$"), ['memory']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):online-evaluation-config/(?P<OnlineEvaluationConfigId>.+?)$"), ['online-evaluation-config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):policy-engine/(?P<PolicyEngineId>.+?)$"), ['policy-engine']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):runtime/(?P<RuntimeId>.+?)$"), ['runtime']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):token-vault/(?P<TokenVaultId>.+?)$"), ['token-vault']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-agentcore:(?P<Region>[\w-]*):(?P<Account>\d{12}):workload-identity-directory/(?P<DirectoryId>.+?)$"), ['workload-identity-directory']),
    ],
    'bedrock-mantle': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bedrock-mantle:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ResourceId>.+?)$"), ['project']),
    ],
    'billing': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):billing::(?P<Account>\d{12}):billingview/(?P<ResourceId>.+?)$"), ['billingview']),
    ],
    'billingconductor': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):billingconductor::(?P<Account>\d{12}):billinggroup/(?P<BillingGroupId>.+?)$"), ['billinggroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):billingconductor::(?P<Account>\d{12}):customlineitem/(?P<CustomLineItemId>.+?)$"), ['customlineitem']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):billingconductor::(?P<Account>\d{12}):pricingplan/(?P<PricingPlanId>.+?)$"), ['pricingplan']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):billingconductor::(?P<Account>\d{12}):pricingrule/(?P<PricingRuleId>.+?)$"), ['pricingrule']),
    ],
    'braket': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):braket:(?P<Region>[\w-]*):(?P<Account>\d{12}):job/(?P<JobName>.+?)$"), ['job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):braket:(?P<Region>[\w-]*):(?P<Account>\d{12}):quantum-task/(?P<RandomId>.+?)$"), ['quantum-task']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):braket:(?P<Region>[\w-]*):(?P<Account>\d{12}):spending-limit/(?P<RandomId>.+?)$"), ['spending-limit']),
    ],
    'budgets': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):budgets::(?P<Account>\d{12}):budget/(?P<BudgetName>.+?)/action/(?P<ActionId>.+?)$"), ['budgetAction']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):budgets::(?P<Account>\d{12}):budget/(?P<BudgetName>.+?)$"), ['budget']),
    ],
    'bugbust': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):bugbust:(?P<Region>[\w-]*):(?P<Account>\d{12}):events/(?P<EventId>.+?)$"), ['Event']),
    ],
    'cases': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cases:(?P<Region>[\w-]*):(?P<Account>\d{12}):domain/(?P<DomainId>.+?)/case/(?P<CaseId>.+?)/related-item/(?P<RelatedItemId>.+?)$"), ['RelatedItem']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cases:(?P<Region>[\w-]*):(?P<Account>\d{12}):domain/(?P<DomainId>.+?)/case/(?P<CaseId>.+?)$"), ['Case']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cases:(?P<Region>[\w-]*):(?P<Account>\d{12}):domain/(?P<DomainId>.+?)/case-rule/(?P<CaseRuleId>.+?)$"), ['CaseRule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cases:(?P<Region>[\w-]*):(?P<Account>\d{12}):domain/(?P<DomainId>.+?)/field/(?P<FieldId>.+?)$"), ['Field']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cases:(?P<Region>[\w-]*):(?P<Account>\d{12}):domain/(?P<DomainId>.+?)/layout/(?P<LayoutId>.+?)$"), ['Layout']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cases:(?P<Region>[\w-]*):(?P<Account>\d{12}):domain/(?P<DomainId>.+?)/template/(?P<TemplateId>.+?)$"), ['Template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cases:(?P<Region>[\w-]*):(?P<Account>\d{12}):domain/(?P<DomainId>.+?)$"), ['Domain']),
    ],
    'cassandra': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cassandra:(?P<Region>[\w-]*):(?P<Account>\d{12}):/keyspace/(?P<KeyspaceName>.+?)/table/(?P<TableName>.+?)/stream/(?P<StreamLabel>.+?)$"), ['stream']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cassandra:(?P<Region>[\w-]*):(?P<Account>\d{12}):/keyspace/(?P<KeyspaceName>.+?)/table/(?P<TableName>.+?)$"), ['table']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cassandra:(?P<Region>[\w-]*):(?P<Account>\d{12}):/keyspace/(?P<KeyspaceName>.+?)/$"), ['keyspace']),
    ],
    'catalog': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):catalog:(?P<Region>[\w-]*):(?P<Account>\d{12}):portfolio/(?P<PortfolioId>.+?)$"), ['Portfolio']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):catalog:(?P<Region>[\w-]*):(?P<Account>\d{12}):product/(?P<ProductId>.+?)$"), ['Product']),
    ],
    'ce': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ce::(?P<Account>\d{12}):anomalymonitor/(?P<Identifier>.+?)$"), ['anomalymonitor']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ce::(?P<Account>\d{12}):anomalysubscription/(?P<Identifier>.+?)$"), ['anomalysubscription']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ce::(?P<Account>\d{12}):costcategory/(?P<Identifier>.+?)$"), ['costcategory']),
    ],
    'chatbot': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chatbot::(?P<Account>\d{12}):chat-configuration/(?P<ConfigurationType>.+?)/(?P<ChatbotConfigurationName>.+?)$"), ['ChatbotConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chatbot::(?P<Account>\d{12}):custom-action/(?P<ActionName>.+?)$"), ['custom-action']),
    ],
    'chime': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chime::(?P<AccountId>.+?):meeting/(?P<MeetingId>.+?)$"), ['meeting']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chime:(?P<Region>[\w-]*):(?P<AccountId>.+?):app-instance/(?P<AppInstanceId>.+?)/bot/(?P<AppInstanceBotId>.+?)$"), ['app-instance-bot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chime:(?P<Region>[\w-]*):(?P<AccountId>.+?):app-instance/(?P<AppInstanceId>.+?)/channel/(?P<ChannelId>.+?)$"), ['channel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chime:(?P<Region>[\w-]*):(?P<AccountId>.+?):app-instance/(?P<AppInstanceId>.+?)/channel-flow/(?P<ChannelFlowId>.+?)$"), ['channel-flow']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chime:(?P<Region>[\w-]*):(?P<AccountId>.+?):app-instance/(?P<AppInstanceId>.+?)/user/(?P<AppInstanceUserId>.+?)$"), ['app-instance-user']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chime:(?P<Region>[\w-]*):(?P<AccountId>.+?):app-instance/(?P<AppInstanceId>.+?)$"), ['app-instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chime:(?P<Region>[\w-]*):(?P<AccountId>.+?):media-insights-pipeline-configuration/(?P<ConfigurationName>.+?)$"), ['media-insights-pipeline-configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chime:(?P<Region>[\w-]*):(?P<AccountId>.+?):media-pipeline/(?P<MediaPipelineId>.+?)$"), ['media-pipeline']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chime:(?P<Region>[\w-]*):(?P<AccountId>.+?):media-pipeline-kinesis-video-stream-pool/(?P<PoolName>.+?)$"), ['media-pipeline-kinesis-video-stream-pool']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chime:(?P<Region>[\w-]*):(?P<AccountId>.+?):sma/(?P<SipMediaApplicationId>.+?)$"), ['sip-media-application']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chime:(?P<Region>[\w-]*):(?P<AccountId>.+?):vc/(?P<VoiceConnectorId>.+?)$"), ['voice-connector']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chime:(?P<Region>[\w-]*):(?P<AccountId>.+?):voice-profile/(?P<VoiceProfileId>.+?)$"), ['voice-profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):chime:(?P<Region>[\w-]*):(?P<AccountId>.+?):voice-profile-domain/(?P<VoiceProfileDomainId>.+?)$"), ['voice-profile-domain']),
    ],
    'cleanrooms': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms:(?P<Region>[\w-]*):(?P<Account>\d{12}):membership/(?P<MembershipId>.+?)/analysistemplate/(?P<AnalysisTemplateId>.+?)$"), ['analysistemplate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms:(?P<Region>[\w-]*):(?P<Account>\d{12}):membership/(?P<MembershipId>.+?)/configuredaudiencemodelassociation/(?P<ConfiguredAudienceModelAssociationId>.+?)$"), ['configuredaudiencemodelassociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms:(?P<Region>[\w-]*):(?P<Account>\d{12}):membership/(?P<MembershipId>.+?)/configuredtableassociation/(?P<ConfiguredTableAssociationId>.+?)$"), ['configuredtableassociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms:(?P<Region>[\w-]*):(?P<Account>\d{12}):membership/(?P<MembershipId>.+?)/idmappingtable/(?P<IdMappingTableId>.+?)$"), ['idmappingtable']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms:(?P<Region>[\w-]*):(?P<Account>\d{12}):membership/(?P<MembershipId>.+?)/idnamespaceassociation/(?P<IdNamespaceAssociationId>.+?)$"), ['idnamespaceassociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms:(?P<Region>[\w-]*):(?P<Account>\d{12}):membership/(?P<MembershipId>.+?)/privacybudgettemplate/(?P<PrivacyBudgetTemplateId>.+?)$"), ['privacybudgettemplate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms:(?P<Region>[\w-]*):(?P<Account>\d{12}):collaboration/(?P<CollaborationId>.+?)$"), ['collaboration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms:(?P<Region>[\w-]*):(?P<Account>\d{12}):configuredtable/(?P<ConfiguredTableId>.+?)$"), ['configuredtable']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms:(?P<Region>[\w-]*):(?P<Account>\d{12}):membership/(?P<MembershipId>.+?)$"), ['membership']),
    ],
    'cleanrooms-ml': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms-ml:(?P<Region>[\w-]*):(?P<Account>\d{12}):membership/(?P<MembershipId>.+?)/configured-model-algorithm-association/(?P<ResourceId>.+?)$"), ['ConfiguredModelAlgorithmAssociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms-ml:(?P<Region>[\w-]*):(?P<Account>\d{12}):membership/(?P<MembershipId>.+?)/ml-input-channel/(?P<ResourceId>.+?)$"), ['MLInputChannel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms-ml:(?P<Region>[\w-]*):(?P<Account>\d{12}):membership/(?P<MembershipId>.+?)/trained-model/(?P<ResourceId>.+?)$"), ['TrainedModel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms-ml:(?P<Region>[\w-]*):(?P<Account>\d{12}):membership/(?P<MembershipId>.+?)/trained-model-inference-job/(?P<ResourceId>.+?)$"), ['TrainedModelInferenceJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms-ml:(?P<Region>[\w-]*):(?P<Account>\d{12}):audience-generation-job/(?P<ResourceId>.+?)$"), ['audiencegenerationjob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms-ml:(?P<Region>[\w-]*):(?P<Account>\d{12}):audience-model/(?P<ResourceId>.+?)$"), ['audiencemodel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms-ml:(?P<Region>[\w-]*):(?P<Account>\d{12}):configured-audience-model/(?P<ResourceId>.+?)$"), ['configuredaudiencemodel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms-ml:(?P<Region>[\w-]*):(?P<Account>\d{12}):configured-model-algorithm/(?P<ResourceId>.+?)$"), ['ConfiguredModelAlgorithm']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cleanrooms-ml:(?P<Region>[\w-]*):(?P<Account>\d{12}):training-dataset/(?P<ResourceId>.+?)$"), ['trainingdataset']),
    ],
    'cloud9': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloud9:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment:(?P<ResourceId>.+?)$"), ['environment']),
    ],
    'clouddirectory': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):clouddirectory:(?P<Region>[\w-]*):(?P<Account>\d{12}):directory/(?P<DirectoryId>.+?)/schema/(?P<SchemaName>.+?)/(?P<Version>.+?)$"), ['appliedSchema']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):clouddirectory:(?P<Region>[\w-]*):(?P<Account>\d{12}):schema/published/(?P<SchemaName>.+?)/(?P<Version>.+?)$"), ['publishedSchema']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):clouddirectory:(?P<Region>[\w-]*):(?P<Account>\d{12}):schema/development/(?P<SchemaName>.+?)$"), ['developmentSchema']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):clouddirectory:(?P<Region>[\w-]*):(?P<Account>\d{12}):directory/(?P<DirectoryId>.+?)$"), ['directory']),
    ],
    'cloudformation': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudformation:(?P<Region>[\w-]*):(?P<Account>\d{12}):changeSet/(?P<ChangeSetName>.+?)/(?P<Id>.+?)$"), ['changeset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudformation:(?P<Region>[\w-]*):(?P<Account>\d{12}):stack/(?P<StackName>.+?)/(?P<Id>.+?)$"), ['stack']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudformation:(?P<Region>[\w-]*):(?P<Account>\d{12}):stackset/(?P<StackSetName>.+?):(?P<Id>.+?)$"), ['stackset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudformation:(?P<Region>[\w-]*):(?P<Account>\d{12}):type/hook/(?P<Type>.+?)$"), ['typeHook']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudformation:(?P<Region>[\w-]*):(?P<Account>\d{12}):type/resource/(?P<Type>.+?)$"), ['type']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudformation:(?P<Region>[\w-]*):(?P<Account>\d{12}):generatedTemplate/(?P<Id>.+?)$"), ['generatedtemplate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudformation:(?P<Region>[\w-]*):(?P<Account>\d{12}):resourceScan/(?P<Id>.+?)$"), ['resourcescan']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudformation:(?P<Region>[\w-]*):(?P<Account>\d{12}):stackset-target/(?P<StackSetTarget>.+?)$"), ['stackset-target']),
    ],
    'cloudfront': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):anycast-ip-list/(?P<Id>.+?)$"), ['anycast-ip-list']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):cache-policy/(?P<Id>.+?)$"), ['cache-policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):connection-function/(?P<Id>.+?)$"), ['connection-function']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):connection-group/(?P<Id>.+?)$"), ['connection-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):continuous-deployment-policy/(?P<Id>.+?)$"), ['continuous-deployment-policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):distribution/(?P<DistributionId>.+?)$"), ['distribution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):distribution-tenant/(?P<Id>.+?)$"), ['distribution-tenant']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):field-level-encryption-config/(?P<Id>.+?)$"), ['field-level-encryption-config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):field-level-encryption-profile/(?P<Id>.+?)$"), ['field-level-encryption-profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):function/(?P<Name>.+?)$"), ['function']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):key-value-store/(?P<Name>.+?)$"), ['key-value-store']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):key-value-store/(?P<ResourceId>.+?)$"), ['key-value-store']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):origin-access-control/(?P<Id>.+?)$"), ['origin-access-control']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):origin-access-identity/(?P<Id>.+?)$"), ['origin-access-identity']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):origin-request-policy/(?P<Id>.+?)$"), ['origin-request-policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):realtime-log-config/(?P<Name>.+?)$"), ['realtime-log-config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):response-headers-policy/(?P<Id>.+?)$"), ['response-headers-policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):streaming-distribution/(?P<DistributionId>.+?)$"), ['streaming-distribution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):trust-store/(?P<Id>.+?)$"), ['trust-store']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudfront::(?P<Account>\d{12}):vpcorigin/(?P<Id>.+?)$"), ['vpcorigin']),
    ],
    'cloudhsm': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudhsm:(?P<Region>[\w-]*):(?P<Account>\d{12}):backup/(?P<CloudHsmBackupInstanceName>.+?)$"), ['backup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudhsm:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster/(?P<CloudHsmClusterInstanceName>.+?)$"), ['cluster']),
    ],
    'cloudsearch': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudsearch:(?P<Region>[\w-]*):(?P<Account>\d{12}):domain/(?P<DomainName>.+?)$"), ['domain']),
    ],
    'cloudshell': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudshell:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment/(?P<EnvironmentId>.+?)$"), ['Environment']),
    ],
    'cloudtrail': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudtrail:(?P<Region>[\w-]*):(?P<Account>\d{12}):channel/(?P<ChannelId>.+?)$"), ['channel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudtrail:(?P<Region>[\w-]*):(?P<Account>\d{12}):dashboard/(?P<DashboardName>.+?)$"), ['dashboard']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudtrail:(?P<Region>[\w-]*):(?P<Account>\d{12}):eventdatastore/(?P<EventDataStoreId>.+?)$"), ['eventdatastore']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudtrail:(?P<Region>[\w-]*):(?P<Account>\d{12}):trail/(?P<TrailName>.+?)$"), ['trail']),
    ],
    'cloudwatch': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudwatch::(?P<Account>\d{12}):dashboard/(?P<DashboardName>.+?)$"), ['dashboard']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudwatch:(?P<Region>[\w-]*):(?P<Account>\d{12}):service/(?P<ServiceName>.+?)-(?P<UniqueAttributesHex>.+?)$"), ['service']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudwatch:(?P<Region>[\w-]*):(?P<Account>\d{12}):alarm:(?P<AlarmName>.+?)$"), ['alarm']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudwatch:(?P<Region>[\w-]*):(?P<Account>\d{12}):insight-rule/(?P<InsightRuleName>.+?)$"), ['insight-rule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudwatch:(?P<Region>[\w-]*):(?P<Account>\d{12}):metric-stream/(?P<MetricStreamName>.+?)$"), ['metric-stream']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cloudwatch:(?P<Region>[\w-]*):(?P<Account>\d{12}):slo/(?P<SloName>.+?)$"), ['slo']),
    ],
    'codeartifact': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codeartifact:(?P<Region>[\w-]*):(?P<Account>\d{12}):package/(?P<DomainName>.+?)/(?P<RepositoryName>.+?)/(?P<PackageFormat>.+?)/(?P<PackageNamespace>.+?)/(?P<PackageName>.+?)$"), ['package']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codeartifact:(?P<Region>[\w-]*):(?P<Account>\d{12}):package-group/(?P<DomainName>.+?)(?P<EncodedPackageGroupPattern>.+?)$"), ['package-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codeartifact:(?P<Region>[\w-]*):(?P<Account>\d{12}):repository/(?P<DomainName>.+?)/(?P<RepositoryName>.+?)$"), ['repository']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codeartifact:(?P<Region>[\w-]*):(?P<Account>\d{12}):domain/(?P<DomainName>.+?)$"), ['domain']),
    ],
    'codebuild': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codebuild:(?P<Region>[\w-]*):(?P<Account>\d{12}):fleet/(?P<FleetName>.+?):(?P<FleetId>.+?)$"), ['fleet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codebuild:(?P<Region>[\w-]*):(?P<Account>\d{12}):report/(?P<ReportGroupName>.+?):(?P<ReportId>.+?)$"), ['report']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codebuild:(?P<Region>[\w-]*):(?P<Account>\d{12}):build/(?P<BuildId>.+?)$"), ['build']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codebuild:(?P<Region>[\w-]*):(?P<Account>\d{12}):build-batch/(?P<BuildBatchId>.+?)$"), ['build-batch']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codebuild:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ProjectName>.+?)$"), ['project']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codebuild:(?P<Region>[\w-]*):(?P<Account>\d{12}):report-group/(?P<ReportGroupName>.+?)$"), ['report-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codebuild:(?P<Region>[\w-]*):(?P<Account>\d{12}):sandbox/(?P<SandboxId>.+?)$"), ['sandbox']),
    ],
    'codecatalyst': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codecatalyst:::space/(?P<SpaceId>.+?)/project/(?P<ProjectId>.+?)$"), ['project']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codecatalyst:::space/(?P<SpaceId>.+?)$"), ['space']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codecatalyst:(?P<Region>[\w-]*):(?P<Account>\d{12}):/connections/(?P<ConnectionId>.+?)$"), ['connections']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codecatalyst:(?P<Region>[\w-]*):(?P<Account>\d{12}):/identity-center-applications/(?P<IdentityCenterApplicationId>.+?)$"), ['identity-center-applications']),
    ],
    'codecommit': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codecommit:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<RepositoryName>.+?)$"), ['repository']),
    ],
    'codeconnections': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codeconnections:(?P<Region>[\w-]*):(?P<Account>\d{12}):connection/(?P<ConnectionId>.+?)$"), ['Connection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codeconnections:(?P<Region>[\w-]*):(?P<Account>\d{12}):host/(?P<HostId>.+?)$"), ['Host']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codeconnections:(?P<Region>[\w-]*):(?P<Account>\d{12}):repository-link/(?P<RepositoryLinkId>.+?)$"), ['RepositoryLink']),
    ],
    'codedeploy': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codedeploy:(?P<Region>[\w-]*):(?P<Account>\d{12}):deploymentgroup:(?P<ApplicationName>.+?)/(?P<DeploymentGroupName>.+?)$"), ['deploymentgroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codedeploy:(?P<Region>[\w-]*):(?P<Account>\d{12}):application:(?P<ApplicationName>.+?)$"), ['application']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codedeploy:(?P<Region>[\w-]*):(?P<Account>\d{12}):deploymentconfig:(?P<DeploymentConfigurationName>.+?)$"), ['deploymentconfig']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codedeploy:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance:(?P<InstanceName>.+?)$"), ['instance']),
    ],
    'codeguru-profiler': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codeguru-profiler:(?P<Region>[\w-]*):(?P<Account>\d{12}):profilingGroup/(?P<ProfilingGroupName>.+?)$"), ['ProfilingGroup']),
    ],
    'codeguru-reviewer': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codeguru-reviewer:(?P<Region>[\w-]*):(?P<Account>\d{12}):association:(?P<ResourceId>.+?):codereview:(?P<CodeReviewId>.+?)$"), ['codereview']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codeguru-reviewer:(?P<Region>[\w-]*):(?P<Account>\d{12}):association:(?P<ResourceId>.+?)$"), ['association']),
    ],
    'codeguru-security': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codeguru-security:(?P<Region>[\w-]*):(?P<Account>\d{12}):scans/(?P<ScanName>.+?)$"), ['ScanName']),
    ],
    'codepipeline': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codepipeline:(?P<Region>[\w-]*):(?P<Account>\d{12}):actiontype:(?P<Owner>.+?)/(?P<Category>.+?)/(?P<Provider>.+?)/(?P<Version>.+?)$"), ['actiontype']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codepipeline:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<PipelineName>.+?)/(?P<StageName>.+?)/(?P<ActionName>.+?)$"), ['action']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codepipeline:(?P<Region>[\w-]*):(?P<Account>\d{12}):webhook:(?P<WebhookName>.+?)$"), ['webhook']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codepipeline:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<PipelineName>.+?)/(?P<StageName>.+?)$"), ['stage']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codepipeline:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<PipelineName>.+?)$"), ['pipeline']),
    ],
    'codestar': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codestar:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ProjectId>.+?)$"), ['project']),
    ],
    'codestar-connections': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codestar-connections:(?P<Region>[\w-]*):(?P<Account>\d{12}):connection/(?P<ConnectionId>.+?)$"), ['Connection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codestar-connections:(?P<Region>[\w-]*):(?P<Account>\d{12}):host/(?P<HostId>.+?)$"), ['Host']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codestar-connections:(?P<Region>[\w-]*):(?P<Account>\d{12}):repository-link/(?P<RepositoryLinkId>.+?)$"), ['RepositoryLink']),
    ],
    'codestar-notifications': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codestar-notifications:(?P<Region>[\w-]*):(?P<Account>\d{12}):notificationrule/(?P<NotificationRuleId>.+?)$"), ['notificationrule']),
    ],
    'codewhisperer': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codewhisperer:(?P<Region>[\w-]*):(?P<Account>\d{12}):customization/(?P<Identifier>.+?)$"), ['customization']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):codewhisperer:(?P<Region>[\w-]*):(?P<Account>\d{12}):profile/(?P<Identifier>.+?)$"), ['profile']),
    ],
    'cognito-identity': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cognito-identity:(?P<Region>[\w-]*):(?P<Account>\d{12}):identitypool/(?P<IdentityPoolId>.+?)$"), ['identitypool']),
    ],
    'cognito-idp': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cognito-idp:(?P<Region>[\w-]*):(?P<Account>\d{12}):userpool/(?P<UserPoolId>.+?)$"), ['userpool']),
    ],
    'cognito-sync': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cognito-sync:(?P<Region>[\w-]*):(?P<Account>\d{12}):identitypool/(?P<IdentityPoolId>.+?)/identity/(?P<IdentityId>.+?)/dataset/(?P<DatasetName>.+?)$"), ['dataset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cognito-sync:(?P<Region>[\w-]*):(?P<Account>\d{12}):identitypool/(?P<IdentityPoolId>.+?)/identity/(?P<IdentityId>.+?)$"), ['identity']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cognito-sync:(?P<Region>[\w-]*):(?P<Account>\d{12}):identitypool/(?P<IdentityPoolId>.+?)$"), ['identitypool']),
    ],
    'comprehend': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):flywheel/(?P<FlywheelName>.+?)/dataset/(?P<DatasetName>.+?)$"), ['flywheel-dataset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):document-classification-job/(?P<JobId>.+?)$"), ['document-classification-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):document-classifier/(?P<DocumentClassifierName>.+?)$"), ['document-classifier']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):document-classifier-endpoint/(?P<DocumentClassifierEndpointName>.+?)$"), ['document-classifier-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):dominant-language-detection-job/(?P<JobId>.+?)$"), ['dominant-language-detection-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):entities-detection-job/(?P<JobId>.+?)$"), ['entities-detection-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):entity-recognizer/(?P<EntityRecognizerName>.+?)$"), ['entity-recognizer']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):entity-recognizer-endpoint/(?P<EntityRecognizerEndpointName>.+?)$"), ['entity-recognizer-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):events-detection-job/(?P<JobId>.+?)$"), ['events-detection-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):flywheel/(?P<FlywheelName>.+?)$"), ['flywheel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):key-phrases-detection-job/(?P<JobId>.+?)$"), ['key-phrases-detection-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):pii-entities-detection-job/(?P<JobId>.+?)$"), ['pii-entities-detection-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):sentiment-detection-job/(?P<JobId>.+?)$"), ['sentiment-detection-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):targeted-sentiment-detection-job/(?P<JobId>.+?)$"), ['targeted-sentiment-detection-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):comprehend:(?P<Region>[\w-]*):(?P<Account>\d{12}):topics-detection-job/(?P<JobId>.+?)$"), ['topics-detection-job']),
    ],
    'compute-optimizer': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):compute-optimizer::(?P<Account>\d{12}):automation-rule/(?P<RuleId>.+?)$"), ['AutomationRule']),
    ],
    'config': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):config:(?P<Region>[\w-]*):(?P<Account>\d{12}):aggregation-authorization/(?P<AggregatorAccount>.+?)/(?P<AggregatorRegion>.+?)$"), ['AggregationAuthorization']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):config:(?P<Region>[\w-]*):(?P<Account>\d{12}):configuration-recorder/(?P<RecorderName>.+?)/(?P<RecorderId>.+?)$"), ['ConfigurationRecorder']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):config:(?P<Region>[\w-]*):(?P<Account>\d{12}):conformance-pack/(?P<ConformancePackName>.+?)/(?P<ConformancePackId>.+?)$"), ['ConformancePack']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):config:(?P<Region>[\w-]*):(?P<Account>\d{12}):stored-query/(?P<StoredQueryName>.+?)/(?P<StoredQueryId>.+?)$"), ['StoredQuery']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):config:(?P<Region>[\w-]*):(?P<Account>\d{12}):config-aggregator/(?P<AggregatorId>.+?)$"), ['ConfigurationAggregator']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):config:(?P<Region>[\w-]*):(?P<Account>\d{12}):config-rule/(?P<ConfigRuleId>.+?)$"), ['ConfigRule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):config:(?P<Region>[\w-]*):(?P<Account>\d{12}):organization-config-rule/(?P<OrganizationConfigRuleId>.+?)$"), ['OrganizationConfigRule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):config:(?P<Region>[\w-]*):(?P<Account>\d{12}):organization-conformance-pack/(?P<OrganizationConformancePackId>.+?)$"), ['OrganizationConformancePack']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):config:(?P<Region>[\w-]*):(?P<Account>\d{12}):remediation-configuration/(?P<RemediationConfigurationId>.+?)$"), ['RemediationConfiguration']),
    ],
    'connect': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):aws:view/(?P<ViewId>.+?):(?P<ViewQualifier>.+?)$"), ['qualified-aws-managed-view']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):aws:view/(?P<ViewId>.+?)$"), ['aws-managed-view']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/view/(?P<ViewId>.+?):(?P<ViewQualifier>.+?)$"), ['qualified-customer-managed-view']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/view/(?P<ViewId>.+?):(?P<ViewVersion>.+?)$"), ['customer-managed-view-version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/agent/(?P<UserId>.+?)$"), ['user']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/agent-group/(?P<HierarchyGroupId>.+?)$"), ['hierarchy-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/agent-state/(?P<AgentStatusId>.+?)$"), ['agent-status']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/authentication-profile/(?P<AuthenticationProfileId>.+?)$"), ['authentication-profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/contact/(?P<ContactId>.+?)$"), ['contact']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/contact-evaluation/(?P<EvaluationId>.+?)$"), ['contact-evaluation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/contact-flow/(?P<ContactFlowId>.+?)$"), ['contact-flow']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/data-table/(?P<DataTableId>.+?)$"), ['data-table']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/email-address/(?P<EmailAddressId>.+?)$"), ['email-address']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/evaluation-form/(?P<FormId>.+?)$"), ['evaluation-form']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/file/(?P<FileId>.+?)$"), ['attached-file']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/flow-module/(?P<ContactFlowModuleId>.+?)$"), ['contact-flow-module']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/integration-association/(?P<IntegrationAssociationId>.+?)$"), ['integration-association']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/operating-hours/(?P<HoursOfOperationId>.+?)$"), ['hours-of-operation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/phone-number/(?P<PhoneNumberId>.+?)$"), ['legacy-phone-number']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/prompt/(?P<PromptId>.+?)$"), ['prompt']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/queue/(?P<QueueId>.+?)$"), ['queue']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/routing-profile/(?P<RoutingProfileId>.+?)$"), ['routing-profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/rule/(?P<RuleId>.+?)$"), ['rule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/security-profile/(?P<SecurityProfileId>.+?)$"), ['security-profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/task-template/(?P<TaskTemplateId>.+?)$"), ['task-template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/transfer-destination/(?P<QuickConnectId>.+?)$"), ['quick-connect']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/use-case/(?P<UseCaseId>.+?)$"), ['use-case']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/view/(?P<ViewId>.+?)$"), ['customer-managed-view']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/vocabulary/(?P<VocabularyId>.+?)$"), ['vocabulary']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/workspace/(?P<WorkspaceId>.+?)$"), ['workspace']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)$"), ['instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):phone-number/(?P<PhoneNumberId>.+?)$"), ['phone-number']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect:(?P<Region>[\w-]*):(?P<Account>\d{12}):traffic-distribution-group/(?P<TrafficDistributionGroupId>.+?)$"), ['traffic-distribution-group']),
    ],
    'connect-campaigns': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):connect-campaigns:(?P<Region>[\w-]*):(?P<Account>\d{12}):campaign/(?P<CampaignId>.+?)$"), ['campaign']),
    ],
    'consoleapp': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):consoleapp::(?P<Account>\d{12}):device/(?P<DeviceId>.+?)/identity/(?P<IdentityId>.+?)$"), ['DeviceIdentity']),
    ],
    'controlcatalog': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):controlcatalog:::common-control/(?P<CommonControlId>.+?)$"), ['common-control']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):controlcatalog:::control/(?P<ControlId>.+?)$"), ['control']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):controlcatalog:::domain/(?P<DomainId>.+?)$"), ['domain']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):controlcatalog:::objective/(?P<ObjectiveId>.+?)$"), ['objective']),
    ],
    'controltower': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):controltower:(?P<Region>[\w-]*)::baseline/(?P<BaselineId>.+?)$"), ['Baseline']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):controltower:(?P<Region>[\w-]*):(?P<Account>\d{12}):enabledbaseline/(?P<EnabledBaselineId>.+?)$"), ['EnabledBaseline']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):controltower:(?P<Region>[\w-]*):(?P<Account>\d{12}):enabledcontrol/(?P<EnabledControlId>.+?)$"), ['EnabledControl']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):controltower:(?P<Region>[\w-]*):(?P<Account>\d{12}):landingzone/(?P<LandingZoneId>.+?)$"), ['LandingZone']),
    ],
    'cur': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):cur:(?P<Region>[\w-]*):(?P<Account>\d{12}):definition/(?P<ReportName>.+?)$"), ['cur']),
    ],
    'databrew': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):databrew:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataset/(?P<ResourceId>.+?)$"), ['Dataset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):databrew:(?P<Region>[\w-]*):(?P<Account>\d{12}):job/(?P<ResourceId>.+?)$"), ['Job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):databrew:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ResourceId>.+?)$"), ['Project']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):databrew:(?P<Region>[\w-]*):(?P<Account>\d{12}):recipe/(?P<ResourceId>.+?)$"), ['Recipe']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):databrew:(?P<Region>[\w-]*):(?P<Account>\d{12}):ruleset/(?P<ResourceId>.+?)$"), ['Ruleset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):databrew:(?P<Region>[\w-]*):(?P<Account>\d{12}):schedule/(?P<ResourceId>.+?)$"), ['Schedule']),
    ],
    'dataexchange': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dataexchange:(?P<Region>[\w-]*)::data-sets/(?P<DataSetId>.+?)/revisions/(?P<RevisionId>.+?)/assets/(?P<AssetId>.+?)$"), ['entitled-assets']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dataexchange:(?P<Region>[\w-]*)::data-sets/(?P<DataSetId>.+?)/revisions/(?P<RevisionId>.+?)$"), ['entitled-revisions']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dataexchange:(?P<Region>[\w-]*)::data-sets/(?P<DataSetId>.+?)$"), ['entitled-data-sets']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dataexchange:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-sets/(?P<DataSetId>.+?)/revisions/(?P<RevisionId>.+?)/assets/(?P<AssetId>.+?)$"), ['assets']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dataexchange:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-sets/(?P<DataSetId>.+?)/revisions/(?P<RevisionId>.+?)$"), ['revisions']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dataexchange:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-grants/(?P<DataGrantId>.+?)$"), ['data-grants']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dataexchange:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-sets/(?P<DataSetId>.+?)$"), ['data-sets']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dataexchange:(?P<Region>[\w-]*):(?P<Account>\d{12}):event-actions/(?P<EventActionId>.+?)$"), ['event-actions']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dataexchange:(?P<Region>[\w-]*):(?P<Account>\d{12}):jobs/(?P<JobId>.+?)$"), ['jobs']),
    ],
    'datapipeline': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):datapipeline:(?P<Region>[\w-]*):(?P<Account>\d{12}):pipeline/(?P<PipelineId>.+?)$"), ['pipeline']),
    ],
    'datasync': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):datasync:(?P<Region>[\w-]*):(?P<AccountId>.+?):system/(?P<StorageSystemId>.+?)/job/(?P<DiscoveryJobId>.+?)$"), ['discoveryjob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):datasync:(?P<Region>[\w-]*):(?P<AccountId>.+?):task/(?P<TaskId>.+?)/execution/(?P<ExecutionId>.+?)$"), ['taskexecution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):datasync:(?P<Region>[\w-]*):(?P<AccountId>.+?):agent/(?P<AgentId>.+?)$"), ['agent']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):datasync:(?P<Region>[\w-]*):(?P<AccountId>.+?):location/(?P<LocationId>.+?)$"), ['location']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):datasync:(?P<Region>[\w-]*):(?P<AccountId>.+?):system/(?P<StorageSystemId>.+?)$"), ['storagesystem']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):datasync:(?P<Region>[\w-]*):(?P<AccountId>.+?):task/(?P<TaskId>.+?)$"), ['task']),
    ],
    'datazone': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):datazone:(?P<Region>[\w-]*):(?P<Account>\d{12}):domain/(?P<DomainId>.+?)$"), ['domain']),
    ],
    'dax': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dax:(?P<Region>[\w-]*):(?P<Account>\d{12}):cache/(?P<ClusterName>.+?)$"), ['application']),
    ],
    'deadline': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):deadline:(?P<Region>[\w-]*):(?P<Account>\d{12}):farm/(?P<FarmId>.+?)/fleet/(?P<FleetId>.+?)/worker/(?P<WorkerId>.+?)$"), ['worker']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):deadline:(?P<Region>[\w-]*):(?P<Account>\d{12}):farm/(?P<FarmId>.+?)/queue/(?P<QueueId>.+?)/job/(?P<JobId>.+?)$"), ['job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):deadline:(?P<Region>[\w-]*):(?P<Account>\d{12}):farm/(?P<FarmId>.+?)/budget/(?P<BudgetId>.+?)$"), ['budget']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):deadline:(?P<Region>[\w-]*):(?P<Account>\d{12}):farm/(?P<FarmId>.+?)/fleet/(?P<FleetId>.+?)$"), ['fleet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):deadline:(?P<Region>[\w-]*):(?P<Account>\d{12}):farm/(?P<FarmId>.+?)/queue/(?P<QueueId>.+?)$"), ['queue']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):deadline:(?P<Region>[\w-]*):(?P<Account>\d{12}):farm/(?P<FarmId>.+?)$"), ['farm']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):deadline:(?P<Region>[\w-]*):(?P<Account>\d{12}):license-endpoint/(?P<LicenseEndpointId>.+?)$"), ['license-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):deadline:(?P<Region>[\w-]*):(?P<Account>\d{12}):monitor/(?P<MonitorId>.+?)$"), ['monitor']),
    ],
    'detective': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):detective:(?P<Region>[\w-]*):(?P<Account>\d{12}):graph:(?P<ResourceId>.+?)$"), ['Graph']),
    ],
    'devicefarm': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*)::device:(?P<ResourceId>.+?)$"), ['device']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*)::deviceinstance:(?P<ResourceId>.+?)$"), ['deviceinstance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):artifact:(?P<ResourceId>.+?)$"), ['artifact']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):devicepool:(?P<ResourceId>.+?)$"), ['devicepool']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):instanceprofile:(?P<ResourceId>.+?)$"), ['instanceprofile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):job:(?P<ResourceId>.+?)$"), ['job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):networkprofile:(?P<ResourceId>.+?)$"), ['networkprofile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):project:(?P<ResourceId>.+?)$"), ['project']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):run:(?P<ResourceId>.+?)$"), ['run']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):sample:(?P<ResourceId>.+?)$"), ['sample']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):session:(?P<ResourceId>.+?)$"), ['session']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):suite:(?P<ResourceId>.+?)$"), ['suite']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):test:(?P<ResourceId>.+?)$"), ['test']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):testgrid-project:(?P<ResourceId>.+?)$"), ['testgrid-project']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):testgrid-session:(?P<ResourceId>.+?)$"), ['testgrid-session']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):upload:(?P<ResourceId>.+?)$"), ['upload']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):devicefarm:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpceconfiguration:(?P<ResourceId>.+?)$"), ['vpceconfiguration']),
    ],
    'directconnect': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):directconnect::(?P<Account>\d{12}):dx-gateway/(?P<DirectConnectGatewayId>.+?)$"), ['dx-gateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):directconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):dxcon/(?P<ConnectionId>.+?)$"), ['dxcon']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):directconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):dxlag/(?P<LagId>.+?)$"), ['dxlag']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):directconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):dxvif/(?P<VirtualInterfaceId>.+?)$"), ['dxvif']),
    ],
    'dlm': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dlm:(?P<Region>[\w-]*):(?P<Account>\d{12}):policy/(?P<ResourceName>.+?)$"), ['policy']),
    ],
    'dms': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dms:(?P<Region>[\w-]*):(?P<Account>\d{12}):assessment-run:(?P<AssessmentRunId>.+?)$"), ['ReplicationTaskAssessmentRun']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dms:(?P<Region>[\w-]*):(?P<Account>\d{12}):cert:(?P<CertificateId>.+?)$"), ['Certificate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dms:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-migration:(?P<DataMigrationId>.+?)$"), ['DataMigration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dms:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-provider:(?P<DataProviderId>.+?)$"), ['DataProvider']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dms:(?P<Region>[\w-]*):(?P<Account>\d{12}):endpoint:(?P<EndpointId>.+?)$"), ['Endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dms:(?P<Region>[\w-]*):(?P<Account>\d{12}):es:(?P<EventSubscriptionId>.+?)$"), ['EventSubscription']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dms:(?P<Region>[\w-]*):(?P<Account>\d{12}):individual-assessment:(?P<IndividualAssessmentId>.+?)$"), ['ReplicationTaskIndividualAssessment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dms:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance-profile:(?P<InstanceProfileId>.+?)$"), ['InstanceProfile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dms:(?P<Region>[\w-]*):(?P<Account>\d{12}):migration-project:(?P<MigrationProjectId>.+?)$"), ['MigrationProject']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dms:(?P<Region>[\w-]*):(?P<Account>\d{12}):rep:(?P<ReplicationInstanceId>.+?)$"), ['ReplicationInstance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dms:(?P<Region>[\w-]*):(?P<Account>\d{12}):replication-config:(?P<ReplicationConfigId>.+?)$"), ['ReplicationConfig']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dms:(?P<Region>[\w-]*):(?P<Account>\d{12}):subgrp:(?P<SubnetGroupName>.+?)$"), ['ReplicationSubnetGroup', 'subgrp']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dms:(?P<Region>[\w-]*):(?P<Account>\d{12}):task:(?P<TaskId>.+?)$"), ['ReplicationTask']),
    ],
    'docdb-elastic': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):docdb-elastic:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster/(?P<ResourceId>.+?)$"), ['cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):docdb-elastic:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster-snapshot/(?P<ResourceId>.+?)$"), ['cluster-snapshot']),
    ],
    'drs': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):drs:(?P<Region>[\w-]*):(?P<Account>\d{12}):job/(?P<JobID>.+?)$"), ['JobResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):drs:(?P<Region>[\w-]*):(?P<Account>\d{12}):launch-configuration-template/(?P<LaunchConfigurationTemplateID>.+?)$"), ['LaunchConfigurationTemplateResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):drs:(?P<Region>[\w-]*):(?P<Account>\d{12}):recovery-instance/(?P<RecoveryInstanceID>.+?)$"), ['RecoveryInstanceResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):drs:(?P<Region>[\w-]*):(?P<Account>\d{12}):replication-configuration-template/(?P<ReplicationConfigurationTemplateID>.+?)$"), ['ReplicationConfigurationTemplateResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):drs:(?P<Region>[\w-]*):(?P<Account>\d{12}):source-network/(?P<SourceNetworkID>.+?)$"), ['SourceNetworkResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):drs:(?P<Region>[\w-]*):(?P<Account>\d{12}):source-server/(?P<SourceServerID>.+?)$"), ['SourceServerResource']),
    ],
    'ds': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ds:(?P<Region>[\w-]*):(?P<Account>\d{12}):directory/(?P<DirectoryId>.+?)$"), ['directory']),
    ],
    'dsql': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dsql:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster/(?P<Identifier>.+?)$"), ['Cluster']),
    ],
    'dynamodb': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dynamodb::(?P<Account>\d{12}):global-table/(?P<GlobalTableName>.+?)$"), ['global-table']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dynamodb:(?P<Region>[\w-]*):(?P<Account>\d{12}):table/(?P<TableName>.+?)/backup/(?P<BackupName>.+?)$"), ['backup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dynamodb:(?P<Region>[\w-]*):(?P<Account>\d{12}):table/(?P<TableName>.+?)/export/(?P<ExportName>.+?)$"), ['export']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dynamodb:(?P<Region>[\w-]*):(?P<Account>\d{12}):table/(?P<TableName>.+?)/import/(?P<ImportName>.+?)$"), ['import']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dynamodb:(?P<Region>[\w-]*):(?P<Account>\d{12}):table/(?P<TableName>.+?)/index/(?P<IndexName>.+?)$"), ['index']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dynamodb:(?P<Region>[\w-]*):(?P<Account>\d{12}):table/(?P<TableName>.+?)/stream/(?P<StreamLabel>.+?)$"), ['stream']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):dynamodb:(?P<Region>[\w-]*):(?P<Account>\d{12}):table/(?P<TableName>.+?)$"), ['table']),
    ],
    'ec2': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2::(?P<Account>\d{12}):ipam/(?P<IpamId>.+?)$"), ['ipam']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2::(?P<Account>\d{12}):ipam-external-resource-verification-token/(?P<IpamExternalResourceVerificationTokenId>.+?)$"), ['ipam-external-resource-verification-token']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2::(?P<Account>\d{12}):ipam-policy/(?P<IpamPolicyId>.+?)$"), ['ipam-policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2::(?P<Account>\d{12}):ipam-pool/(?P<IpamPoolId>.+?)$"), ['ipam-pool']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2::(?P<Account>\d{12}):ipam-prefix-list-resolver/(?P<IpamPrefixListResolverId>.+?)$"), ['ipam-prefix-list-resolver']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2::(?P<Account>\d{12}):ipam-prefix-list-resolver-target/(?P<IpamPrefixListResolverTargetId>.+?)$"), ['ipam-prefix-list-resolver-target']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2::(?P<Account>\d{12}):ipam-resource-discovery/(?P<IpamResourceDiscoveryId>.+?)$"), ['ipam-resource-discovery']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2::(?P<Account>\d{12}):ipam-resource-discovery-association/(?P<IpamResourceDiscoveryAssociationId>.+?)$"), ['ipam-resource-discovery-association']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2::(?P<Account>\d{12}):ipam-scope/(?P<IpamScopeId>.+?)$"), ['ipam-scope']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):capacity-block/(?P<CapacityBlockId>.+?)$"), ['capacity-block']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):capacity-manager-data-export/(?P<CapacityManagerDataExportId>.+?)$"), ['capacity-manager-data-export']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):capacity-reservation/(?P<CapacityReservationId>.+?)$"), ['capacity-reservation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):capacity-reservation-fleet/(?P<CapacityReservationFleetId>.+?)$"), ['capacity-reservation-fleet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):carrier-gateway/(?P<CarrierGatewayId>.+?)$"), ['carrier-gateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):client-vpn-endpoint/(?P<ClientVpnEndpointId>.+?)$"), ['client-vpn-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):coip-pool/(?P<Ipv4PoolCoipId>.+?)$"), ['coip-pool']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):customer-gateway/(?P<CustomerGatewayId>.+?)$"), ['customer-gateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):declarative-policies-report/(?P<DeclarativePoliciesReportId>.+?)$"), ['declarative-policies-report']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):dedicated-host/(?P<DedicatedHostId>.+?)$"), ['dedicated-host']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):dhcp-options/(?P<DhcpOptionsId>.+?)$"), ['dhcp-options']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):egress-only-internet-gateway/(?P<EgressOnlyInternetGatewayId>.+?)$"), ['egress-only-internet-gateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):elastic-gpu/(?P<ElasticGpuId>.+?)$"), ['elastic-gpu']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):elastic-ip/(?P<AllocationId>.+?)$"), ['elastic-ip']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):export-image-task/(?P<ExportImageTaskId>.+?)$"), ['export-image-task']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):export-instance-task/(?P<ExportTaskId>.+?)$"), ['export-instance-task']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):fleet/(?P<FleetId>.+?)$"), ['fleet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):fpga-image/(?P<FpgaImageId>.+?)$"), ['fpga-image']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):host-reservation/(?P<HostReservationId>.+?)$"), ['host-reservation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):image/(?P<ImageId>.+?)$"), ['image']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):image-usage-report/(?P<ImageUsageReportId>.+?)$"), ['image-usage-report']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):import-image-task/(?P<ImportImageTaskId>.+?)$"), ['import-image-task']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):import-snapshot-task/(?P<ImportSnapshotTaskId>.+?)$"), ['import-snapshot-task']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)$"), ['instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance-connect-endpoint/(?P<InstanceConnectEndpointId>.+?)$"), ['instance-connect-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance-event-window/(?P<InstanceEventWindowId>.+?)$"), ['instance-event-window']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):internet-gateway/(?P<InternetGatewayId>.+?)$"), ['internet-gateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):ipv4pool-ec2/(?P<Ipv4PoolEc2Id>.+?)$"), ['ipv4pool-ec2']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):ipv6pool-ec2/(?P<Ipv6PoolEc2Id>.+?)$"), ['ipv6pool-ec2']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):key-pair/(?P<KeyPairName>.+?)$"), ['key-pair']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):launch-template/(?P<LaunchTemplateId>.+?)$"), ['launch-template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):local-gateway/(?P<LocalGatewayId>.+?)$"), ['local-gateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):local-gateway-route-table/(?P<LocalGatewayRoutetableId>.+?)$"), ['local-gateway-route-table']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):local-gateway-route-table-virtual-interface-group-association/(?P<LocalGatewayRouteTableVirtualInterfaceGroupAssociationId>.+?)$"), ['local-gateway-route-table-virtual-interface-group-association']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):local-gateway-route-table-vpc-association/(?P<LocalGatewayRouteTableVpcAssociationId>.+?)$"), ['local-gateway-route-table-vpc-association']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):local-gateway-virtual-interface/(?P<LocalGatewayVirtualInterfaceId>.+?)$"), ['local-gateway-virtual-interface']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):local-gateway-virtual-interface-group/(?P<LocalGatewayVirtualInterfaceGroupId>.+?)$"), ['local-gateway-virtual-interface-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):mac-modification-task/(?P<MacModificationTaskId>.+?)$"), ['mac-modification-task']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):natgateway/(?P<NatGatewayId>.+?)$"), ['natgateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):network-acl/(?P<NaclId>.+?)$"), ['network-acl']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):network-insights-access-scope/(?P<NetworkInsightsAccessScopeId>.+?)$"), ['network-insights-access-scope']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):network-insights-access-scope-analysis/(?P<NetworkInsightsAccessScopeAnalysisId>.+?)$"), ['network-insights-access-scope-analysis']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):network-insights-analysis/(?P<NetworkInsightsAnalysisId>.+?)$"), ['network-insights-analysis']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):network-insights-path/(?P<NetworkInsightsPathId>.+?)$"), ['network-insights-path']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):network-interface/(?P<NetworkInterfaceId>.+?)$"), ['network-interface']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):outpost-lag/(?P<OutpostLagId>.+?)$"), ['outpost-lag']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):placement-group/(?P<PlacementGroupName>.+?)$"), ['placement-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):prefix-list/(?P<PrefixListId>.+?)$"), ['prefix-list']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):replace-root-volume-task/(?P<ReplaceRootVolumeTaskId>.+?)$"), ['replace-root-volume-task']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):reserved-instances/(?P<ReservationId>.+?)$"), ['reserved-instances']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):route-server/(?P<RouteServerId>.+?)$"), ['route-server']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):route-server-endpoint/(?P<RouteServerEndpointId>.+?)$"), ['route-server-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):route-server-peer/(?P<RouteServerPeerId>.+?)$"), ['route-server-peer']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):route-table/(?P<RouteTableId>.+?)$"), ['route-table']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):security-group/(?P<SecurityGroupId>.+?)$"), ['security-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):security-group-rule/(?P<SecurityGroupRuleId>.+?)$"), ['security-group-rule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):snapshot/(?P<SnapshotId>.+?)$"), ['snapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):spot-fleet-request/(?P<SpotFleetRequestId>.+?)$"), ['spot-fleet-request']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):spot-instances-request/(?P<SpotInstanceRequestId>.+?)$"), ['spot-instances-request']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):subnet/(?P<SubnetId>.+?)$"), ['subnet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):subnet-cidr-reservation/(?P<SubnetCidrReservationId>.+?)$"), ['subnet-cidr-reservation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):traffic-mirror-filter/(?P<TrafficMirrorFilterId>.+?)$"), ['traffic-mirror-filter']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):traffic-mirror-filter-rule/(?P<TrafficMirrorFilterRuleId>.+?)$"), ['traffic-mirror-filter-rule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):traffic-mirror-session/(?P<TrafficMirrorSessionId>.+?)$"), ['traffic-mirror-session']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):traffic-mirror-target/(?P<TrafficMirrorTargetId>.+?)$"), ['traffic-mirror-target']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):transit-gateway/(?P<TransitGatewayId>.+?)$"), ['transit-gateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):transit-gateway-attachment/(?P<TransitGatewayAttachmentId>.+?)$"), ['transit-gateway-attachment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):transit-gateway-connect-peer/(?P<TransitGatewayConnectPeerId>.+?)$"), ['transit-gateway-connect-peer']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):transit-gateway-metering-policy/(?P<TransitGatewayMeteringPolicyId>.+?)$"), ['transit-gateway-metering-policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):transit-gateway-multicast-domain/(?P<TransitGatewayMulticastDomainId>.+?)$"), ['transit-gateway-multicast-domain']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):transit-gateway-policy-table/(?P<TransitGatewayPolicyTableId>.+?)$"), ['transit-gateway-policy-table']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):transit-gateway-route-table/(?P<TransitGatewayRouteTableId>.+?)$"), ['transit-gateway-route-table']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):transit-gateway-route-table-announcement/(?P<TransitGatewayRouteTableAnnouncementId>.+?)$"), ['transit-gateway-route-table-announcement']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):verified-access-endpoint/(?P<VerifiedAccessEndpointId>.+?)$"), ['verified-access-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):verified-access-endpoint-target/(?P<VerifiedAccessEndpointTargetId>.+?)$"), ['verified-access-endpoint-target']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):verified-access-group/(?P<VerifiedAccessGroupId>.+?)$"), ['verified-access-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):verified-access-instance/(?P<VerifiedAccessInstanceId>.+?)$"), ['verified-access-instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):verified-access-policy/(?P<VerifiedAccessPolicyId>.+?)$"), ['verified-access-policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):verified-access-trust-provider/(?P<VerifiedAccessTrustProviderId>.+?)$"), ['verified-access-trust-provider']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):volume/(?P<VolumeId>.+?)$"), ['volume']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpc/(?P<VpcId>.+?)$"), ['vpc']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpc-block-public-access-exclusion/(?P<VpcBlockPublicAccessExclusionId>.+?)$"), ['vpc-block-public-access-exclusion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpc-encryption-control/(?P<VpcEncryptionControlId>.+?)$"), ['vpc-encryption-control']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpc-endpoint/(?P<VpcEndpointId>.+?)$"), ['vpc-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpc-endpoint-connection/(?P<VpcEndpointConnectionId>.+?)$"), ['vpc-endpoint-connection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpc-endpoint-service/(?P<VpcEndpointServiceId>.+?)$"), ['vpc-endpoint-service']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpc-endpoint-service-permission/(?P<VpcEndpointServicePermissionId>.+?)$"), ['vpc-endpoint-service-permission']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpc-flow-log/(?P<VpcFlowLogId>.+?)$"), ['vpc-flow-log']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpc-peering-connection/(?P<VpcPeeringConnectionId>.+?)$"), ['vpc-peering-connection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpn-concentrator/(?P<VpnConcentratorId>.+?)$"), ['vpn-concentrator']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpn-connection/(?P<VpnConnectionId>.+?)$"), ['vpn-connection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpn-connection-device-type/(?P<VpnConnectionDeviceTypeId>.+?)$"), ['vpn-connection-device-type']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ec2:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpn-gateway/(?P<VpnGatewayId>.+?)$"), ['vpn-gateway']),
    ],
    'ecr': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ecr:(?P<Region>[\w-]*):(?P<Account>\d{12}):repository/(?P<RepositoryName>.+?)$"), ['repository']),
    ],
    'ecr-public': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ecr-public::(?P<Account>\d{12}):registry/(?P<RegistryId>.+?)$"), ['registry']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ecr-public::(?P<Account>\d{12}):repository/(?P<RepositoryName>.+?)$"), ['repository']),
    ],
    'ecs': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ecs:(?P<Region>[\w-]*):(?P<Account>\d{12}):service-deployment/(?P<ClusterName>.+?)/(?P<ServiceName>.+?)/(?P<ServiceDeploymentId>.+?)$"), ['service-deployment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ecs:(?P<Region>[\w-]*):(?P<Account>\d{12}):service-revision/(?P<ClusterName>.+?)/(?P<ServiceName>.+?)/(?P<ServiceRevisionId>.+?)$"), ['service-revision']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ecs:(?P<Region>[\w-]*):(?P<Account>\d{12}):task-set/(?P<ClusterName>.+?)/(?P<ServiceName>.+?)/(?P<TaskSetId>.+?)$"), ['task-set']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ecs:(?P<Region>[\w-]*):(?P<Account>\d{12}):container-instance/(?P<ClusterName>.+?)/(?P<ContainerInstanceId>.+?)$"), ['container-instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ecs:(?P<Region>[\w-]*):(?P<Account>\d{12}):service/(?P<ClusterName>.+?)/(?P<ServiceName>.+?)$"), ['service']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ecs:(?P<Region>[\w-]*):(?P<Account>\d{12}):task/(?P<ClusterName>.+?)/(?P<TaskId>.+?)$"), ['task']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ecs:(?P<Region>[\w-]*):(?P<Account>\d{12}):task-definition/(?P<TaskDefinitionFamilyName>.+?):(?P<TaskDefinitionRevisionNumber>.+?)$"), ['task-definition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ecs:(?P<Region>[\w-]*):(?P<Account>\d{12}):capacity-provider/(?P<CapacityProviderName>.+?)$"), ['capacity-provider']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ecs:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster/(?P<ClusterName>.+?)$"), ['cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ecs:(?P<Region>[\w-]*):(?P<Account>\d{12}):task/(?P<TaskId>.+?)$"), ['task']),
    ],
    'eks': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks::aws:cluster-access-policy/(?P<AccessPolicyName>.+?)$"), ['access-policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):access-entry/(?P<ClusterName>.+?)/(?P<IamIdentityType>.+?)/(?P<IamIdentityAccountID>.+?)/(?P<IamIdentityName>.+?)/(?P<UUID>.+?)$"), ['access-entry']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):capability/(?P<ClusterName>.+?)/(?P<CapabilityType>.+?)/(?P<CapabilityName>.+?)/(?P<UUID>.+?)$"), ['capability']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):daemonset/(?P<ClusterName>.+?)/(?P<Namespace>.+?)/(?P<DaemonSetName>.+?)/(?P<UUID>.+?)$"), ['daemonset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):deployment/(?P<ClusterName>.+?)/(?P<Namespace>.+?)/(?P<DeploymentName>.+?)/(?P<UUID>.+?)$"), ['deployment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):endpointslice/(?P<ClusterName>.+?)/(?P<Namespace>.+?)/(?P<EndpointSliceName>.+?)/(?P<UUID>.+?)$"), ['endpointslice']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):identityproviderconfig/(?P<ClusterName>.+?)/(?P<IdentityProviderType>.+?)/(?P<IdentityProviderConfigName>.+?)/(?P<UUID>.+?)$"), ['identityproviderconfig']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):ingress/(?P<ClusterName>.+?)/(?P<Namespace>.+?)/(?P<IngressName>.+?)/(?P<UUID>.+?)$"), ['ingress']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):replicaset/(?P<ClusterName>.+?)/(?P<Namespace>.+?)/(?P<ReplicaSetName>.+?)/(?P<UUID>.+?)$"), ['replicaset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):service/(?P<ClusterName>.+?)/(?P<Namespace>.+?)/(?P<ServiceName>.+?)/(?P<UUID>.+?)$"), ['service']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):statefulset/(?P<ClusterName>.+?)/(?P<Namespace>.+?)/(?P<StatefulSetName>.+?)/(?P<UUID>.+?)$"), ['statefulset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):addon/(?P<ClusterName>.+?)/(?P<AddonName>.+?)/(?P<UUID>.+?)$"), ['addon']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):fargateprofile/(?P<ClusterName>.+?)/(?P<FargateProfileName>.+?)/(?P<UUID>.+?)$"), ['fargateprofile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):namespace/(?P<ClusterName>.+?)/(?P<NamespaceName>.+?)/(?P<UUID>.+?)$"), ['namespace']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):nodegroup/(?P<ClusterName>.+?)/(?P<NodegroupName>.+?)/(?P<UUID>.+?)$"), ['nodegroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):persistentvolume/(?P<ClusterName>.+?)/(?P<PersistentVolumeName>.+?)/(?P<UUID>.+?)$"), ['persistentvolume']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):podidentityassociation/(?P<ClusterName>.+?)/(?P<UUID>.+?)$"), ['podidentityassociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster/(?P<ClusterName>.+?)$"), ['cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):dashboard/(?P<DashboardName>.+?)$"), ['dashboard']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):eks:(?P<Region>[\w-]*):(?P<Account>\d{12}):eks-anywhere-subscription/(?P<UUID>.+?)$"), ['eks-anywhere-subscription']),
    ],
    'elastic-inference': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elastic-inference:(?P<Region>[\w-]*):(?P<Account>\d{12}):elastic-inference-accelerator/(?P<AcceleratorId>.+?)$"), ['elastic-inference']),
    ],
    'elasticache': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticache::(?P<Account>\d{12}):globalreplicationgroup:(?P<GlobalReplicationGroupId>.+?)$"), ['globalreplicationgroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticache:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster:(?P<CacheClusterId>.+?)$"), ['cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticache:(?P<Region>[\w-]*):(?P<Account>\d{12}):parametergroup:(?P<CacheParameterGroupName>.+?)$"), ['parametergroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticache:(?P<Region>[\w-]*):(?P<Account>\d{12}):replicationgroup:(?P<ReplicationGroupId>.+?)$"), ['replicationgroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticache:(?P<Region>[\w-]*):(?P<Account>\d{12}):reserved-instance:(?P<ReservedCacheNodeId>.+?)$"), ['reserved-instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticache:(?P<Region>[\w-]*):(?P<Account>\d{12}):securitygroup:(?P<CacheSecurityGroupName>.+?)$"), ['securitygroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticache:(?P<Region>[\w-]*):(?P<Account>\d{12}):serverlesscache:(?P<ServerlessCacheName>.+?)$"), ['serverlesscache']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticache:(?P<Region>[\w-]*):(?P<Account>\d{12}):serverlesscachesnapshot:(?P<ServerlessCacheSnapshotName>.+?)$"), ['serverlesscachesnapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticache:(?P<Region>[\w-]*):(?P<Account>\d{12}):snapshot:(?P<SnapshotName>.+?)$"), ['snapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticache:(?P<Region>[\w-]*):(?P<Account>\d{12}):subnetgroup:(?P<CacheSubnetGroupName>.+?)$"), ['subnetgroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticache:(?P<Region>[\w-]*):(?P<Account>\d{12}):user:(?P<UserId>.+?)$"), ['user']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticache:(?P<Region>[\w-]*):(?P<Account>\d{12}):usergroup:(?P<UserGroupId>.+?)$"), ['usergroup']),
    ],
    'elasticbeanstalk': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticbeanstalk:(?P<Region>[\w-]*)::platform/(?P<PlatformNameWithVersion>.+?)$"), ['platform']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticbeanstalk:(?P<Region>[\w-]*)::solutionstack/(?P<SolutionStackName>.+?)$"), ['solutionstack']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticbeanstalk:(?P<Region>[\w-]*):(?P<Account>\d{12}):applicationversion/(?P<ApplicationName>.+?)/(?P<VersionLabel>.+?)$"), ['applicationversion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticbeanstalk:(?P<Region>[\w-]*):(?P<Account>\d{12}):configurationtemplate/(?P<ApplicationName>.+?)/(?P<TemplateName>.+?)$"), ['configurationtemplate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticbeanstalk:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment/(?P<ApplicationName>.+?)/(?P<EnvironmentName>.+?)$"), ['environment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticbeanstalk:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationName>.+?)$"), ['application']),
    ],
    'elasticfilesystem': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticfilesystem:(?P<Region>[\w-]*):(?P<Account>\d{12}):access-point/(?P<AccessPointId>.+?)$"), ['access-point']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticfilesystem:(?P<Region>[\w-]*):(?P<Account>\d{12}):file-system/(?P<FileSystemId>.+?)$"), ['file-system']),
    ],
    'elasticloadbalancing': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticloadbalancing:(?P<Region>[\w-]*):(?P<Account>\d{12}):listener-rule/app/(?P<LoadBalancerName>.+?)/(?P<LoadBalancerId>.+?)/(?P<ListenerId>.+?)/(?P<ListenerRuleId>.+?)$"), ['listener-rule/app']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticloadbalancing:(?P<Region>[\w-]*):(?P<Account>\d{12}):listener-rule/net/(?P<LoadBalancerName>.+?)/(?P<LoadBalancerId>.+?)/(?P<ListenerId>.+?)/(?P<ListenerRuleId>.+?)$"), ['listener-rule/net']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticloadbalancing:(?P<Region>[\w-]*):(?P<Account>\d{12}):listener/app/(?P<LoadBalancerName>.+?)/(?P<LoadBalancerId>.+?)/(?P<ListenerId>.+?)$"), ['listener/app']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticloadbalancing:(?P<Region>[\w-]*):(?P<Account>\d{12}):listener/gwy/(?P<LoadBalancerName>.+?)/(?P<LoadBalancerId>.+?)/(?P<ListenerId>.+?)$"), ['listener/gwy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticloadbalancing:(?P<Region>[\w-]*):(?P<Account>\d{12}):listener/net/(?P<LoadBalancerName>.+?)/(?P<LoadBalancerId>.+?)/(?P<ListenerId>.+?)$"), ['listener/net']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticloadbalancing:(?P<Region>[\w-]*):(?P<Account>\d{12}):loadbalancer/app/(?P<LoadBalancerName>.+?)/(?P<LoadBalancerId>.+?)$"), ['loadbalancer/app/', 'loadbalancer/app']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticloadbalancing:(?P<Region>[\w-]*):(?P<Account>\d{12}):loadbalancer/gwy/(?P<LoadBalancerName>.+?)/(?P<LoadBalancerId>.+?)$"), ['loadbalancer/gwy/', 'loadbalancer/gwy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticloadbalancing:(?P<Region>[\w-]*):(?P<Account>\d{12}):loadbalancer/net/(?P<LoadBalancerName>.+?)/(?P<LoadBalancerId>.+?)$"), ['loadbalancer/net/', 'loadbalancer/net']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticloadbalancing:(?P<Region>[\w-]*):(?P<Account>\d{12}):targetgroup/(?P<TargetGroupName>.+?)/(?P<TargetGroupId>.+?)$"), ['targetgroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticloadbalancing:(?P<Region>[\w-]*):(?P<Account>\d{12}):truststore/(?P<TrustStoreName>.+?)/(?P<TrustStoreId>.+?)$"), ['truststore']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticloadbalancing:(?P<Region>[\w-]*):(?P<Account>\d{12}):loadbalancer/(?P<LoadBalancerName>.+?)$"), ['loadbalancer']),
    ],
    'elasticmapreduce': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticmapreduce:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster/(?P<ClusterId>.+?)$"), ['cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticmapreduce:(?P<Region>[\w-]*):(?P<Account>\d{12}):editor/(?P<EditorId>.+?)$"), ['editor']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticmapreduce:(?P<Region>[\w-]*):(?P<Account>\d{12}):notebook-execution/(?P<NotebookExecutionId>.+?)$"), ['notebook-execution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elasticmapreduce:(?P<Region>[\w-]*):(?P<Account>\d{12}):studio/(?P<StudioId>.+?)$"), ['studio']),
    ],
    'elastictranscoder': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elastictranscoder:(?P<Region>[\w-]*):(?P<Account>\d{12}):job/(?P<JobId>.+?)$"), ['job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elastictranscoder:(?P<Region>[\w-]*):(?P<Account>\d{12}):pipeline/(?P<PipelineId>.+?)$"), ['pipeline']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elastictranscoder:(?P<Region>[\w-]*):(?P<Account>\d{12}):preset/(?P<PresetId>.+?)$"), ['preset']),
    ],
    'elemental-appliances-software': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elemental-appliances-software:(?P<Region>[\w-]*):(?P<Account>\d{12}):quote/(?P<ResourceId>.+?)$"), ['quote']),
    ],
    'elemental-support-cases': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):elemental-support-cases::(?P<Account>\d{12}):case/(?P<ResourceId>.+?)$"), ['case']),
    ],
    'emr-containers': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):emr-containers:(?P<Region>[\w-]*):(?P<Account>\d{12}):/virtualclusters/(?P<VirtualClusterId>.+?)/endpoints/(?P<EndpointId>.+?)$"), ['managedEndpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):emr-containers:(?P<Region>[\w-]*):(?P<Account>\d{12}):/virtualclusters/(?P<VirtualClusterId>.+?)/jobruns/(?P<JobRunId>.+?)$"), ['jobRun']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):emr-containers:(?P<Region>[\w-]*):(?P<Account>\d{12}):/jobtemplates/(?P<JobTemplateId>.+?)$"), ['jobTemplate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):emr-containers:(?P<Region>[\w-]*):(?P<Account>\d{12}):/securityconfigurations/(?P<SecurityConfigurationId>.+?)$"), ['securityConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):emr-containers:(?P<Region>[\w-]*):(?P<Account>\d{12}):/virtualclusters/(?P<VirtualClusterId>.+?)$"), ['virtualCluster']),
    ],
    'emr-serverless': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):emr-serverless:(?P<Region>[\w-]*):(?P<Account>\d{12}):/applications/(?P<ApplicationId>.+?)/jobruns/(?P<JobRunId>.+?)$"), ['jobRun']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):emr-serverless:(?P<Region>[\w-]*):(?P<Account>\d{12}):/applications/(?P<ApplicationId>.+?)$"), ['application']),
    ],
    'entityresolution': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):entityresolution:(?P<Region>[\w-]*):(?P<Account>\d{12}):providerservice/(?P<ProviderName>.+?)/(?P<ProviderServiceName>.+?)$"), ['ProviderService']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):entityresolution:(?P<Region>[\w-]*):(?P<Account>\d{12}):idmappingworkflow/(?P<WorkflowName>.+?)$"), ['IdMappingWorkflow']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):entityresolution:(?P<Region>[\w-]*):(?P<Account>\d{12}):idnamespace/(?P<IdNamespaceName>.+?)$"), ['IdNamespace']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):entityresolution:(?P<Region>[\w-]*):(?P<Account>\d{12}):matchingworkflow/(?P<WorkflowName>.+?)$"), ['MatchingWorkflow']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):entityresolution:(?P<Region>[\w-]*):(?P<Account>\d{12}):schemamapping/(?P<SchemaName>.+?)$"), ['SchemaMapping']),
    ],
    'es': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):es:(?P<Region>[\w-]*):(?P<Account>\d{12}):domain/(?P<DomainName>.+?)$"), ['domain']),
    ],
    'events': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):events:(?P<Region>[\w-]*)::event-source/(?P<EventSourceName>.+?)$"), ['event-source']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):events:(?P<Region>[\w-]*):(?P<Account>\d{12}):rule/(?P<EventBusName>.+?)/(?P<RuleName>.+?)$"), ['rule-on-custom-event-bus']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):events:(?P<Region>[\w-]*):(?P<Account>\d{12}):api-destination/(?P<ApiDestinationName>.+?)$"), ['api-destination']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):events:(?P<Region>[\w-]*):(?P<Account>\d{12}):archive/(?P<ArchiveName>.+?)$"), ['archive']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):events:(?P<Region>[\w-]*):(?P<Account>\d{12}):connection/(?P<ConnectionName>.+?)$"), ['connection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):events:(?P<Region>[\w-]*):(?P<Account>\d{12}):endpoint/(?P<EndpointName>.+?)$"), ['endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):events:(?P<Region>[\w-]*):(?P<Account>\d{12}):event-bus/(?P<EventBusName>.+?)$"), ['event-bus']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):events:(?P<Region>[\w-]*):(?P<Account>\d{12}):replay/(?P<ReplayName>.+?)$"), ['replay']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):events:(?P<Region>[\w-]*):(?P<Account>\d{12}):rule/(?P<RuleName>.+?)$"), ['rule-on-default-event-bus', 'rule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):events:(?P<Region>[\w-]*):(?P<Account>\d{12}):target/create-snapshot$"), ['create-snapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):events:(?P<Region>[\w-]*):(?P<Account>\d{12}):target/reboot-instance$"), ['reboot-instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):events:(?P<Region>[\w-]*):(?P<Account>\d{12}):target/stop-instance$"), ['stop-instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):events:(?P<Region>[\w-]*):(?P<Account>\d{12}):target/terminate-instance$"), ['terminate-instance']),
    ],
    'evidently': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):evidently:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ProjectName>.+?)/experiment/(?P<ExperimentName>.+?)$"), ['Experiment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):evidently:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ProjectName>.+?)/feature/(?P<FeatureName>.+?)$"), ['Feature']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):evidently:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ProjectName>.+?)/launch/(?P<LaunchName>.+?)$"), ['Launch']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):evidently:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ProjectName>.+?)$"), ['Project']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):evidently:(?P<Region>[\w-]*):(?P<Account>\d{12}):segment/(?P<SegmentName>.+?)$"), ['Segment']),
    ],
    'evs': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):evs:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment/(?P<EnvironmentIdentifier>.+?)$"), ['environment']),
    ],
    'execute-api': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):execute-api:(?P<Region>[\w-]*):(?P<Account>\d{12}):/domainnames/(?P<DomainName>.+?)\+(?P<DomainIdentifier>.+?)$"), ['execute-api-domain']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):execute-api:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<ApiId>.+?)/(?P<Stage>.+?)/(?P<Method>.+?)/(?P<ApiSpecificResourcePath>.+?)$"), ['execute-api-general']),
    ],
    'finspace': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):finspace:(?P<Region>[\w-]*):(?P<Account>\d{12}):kxEnvironment/(?P<EnvironmentId>.+?)/kxDatabase/(?P<KxDatabase>.+?)/kxDataview/(?P<KxDataview>.+?)$"), ['kxDataview']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):finspace:(?P<Region>[\w-]*):(?P<Account>\d{12}):kxEnvironment/(?P<EnvironmentId>.+?)/kxCluster/(?P<KxCluster>.+?)$"), ['kxCluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):finspace:(?P<Region>[\w-]*):(?P<Account>\d{12}):kxEnvironment/(?P<EnvironmentId>.+?)/kxDatabase/(?P<KxDatabase>.+?)$"), ['kxDatabase']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):finspace:(?P<Region>[\w-]*):(?P<Account>\d{12}):kxEnvironment/(?P<EnvironmentId>.+?)/kxScalingGroup/(?P<KxScalingGroup>.+?)$"), ['kxScalingGroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):finspace:(?P<Region>[\w-]*):(?P<Account>\d{12}):kxEnvironment/(?P<EnvironmentId>.+?)/kxUser/(?P<UserName>.+?)$"), ['kxUser']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):finspace:(?P<Region>[\w-]*):(?P<Account>\d{12}):kxEnvironment/(?P<EnvironmentId>.+?)/kxVolume/(?P<KxVolume>.+?)$"), ['kxVolume']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):finspace:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment/(?P<EnvironmentId>.+?)$"), ['environment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):finspace:(?P<Region>[\w-]*):(?P<Account>\d{12}):kxEnvironment/(?P<EnvironmentId>.+?)$"), ['kxEnvironment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):finspace:(?P<Region>[\w-]*):(?P<Account>\d{12}):user/(?P<UserId>.+?)$"), ['user']),
    ],
    'finspace-api': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):finspace-api:(?P<Region>[\w-]*):(?P<Account>\d{12}):/credentials/programmatic$"), ['credential']),
    ],
    'firehose': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):firehose:(?P<Region>[\w-]*):(?P<Account>\d{12}):deliverystream/(?P<DeliveryStreamName>.+?)$"), ['deliverystream']),
    ],
    'fis': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fis:(?P<Region>[\w-]*):(?P<Account>\d{12}):action/(?P<Id>.+?)$"), ['action']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fis:(?P<Region>[\w-]*):(?P<Account>\d{12}):experiment/(?P<Id>.+?)$"), ['experiment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fis:(?P<Region>[\w-]*):(?P<Account>\d{12}):experiment-template/(?P<Id>.+?)$"), ['experiment-template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fis:(?P<Region>[\w-]*):(?P<Account>\d{12}):safety-lever/(?P<Id>.+?)$"), ['safety-lever']),
    ],
    'fms': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fms:(?P<Region>[\w-]*):(?P<Account>\d{12}):applications-list/(?P<Id>.+?)$"), ['applications-list']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fms:(?P<Region>[\w-]*):(?P<Account>\d{12}):policy/(?P<Id>.+?)$"), ['policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fms:(?P<Region>[\w-]*):(?P<Account>\d{12}):protocols-list/(?P<Id>.+?)$"), ['protocols-list']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fms:(?P<Region>[\w-]*):(?P<Account>\d{12}):resource-set/(?P<Id>.+?)$"), ['resource-set']),
    ],
    'forecast': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:::algorithm/(?P<ResourceId>.+?)$"), ['algorithm']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataset/(?P<ResourceId>.+?)$"), ['dataset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataset-group/(?P<ResourceId>.+?)$"), ['datasetGroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataset-import-job/(?P<ResourceId>.+?)$"), ['datasetImportJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:(?P<Region>[\w-]*):(?P<Account>\d{12}):explainability/(?P<ResourceId>.+?)$"), ['explainability']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:(?P<Region>[\w-]*):(?P<Account>\d{12}):explainability-export/(?P<ResourceId>.+?)$"), ['explainabilityExport']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:(?P<Region>[\w-]*):(?P<Account>\d{12}):forecast/(?P<ResourceId>.+?)$"), ['forecast']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:(?P<Region>[\w-]*):(?P<Account>\d{12}):forecast-endpoint/(?P<ResourceId>.+?)$"), ['endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:(?P<Region>[\w-]*):(?P<Account>\d{12}):forecast-export-job/(?P<ResourceId>.+?)$"), ['forecastExport']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:(?P<Region>[\w-]*):(?P<Account>\d{12}):monitor/(?P<ResourceId>.+?)$"), ['monitor']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:(?P<Region>[\w-]*):(?P<Account>\d{12}):predictor/(?P<ResourceId>.+?)$"), ['predictor']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:(?P<Region>[\w-]*):(?P<Account>\d{12}):predictor-backtest-export-job/(?P<ResourceId>.+?)$"), ['predictorBacktestExportJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:(?P<Region>[\w-]*):(?P<Account>\d{12}):what-if-analysis/(?P<ResourceId>.+?)$"), ['whatIfAnalysis']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:(?P<Region>[\w-]*):(?P<Account>\d{12}):what-if-forecast/(?P<ResourceId>.+?)$"), ['whatIfForecast']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):forecast:(?P<Region>[\w-]*):(?P<Account>\d{12}):what-if-forecast-export/(?P<ResourceId>.+?)$"), ['whatIfForecastExport']),
    ],
    'frauddetector': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):frauddetector:(?P<Region>[\w-]*):(?P<Account>\d{12}):batch-import/(?P<ResourcePath>.+?)$"), ['batch-import']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):frauddetector:(?P<Region>[\w-]*):(?P<Account>\d{12}):batch-prediction/(?P<ResourcePath>.+?)$"), ['batch-prediction']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):frauddetector:(?P<Region>[\w-]*):(?P<Account>\d{12}):detector/(?P<ResourcePath>.+?)$"), ['detector']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):frauddetector:(?P<Region>[\w-]*):(?P<Account>\d{12}):detector-version/(?P<ResourcePath>.+?)$"), ['detector-version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):frauddetector:(?P<Region>[\w-]*):(?P<Account>\d{12}):entity-type/(?P<ResourcePath>.+?)$"), ['entity-type']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):frauddetector:(?P<Region>[\w-]*):(?P<Account>\d{12}):event-type/(?P<ResourcePath>.+?)$"), ['event-type']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):frauddetector:(?P<Region>[\w-]*):(?P<Account>\d{12}):external-model/(?P<ResourcePath>.+?)$"), ['external-model']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):frauddetector:(?P<Region>[\w-]*):(?P<Account>\d{12}):label/(?P<ResourcePath>.+?)$"), ['label']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):frauddetector:(?P<Region>[\w-]*):(?P<Account>\d{12}):list/(?P<ResourcePath>.+?)$"), ['list']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):frauddetector:(?P<Region>[\w-]*):(?P<Account>\d{12}):model/(?P<ResourcePath>.+?)$"), ['model']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):frauddetector:(?P<Region>[\w-]*):(?P<Account>\d{12}):model-version/(?P<ResourcePath>.+?)$"), ['model-version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):frauddetector:(?P<Region>[\w-]*):(?P<Account>\d{12}):outcome/(?P<ResourcePath>.+?)$"), ['outcome']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):frauddetector:(?P<Region>[\w-]*):(?P<Account>\d{12}):rule/(?P<ResourcePath>.+?)$"), ['rule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):frauddetector:(?P<Region>[\w-]*):(?P<Account>\d{12}):variable/(?P<ResourcePath>.+?)$"), ['variable']),
    ],
    'freertos': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):freertos:(?P<Region>[\w-]*):(?P<Account>\d{12}):configuration/(?P<ConfigurationName>.+?)$"), ['configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):freertos:(?P<Region>[\w-]*):(?P<Account>\d{12}):subscription/(?P<SubscriptionID>.+?)$"), ['subscription']),
    ],
    'fsx': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fsx:(?P<Region>[\w-]*):(?P<Account>\d{12}):association/(?P<FileSystemIdOrFileCacheId>.+?)/(?P<DataRepositoryAssociationId>.+?)$"), ['association']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fsx:(?P<Region>[\w-]*):(?P<Account>\d{12}):snapshot/(?P<VolumeId>.+?)/(?P<SnapshotId>.+?)$"), ['snapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fsx:(?P<Region>[\w-]*):(?P<Account>\d{12}):storage-virtual-machine/(?P<FileSystemId>.+?)/(?P<StorageVirtualMachineId>.+?)$"), ['storage-virtual-machine']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fsx:(?P<Region>[\w-]*):(?P<Account>\d{12}):volume/(?P<FileSystemId>.+?)/(?P<VolumeId>.+?)$"), ['volume']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fsx:(?P<Region>[\w-]*):(?P<Account>\d{12}):backup/(?P<BackupId>.+?)$"), ['backup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fsx:(?P<Region>[\w-]*):(?P<Account>\d{12}):file-cache/(?P<FileCacheId>.+?)$"), ['file-cache']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fsx:(?P<Region>[\w-]*):(?P<Account>\d{12}):file-system/(?P<FileSystemId>.+?)$"), ['file-system']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):fsx:(?P<Region>[\w-]*):(?P<Account>\d{12}):task/(?P<TaskId>.+?)$"), ['task']),
    ],
    'gamelift': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):gamelift:(?P<Region>[\w-]*)::alias/(?P<AliasId>.+?)$"), ['alias']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):gamelift:(?P<Region>[\w-]*):(?P<Account>\d{12}):build/(?P<BuildId>.+?)$"), ['build']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):gamelift:(?P<Region>[\w-]*):(?P<Account>\d{12}):containerfleet/(?P<FleetId>.+?)$"), ['containerFleet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):gamelift:(?P<Region>[\w-]*):(?P<Account>\d{12}):containergroupdefinition/(?P<Name>.+?)$"), ['containerGroupDefinition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):gamelift:(?P<Region>[\w-]*):(?P<Account>\d{12}):fleet/(?P<FleetId>.+?)$"), ['fleet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):gamelift:(?P<Region>[\w-]*):(?P<Account>\d{12}):gameservergroup/(?P<GameServerGroupName>.+?)$"), ['gameServerGroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):gamelift:(?P<Region>[\w-]*):(?P<Account>\d{12}):gamesessionqueue/(?P<GameSessionQueueName>.+?)$"), ['gameSessionQueue']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):gamelift:(?P<Region>[\w-]*):(?P<Account>\d{12}):location/(?P<LocationId>.+?)$"), ['location']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):gamelift:(?P<Region>[\w-]*):(?P<Account>\d{12}):matchmakingconfiguration/(?P<MatchmakingConfigurationName>.+?)$"), ['matchmakingConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):gamelift:(?P<Region>[\w-]*):(?P<Account>\d{12}):matchmakingruleset/(?P<MatchmakingRuleSetName>.+?)$"), ['matchmakingRuleSet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):gamelift:(?P<Region>[\w-]*):(?P<Account>\d{12}):script/(?P<ScriptId>.+?)$"), ['script']),
    ],
    'gameliftstreams': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):gameliftstreams:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)$"), ['application']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):gameliftstreams:(?P<Region>[\w-]*):(?P<Account>\d{12}):streamgroup/(?P<StreamGroupId>.+?)$"), ['stream group']),
    ],
    'geo': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):geo:(?P<Region>[\w-]*):(?P<Account>\d{12}):api-key/(?P<KeyName>.+?)$"), ['api-key']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):geo:(?P<Region>[\w-]*):(?P<Account>\d{12}):geofence-collection/(?P<GeofenceCollectionName>.+?)$"), ['geofence-collection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):geo:(?P<Region>[\w-]*):(?P<Account>\d{12}):map/(?P<MapName>.+?)$"), ['map']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):geo:(?P<Region>[\w-]*):(?P<Account>\d{12}):place-index/(?P<IndexName>.+?)$"), ['place-index']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):geo:(?P<Region>[\w-]*):(?P<Account>\d{12}):route-calculator/(?P<CalculatorName>.+?)$"), ['route-calculator']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):geo:(?P<Region>[\w-]*):(?P<Account>\d{12}):tracker/(?P<TrackerName>.+?)$"), ['tracker']),
    ],
    'geo-maps': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):geo-maps:(?P<Region>[\w-]*)::provider/default$"), ['provider']),
    ],
    'geo-places': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):geo-places:(?P<Region>[\w-]*)::provider/default$"), ['provider']),
    ],
    'geo-routes': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):geo-routes:(?P<Region>[\w-]*)::provider/default$"), ['provider']),
    ],
    'glacier': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glacier:(?P<Region>[\w-]*):(?P<Account>\d{12}):vaults/(?P<VaultName>.+?)$"), ['vault']),
    ],
    'globalaccelerator': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):globalaccelerator::(?P<Account>\d{12}):accelerator/(?P<ResourceId>.+?)/listener/(?P<ListenerId>.+?)/endpoint-group/(?P<EndpointGroupId>.+?)$"), ['endpointgroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):globalaccelerator::(?P<Account>\d{12}):accelerator/(?P<ResourceId>.+?)/listener/(?P<ListenerId>.+?)$"), ['listener']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):globalaccelerator::(?P<Account>\d{12}):accelerator/(?P<ResourceId>.+?)$"), ['accelerator']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):globalaccelerator::(?P<Account>\d{12}):attachment/(?P<ResourceId>.+?)$"), ['attachment']),
    ],
    'glue': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):tableVersion/(?P<DatabaseName>.+?)/(?P<TableName>.+?)/(?P<TableVersionName>.+?)$"), ['tableversion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):table/(?P<DatabaseName>.+?)/(?P<TableName>.+?)$"), ['table']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):userDefinedFunction/(?P<DatabaseName>.+?)/(?P<UserDefinedFunctionName>.+?)$"), ['userdefinedfunction']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):blueprint/(?P<BlueprintName>.+?)$"), ['blueprint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):catalog/(?P<CatalogName>.+?)$"), ['catalog']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):completion/(?P<CompletionId>.+?)$"), ['completion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):connection/(?P<ConnectionName>.+?)$"), ['connection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):crawler/(?P<CrawlerName>.+?)$"), ['crawler']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):customEntityType/(?P<CustomEntityTypeId>.+?)$"), ['customEntityType']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataQualityRuleset/(?P<RulesetName>.+?)$"), ['dataQualityRuleset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):database/(?P<DatabaseName>.+?)$"), ['database']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):devEndpoint/(?P<DevEndpointName>.+?)$"), ['devendpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):integration:(?P<IntegrationId>.+?)$"), ['integration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):job/(?P<JobName>.+?)$"), ['job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):mlTransform/(?P<TransformId>.+?)$"), ['mlTransform']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):registry/(?P<RegistryName>.+?)$"), ['registry']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):schema/(?P<SchemaName>.+?)$"), ['schema']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):session/(?P<SessionId>.+?)$"), ['session']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):trigger/(?P<TriggerName>.+?)$"), ['trigger']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):usageProfile/(?P<UsageProfileId>.+?)$"), ['usageProfile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):workflow/(?P<WorkflowName>.+?)$"), ['workflow']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):glue:(?P<Region>[\w-]*):(?P<Account>\d{12}):catalog$"), ['rootcatalog']),
    ],
    'grafana': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):grafana:(?P<Region>[\w-]*):(?P<Account>\d{12}):/workspaces/(?P<ResourceId>.+?)$"), ['workspace']),
    ],
    'greengrass': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/definition/connectors/(?P<ConnectorDefinitionId>.+?)/versions/(?P<VersionId>.+?)$"), ['connectorDefinitionVersion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/definition/cores/(?P<CoreDefinitionId>.+?)/versions/(?P<VersionId>.+?)$"), ['coreDefinitionVersion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/definition/devices/(?P<DeviceDefinitionId>.+?)/versions/(?P<VersionId>.+?)$"), ['deviceDefinitionVersion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/definition/functions/(?P<FunctionDefinitionId>.+?)/versions/(?P<VersionId>.+?)$"), ['functionDefinitionVersion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/definition/loggers/(?P<LoggerDefinitionId>.+?)/versions/(?P<VersionId>.+?)$"), ['loggerDefinitionVersion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/definition/resources/(?P<ResourceDefinitionId>.+?)/versions/(?P<VersionId>.+?)$"), ['resourceDefinitionVersion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/definition/subscriptions/(?P<SubscriptionDefinitionId>.+?)/versions/(?P<VersionId>.+?)$"), ['subscriptionDefinitionVersion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/groups/(?P<GroupId>.+?)/certificateauthorities/(?P<CertificateAuthorityId>.+?)$"), ['certificateAuthority']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/groups/(?P<GroupId>.+?)/deployments/(?P<DeploymentId>.+?)$"), ['deployment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/groups/(?P<GroupId>.+?)/versions/(?P<VersionId>.+?)$"), ['groupVersion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):components:(?P<ComponentName>.+?):versions:(?P<ComponentVersion>.+?)$"), ['componentVersion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/bulk/deployments/(?P<BulkDeploymentId>.+?)$"), ['bulkDeployment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/definition/connectors/(?P<ConnectorDefinitionId>.+?)$"), ['connectorDefinition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/definition/cores/(?P<CoreDefinitionId>.+?)$"), ['coreDefinition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/definition/devices/(?P<DeviceDefinitionId>.+?)$"), ['deviceDefinition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/definition/functions/(?P<FunctionDefinitionId>.+?)$"), ['functionDefinition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/definition/loggers/(?P<LoggerDefinitionId>.+?)$"), ['loggerDefinition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/definition/resources/(?P<ResourceDefinitionId>.+?)$"), ['resourceDefinition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/definition/subscriptions/(?P<SubscriptionDefinitionId>.+?)$"), ['subscriptionDefinition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/things/(?P<ThingName>.+?)/connectivityInfo$"), ['connectivityInfo']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/things/(?P<ThingName>.+?)/runtimeconfig$"), ['thingRuntimeConfig']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):/greengrass/groups/(?P<GroupId>.+?)$"), ['group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):components:(?P<ComponentName>.+?)$"), ['component']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):coreDevices:(?P<CoreDeviceThingName>.+?)$"), ['coreDevice']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):greengrass:(?P<Region>[\w-]*):(?P<Account>\d{12}):deployments:(?P<DeploymentId>.+?)$"), ['deployment']),
    ],
    'groundstation': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):groundstation:(?P<Region>[\w-]*):(?P<Account>\d{12}):config/(?P<ConfigType>.+?)/(?P<ConfigId>.+?)$"), ['Config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):groundstation:(?P<Region>[\w-]*):(?P<Account>\d{12}):agent/(?P<AgentId>.+?)$"), ['Agent']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):groundstation:(?P<Region>[\w-]*):(?P<Account>\d{12}):contact/(?P<ContactId>.+?)$"), ['Contact']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):groundstation:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataflow-endpoint-group/(?P<DataflowEndpointGroupId>.+?)$"), ['DataflowEndpointGroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):groundstation:(?P<Region>[\w-]*):(?P<Account>\d{12}):ephemeris/(?P<EphemerisId>.+?)$"), ['EphemerisItem']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):groundstation:(?P<Region>[\w-]*):(?P<Account>\d{12}):groundstation:(?P<GroundStationId>.+?)$"), ['GroundStationResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):groundstation:(?P<Region>[\w-]*):(?P<Account>\d{12}):mission-profile/(?P<MissionProfileId>.+?)$"), ['MissionProfile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):groundstation:(?P<Region>[\w-]*):(?P<Account>\d{12}):satellite/(?P<SatelliteId>.+?)$"), ['Satellite']),
    ],
    'guardduty': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):guardduty:(?P<Region>[\w-]*):(?P<Account>\d{12}):detector/(?P<DetectorId>.+?)/filter/(?P<FilterName>.+?)$"), ['filter']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):guardduty:(?P<Region>[\w-]*):(?P<Account>\d{12}):detector/(?P<DetectorId>.+?)/ipset/(?P<IPSetId>.+?)$"), ['ipset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):guardduty:(?P<Region>[\w-]*):(?P<Account>\d{12}):detector/(?P<DetectorId>.+?)/publishingdestination/(?P<PublishingDestinationId>.+?)$"), ['publishingDestination']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):guardduty:(?P<Region>[\w-]*):(?P<Account>\d{12}):detector/(?P<DetectorId>.+?)/threatentityset/(?P<ThreatEntitySetId>.+?)$"), ['threatentityset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):guardduty:(?P<Region>[\w-]*):(?P<Account>\d{12}):detector/(?P<DetectorId>.+?)/threatintelset/(?P<ThreatIntelSetId>.+?)$"), ['threatintelset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):guardduty:(?P<Region>[\w-]*):(?P<Account>\d{12}):detector/(?P<DetectorId>.+?)/trustedentityset/(?P<TrustedEntitySetId>.+?)$"), ['trustedentityset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):guardduty:(?P<Region>[\w-]*):(?P<Account>\d{12}):detector/(?P<DetectorId>.+?)$"), ['detector']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):guardduty:(?P<Region>[\w-]*):(?P<Account>\d{12}):malware-protection-plan/(?P<MalwareProtectionPlanId>.+?)$"), ['malwareprotectionplan']),
    ],
    'health': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):health:(?P<Region>[\w-]*):(?P<Account>\d{12}):event/(?P<Service>.+?)/(?P<EventTypeCode>.+?)/(?P<EventId>.+?)$"), ['event']),
    ],
    'healthlake': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):healthlake:(?P<Region>[\w-]*):(?P<Account>\d{12}):datastore/fhir/(?P<DatastoreId>.+?)$"), ['datastore']),
    ],
    'honeycode': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):honeycode:(?P<Region>[\w-]*):(?P<Account>\d{12}):screen-automation:workbook/(?P<WorkbookId>.+?)/app/(?P<AppId>.+?)/screen/(?P<ScreenId>.+?)/automation/(?P<AutomationId>.+?)$"), ['screen-automation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):honeycode:(?P<Region>[\w-]*):(?P<Account>\d{12}):screen:workbook/(?P<WorkbookId>.+?)/app/(?P<AppId>.+?)/screen/(?P<ScreenId>.+?)$"), ['screen']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):honeycode:(?P<Region>[\w-]*):(?P<Account>\d{12}):table:workbook/(?P<WorkbookId>.+?)/table/(?P<TableId>.+?)$"), ['table']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):honeycode:(?P<Region>[\w-]*):(?P<Account>\d{12}):workbook:workbook/(?P<WorkbookId>.+?)$"), ['workbook']),
    ],
    'iam': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::aws:contextProvider/(?P<ContextProviderName>.+?)$"), ['context-provider']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):role/aws-service-role/es\.amazonaws\.com/AWSServiceRoleForAmazonOpenSearchService$"), ['es_role']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):role/aws-service-role/opensearchservice\.amazonaws\.com/AWSServiceRoleForAmazonOpenSearchService$"), ['opensearchservice_role']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):assumed-role/(?P<RoleName>.+?)/(?P<RoleSessionName>.+?)$"), ['assumed-role']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):access-report/(?P<EntityPath>.+?)$"), ['access-report']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):delegation-request/(?P<DelegationRequestId>.+?)$"), ['delegation-request']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):federated-user/(?P<UserName>.+?)$"), ['federated-user']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):group/(?P<GroupNameWithPath>.+?)$"), ['group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):instance-profile/(?P<InstanceProfileNameWithPath>.+?)$"), ['instance-profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):mfa/(?P<MfaTokenIdWithPath>.+?)$"), ['mfa']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):oidc-provider/(?P<OidcProviderName>.+?)$"), ['oidc-provider']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):policy/(?P<PolicyNameWithPath>.+?)$"), ['policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):role/(?P<RoleNameWithPath>.+?)$"), ['role']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):role/(?P<RoleName>.+?)$"), ['iam-role']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):saml-provider/(?P<SamlProviderName>.+?)$"), ['saml-provider']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):server-certificate/(?P<CertificateNameWithPath>.+?)$"), ['server-certificate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):sms-mfa/(?P<MfaTokenIdWithPath>.+?)$"), ['sms-mfa']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):user/(?P<AwsUserName>.+?)$"), ['user']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):user/(?P<UserNameWithPath>.+?)$"), ['user']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iam::(?P<Account>\d{12}):root$"), ['root-user']),
    ],
    'identity-sync': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):identity-sync:(?P<Region>[\w-]*):(?P<Account>\d{12}):target/(?P<SyncProfileName>.+?)/(?P<SyncTargetName>.+?)$"), ['SyncTargetResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):identity-sync:(?P<Region>[\w-]*):(?P<Account>\d{12}):profile/(?P<SyncProfileName>.+?)$"), ['SyncProfileResource']),
    ],
    'identitystore': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):identitystore:::group/(?P<GroupId>.+?)$"), ['Group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):identitystore:::membership/(?P<MembershipId>.+?)$"), ['GroupMembership']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):identitystore:::user/(?P<UserId>.+?)$"), ['User']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):identitystore::(?P<Account>\d{12}):identitystore/(?P<IdentityStoreId>.+?)$"), ['Identitystore']),
    ],
    'imagebuilder': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):imagebuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):workflow/(?P<WorkflowType>.+?)/(?P<WorkflowName>.+?)/(?P<WorkflowVersion>.+?)/(?P<WorkflowBuildVersion>.+?)$"), ['workflow']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):imagebuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):component/(?P<ComponentName>.+?)/(?P<ComponentVersion>.+?)/(?P<ComponentBuildVersion>.+?)$"), ['component']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):imagebuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):image/(?P<ImageName>.+?)/(?P<ImageVersion>.+?)/(?P<ImageBuildVersion>.+?)$"), ['image']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):imagebuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):container-recipe/(?P<ContainerRecipeName>.+?)/(?P<ContainerRecipeVersion>.+?)$"), ['containerRecipe']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):imagebuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):image/(?P<ImageName>.+?)/(?P<ImageVersion>.+?)$"), ['imageVersion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):imagebuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):image-recipe/(?P<ImageRecipeName>.+?)/(?P<ImageRecipeVersion>.+?)$"), ['imageRecipe']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):imagebuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):distribution-configuration/(?P<DistributionConfigurationName>.+?)$"), ['distributionConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):imagebuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):image-pipeline/(?P<ImagePipelineName>.+?)$"), ['imagePipeline']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):imagebuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):infrastructure-configuration/(?P<ResourceId>.+?)$"), ['infrastructureConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):imagebuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):lifecycle-execution/(?P<LifecycleExecutionId>.+?)$"), ['lifecycleExecution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):imagebuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):lifecycle-policy/(?P<LifecyclePolicyName>.+?)$"), ['lifecyclePolicy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):imagebuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):workflow-execution/(?P<WorkflowExecutionId>.+?)$"), ['workflowExecution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):imagebuilder:(?P<Region>[\w-]*):(?P<Account>\d{12}):workflow-step-execution/(?P<WorkflowStepExecutionId>.+?)$"), ['workflowStepExecution']),
    ],
    'inspector': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):inspector:(?P<Region>[\w-]*):(?P<Account>\d{12}):target/(?P<TargetId>.+?)/template/(?P<TemplateId>.+?)$"), ['target-template', 'target/template']),
    ],
    'inspector2': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):inspector2:(?P<Region>[\w-]*):(?P<Account>\d{12}):owner/(?P<OwnerId>.+?)/cis-configuration/(?P<CISScanConfigurationId>.+?)$"), ['CIS Scan Configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):inspector2:(?P<Region>[\w-]*):(?P<Account>\d{12}):owner/(?P<OwnerId>.+?)/codesecurity-configuration/(?P<CodeSecurityScanConfigurationId>.+?)$"), ['Code Security Scan Configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):inspector2:(?P<Region>[\w-]*):(?P<Account>\d{12}):owner/(?P<OwnerId>.+?)/filter/(?P<FilterId>.+?)$"), ['Filter']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):inspector2:(?P<Region>[\w-]*):(?P<Account>\d{12}):codesecurity-integration/(?P<CodeSecurityIntegrationId>.+?)$"), ['Code Security Integration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):inspector2:(?P<Region>[\w-]*):(?P<Account>\d{12}):finding/(?P<FindingId>.+?)$"), ['Finding']),
    ],
    'internetmonitor': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):internetmonitor::(?P<Account>\d{12}):internet-event/(?P<InternetEventId>.+?)$"), ['InternetEvent']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):internetmonitor:(?P<Region>[\w-]*):(?P<Account>\d{12}):monitor/(?P<MonitorName>.+?)/health-event/(?P<EventId>.+?)$"), ['HealthEvent']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):internetmonitor:(?P<Region>[\w-]*):(?P<Account>\d{12}):monitor/(?P<MonitorName>.+?)$"), ['Monitor']),
    ],
    'invoicing': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):invoicing::(?P<Account>\d{12}):invoice-unit/(?P<Identifier>.+?)$"), ['invoice-unit']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):invoicing::(?P<Account>\d{12}):procurement-portal-preference/(?P<Identifier>.+?)$"), ['procurement-portal-preference']),
    ],
    'iot': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):package/(?P<PackageName>.+?)/version/(?P<VersionName>.+?)$"), ['packageversion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):domainconfiguration/(?P<DomainConfigurationName>.+?)/(?P<Id>.+?)$"), ['domainconfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):ruledestination/(?P<DestinationType>.+?)/(?P<Uuid>.+?)$"), ['destination']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):authorizer/(?P<AuthorizerName>.+?)$"), ['authorizer']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):billinggroup/(?P<BillingGroupName>.+?)$"), ['billinggroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):cacert/(?P<CACertificate>.+?)$"), ['cacert']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):cert/(?P<Certificate>.+?)$"), ['cert']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):certificateprovider/(?P<CertificateProviderName>.+?)$"), ['certificateprovider']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):client/(?P<ClientId>.+?)$"), ['client']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):command/(?P<CommandId>.+?)$"), ['command']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):custommetric/(?P<MetricName>.+?)$"), ['custommetric']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):dimension/(?P<DimensionName>.+?)$"), ['dimension']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):fleetmetric/(?P<FleetMetricName>.+?)$"), ['fleetmetric']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):index/(?P<IndexName>.+?)$"), ['index']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):job/(?P<JobId>.+?)$"), ['job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):jobtemplate/(?P<JobTemplateId>.+?)$"), ['jobtemplate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):mitigationaction/(?P<MitigationActionName>.+?)$"), ['mitigationaction']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):otaupdate/(?P<OtaUpdateId>.+?)$"), ['otaupdate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):package/(?P<PackageName>.+?)$"), ['package']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):policy/(?P<PolicyName>.+?)$"), ['policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):provisioningtemplate/(?P<ProvisioningTemplate>.+?)$"), ['provisioningtemplate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):rolealias/(?P<RoleAlias>.+?)$"), ['rolealias']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):rule/(?P<RuleName>.+?)$"), ['rule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):scheduledaudit/(?P<ScheduleName>.+?)$"), ['scheduledaudit']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):securityprofile/(?P<SecurityProfileName>.+?)$"), ['securityprofile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):stream/(?P<StreamId>.+?)$"), ['stream']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):thing/(?P<ThingName>.+?)$"), ['thing']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):thingtype/(?P<ThingTypeName>.+?)$"), ['thingtype']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):topic/(?P<TopicName>.+?)$"), ['topic']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):topicfilter/(?P<TopicFilter>.+?)$"), ['topicfilter']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iot:(?P<Region>[\w-]*):(?P<Account>\d{12}):tunnel/(?P<TunnelId>.+?)$"), ['tunnel']),
    ],
    'iotanalytics': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotanalytics:(?P<Region>[\w-]*):(?P<Account>\d{12}):channel/(?P<ChannelName>.+?)$"), ['channel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotanalytics:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataset/(?P<DatasetName>.+?)$"), ['dataset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotanalytics:(?P<Region>[\w-]*):(?P<Account>\d{12}):datastore/(?P<DatastoreName>.+?)$"), ['datastore']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotanalytics:(?P<Region>[\w-]*):(?P<Account>\d{12}):pipeline/(?P<PipelineName>.+?)$"), ['pipeline']),
    ],
    'iotdeviceadvisor': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotdeviceadvisor:(?P<Region>[\w-]*):(?P<Account>\d{12}):suiterun/(?P<SuiteDefinitionId>.+?)/(?P<SuiteRunId>.+?)$"), ['Suiterun']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotdeviceadvisor:(?P<Region>[\w-]*):(?P<Account>\d{12}):suitedefinition/(?P<SuiteDefinitionId>.+?)$"), ['Suitedefinition']),
    ],
    'iotevents': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotevents:(?P<Region>[\w-]*):(?P<Account>\d{12}):alarmModel/(?P<AlarmModelName>.+?)$"), ['alarmModel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotevents:(?P<Region>[\w-]*):(?P<Account>\d{12}):detectorModel/(?P<DetectorModelName>.+?)$"), ['detectorModel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotevents:(?P<Region>[\w-]*):(?P<Account>\d{12}):input/(?P<InputName>.+?)$"), ['input']),
    ],
    'iotfleethub': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotfleethub:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)$"), ['application']),
    ],
    'iotfleetwise': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotfleetwise:(?P<Region>[\w-]*):(?P<Account>\d{12}):campaign/(?P<CampaignName>.+?)$"), ['campaign']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotfleetwise:(?P<Region>[\w-]*):(?P<Account>\d{12}):decoder-manifest/(?P<Name>.+?)$"), ['decodermanifest']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotfleetwise:(?P<Region>[\w-]*):(?P<Account>\d{12}):fleet/(?P<FleetId>.+?)$"), ['fleet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotfleetwise:(?P<Region>[\w-]*):(?P<Account>\d{12}):model-manifest/(?P<Name>.+?)$"), ['modelmanifest']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotfleetwise:(?P<Region>[\w-]*):(?P<Account>\d{12}):signal-catalog/(?P<Name>.+?)$"), ['signalcatalog']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotfleetwise:(?P<Region>[\w-]*):(?P<Account>\d{12}):state-template/(?P<StateTemplateId>.+?)$"), ['statetemplate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotfleetwise:(?P<Region>[\w-]*):(?P<Account>\d{12}):vehicle/(?P<VehicleId>.+?)$"), ['vehicle']),
    ],
    'iotmanagedintegrations': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotmanagedintegrations:(?P<Region>[\w-]*):(?P<Account>\d{12}):account-association/(?P<AccountAssociationId>.+?)$"), ['account-association']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotmanagedintegrations:(?P<Region>[\w-]*):(?P<Account>\d{12}):credential-locker/(?P<Identifier>.+?)$"), ['credential-locker']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotmanagedintegrations:(?P<Region>[\w-]*):(?P<Account>\d{12}):managed-thing/(?P<Identifier>.+?)$"), ['managed-thing']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotmanagedintegrations:(?P<Region>[\w-]*):(?P<Account>\d{12}):ota-task/(?P<Identifier>.+?)$"), ['ota-task']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotmanagedintegrations:(?P<Region>[\w-]*):(?P<Account>\d{12}):provisioning-profile/(?P<Identifier>.+?)$"), ['provisioning-profile']),
    ],
    'iotsitewise': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotsitewise:(?P<Region>[\w-]*):(?P<Account>\d{12}):access-policy/(?P<AccessPolicyId>.+?)$"), ['access-policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotsitewise:(?P<Region>[\w-]*):(?P<Account>\d{12}):asset/(?P<AssetId>.+?)$"), ['asset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotsitewise:(?P<Region>[\w-]*):(?P<Account>\d{12}):asset-model/(?P<AssetModelId>.+?)$"), ['asset-model']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotsitewise:(?P<Region>[\w-]*):(?P<Account>\d{12}):computation-model/(?P<ComputationModelId>.+?)$"), ['computation-model']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotsitewise:(?P<Region>[\w-]*):(?P<Account>\d{12}):dashboard/(?P<DashboardId>.+?)$"), ['dashboard']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotsitewise:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataset/(?P<DatasetId>.+?)$"), ['dataset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotsitewise:(?P<Region>[\w-]*):(?P<Account>\d{12}):gateway/(?P<GatewayId>.+?)$"), ['gateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotsitewise:(?P<Region>[\w-]*):(?P<Account>\d{12}):portal/(?P<PortalId>.+?)$"), ['portal']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotsitewise:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ProjectId>.+?)$"), ['project']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotsitewise:(?P<Region>[\w-]*):(?P<Account>\d{12}):time-series/(?P<TimeSeriesId>.+?)$"), ['time-series']),
    ],
    'iottwinmaker': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iottwinmaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):workspace/(?P<WorkspaceId>.+?)/component-type/(?P<ComponentTypeId>.+?)$"), ['componentType']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iottwinmaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):workspace/(?P<WorkspaceId>.+?)/entity/(?P<EntityId>.+?)$"), ['entity']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iottwinmaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):workspace/(?P<WorkspaceId>.+?)/scene/(?P<SceneId>.+?)$"), ['scene']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iottwinmaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):workspace/(?P<WorkspaceId>.+?)/sync-job/(?P<SyncJobId>.+?)$"), ['syncJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iottwinmaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):metadata-transfer-job/(?P<MetadataTransferJobId>.+?)$"), ['metadataTransferJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iottwinmaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):workspace/(?P<WorkspaceId>.+?)$"), ['workspace']),
    ],
    'iotwireless': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotwireless:(?P<Region>[\w-]*):(?P<Account>\d{12}):Destination/(?P<DestinationName>.+?)$"), ['Destination']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotwireless:(?P<Region>[\w-]*):(?P<Account>\d{12}):DeviceProfile/(?P<DeviceProfileId>.+?)$"), ['DeviceProfile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotwireless:(?P<Region>[\w-]*):(?P<Account>\d{12}):FuotaTask/(?P<FuotaTaskId>.+?)$"), ['FuotaTask']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotwireless:(?P<Region>[\w-]*):(?P<Account>\d{12}):ImportTask/(?P<ImportTaskId>.+?)$"), ['ImportTask']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotwireless:(?P<Region>[\w-]*):(?P<Account>\d{12}):MulticastGroup/(?P<MulticastGroupId>.+?)$"), ['MulticastGroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotwireless:(?P<Region>[\w-]*):(?P<Account>\d{12}):NetworkAnalyzerConfiguration/(?P<NetworkAnalyzerConfigurationName>.+?)$"), ['NetworkAnalyzerConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotwireless:(?P<Region>[\w-]*):(?P<Account>\d{12}):ServiceProfile/(?P<ServiceProfileId>.+?)$"), ['ServiceProfile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotwireless:(?P<Region>[\w-]*):(?P<Account>\d{12}):SidewalkAccount/(?P<SidewalkAccountId>.+?)$"), ['SidewalkAccount']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotwireless:(?P<Region>[\w-]*):(?P<Account>\d{12}):WirelessDevice/(?P<WirelessDeviceId>.+?)$"), ['WirelessDevice']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotwireless:(?P<Region>[\w-]*):(?P<Account>\d{12}):WirelessGateway/(?P<WirelessGatewayId>.+?)$"), ['WirelessGateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iotwireless:(?P<Region>[\w-]*):(?P<Account>\d{12}):WirelessGatewayTaskDefinition/(?P<WirelessGatewayTaskDefinitionId>.+?)$"), ['WirelessGatewayTaskDefinition']),
    ],
    'iq': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iq:(?P<Region>[\w-]*)::paymentRequest/(?P<ConversationId>.+?)/(?P<ProposalId>.+?)/(?P<PaymentRequestId>.+?)$"), ['paymentRequest']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iq:(?P<Region>[\w-]*)::paymentSchedule/(?P<ConversationId>.+?)/(?P<ProposalId>.+?)/(?P<VersionId>.+?)$"), ['paymentSchedule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iq:(?P<Region>[\w-]*)::proposal/(?P<ConversationId>.+?)/(?P<ProposalId>.+?)$"), ['proposal']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iq:(?P<Region>[\w-]*)::attachment/(?P<AttachmentId>.+?)$"), ['attachment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iq:(?P<Region>[\w-]*)::buyer/(?P<BuyerId>.+?)$"), ['buyer']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iq:(?P<Region>[\w-]*)::call/(?P<CallId>.+?)$"), ['call']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iq:(?P<Region>[\w-]*)::company/(?P<CompanyId>.+?)$"), ['company']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iq:(?P<Region>[\w-]*)::conversation/(?P<ConversationId>.+?)$"), ['conversation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iq:(?P<Region>[\w-]*)::expert/(?P<ExpertId>.+?)$"), ['expert']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iq:(?P<Region>[\w-]*)::listing/(?P<ListingId>.+?)$"), ['listing']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iq:(?P<Region>[\w-]*)::request/(?P<RequestId>.+?)$"), ['request']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iq:(?P<Region>[\w-]*)::seller/(?P<SellerAwsAccountId>.+?)$"), ['seller']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iq:(?P<Region>[\w-]*)::token/(?P<TokenId>.+?)$"), ['token']),
    ],
    'iq-permission': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):iq-permission:(?P<Region>[\w-]*)::permission/(?P<PermissionRequestId>.+?)$"), ['permission']),
    ],
    'ivs': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ivs:(?P<Region>[\w-]*):(?P<Account>\d{12}):channel/(?P<ResourceId>.+?)$"), ['Channel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ivs:(?P<Region>[\w-]*):(?P<Account>\d{12}):composition/(?P<ResourceId>.+?)$"), ['Composition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ivs:(?P<Region>[\w-]*):(?P<Account>\d{12}):encoder-configuration/(?P<ResourceId>.+?)$"), ['Encoder-Configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ivs:(?P<Region>[\w-]*):(?P<Account>\d{12}):ingest-configuration/(?P<ResourceId>.+?)$"), ['Ingest-Configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ivs:(?P<Region>[\w-]*):(?P<Account>\d{12}):playback-key/(?P<ResourceId>.+?)$"), ['Playback-Key-Pair']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ivs:(?P<Region>[\w-]*):(?P<Account>\d{12}):playback-restriction-policy/(?P<ResourceId>.+?)$"), ['Playback-Restriction-Policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ivs:(?P<Region>[\w-]*):(?P<Account>\d{12}):public-key/(?P<ResourceId>.+?)$"), ['Public-Key']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ivs:(?P<Region>[\w-]*):(?P<Account>\d{12}):recording-configuration/(?P<ResourceId>.+?)$"), ['Recording-Configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ivs:(?P<Region>[\w-]*):(?P<Account>\d{12}):stage/(?P<ResourceId>.+?)$"), ['Stage']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ivs:(?P<Region>[\w-]*):(?P<Account>\d{12}):storage-configuration/(?P<ResourceId>.+?)$"), ['Storage-Configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ivs:(?P<Region>[\w-]*):(?P<Account>\d{12}):stream-key/(?P<ResourceId>.+?)$"), ['Stream-Key']),
    ],
    'ivschat': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ivschat:(?P<Region>[\w-]*):(?P<Account>\d{12}):logging-configuration/(?P<ResourceId>.+?)$"), ['Logging-Configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ivschat:(?P<Region>[\w-]*):(?P<Account>\d{12}):room/(?P<ResourceId>.+?)$"), ['Room']),
    ],
    'kafka': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kafka:(?P<Region>[\w-]*):(?P<Account>\d{12}):group/(?P<ClusterName>.+?)/(?P<ClusterUuid>.+?)/(?P<GroupName>.+?)$"), ['group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kafka:(?P<Region>[\w-]*):(?P<Account>\d{12}):topic/(?P<ClusterName>.+?)/(?P<ClusterUuid>.+?)/(?P<TopicName>.+?)$"), ['topic']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kafka:(?P<Region>[\w-]*):(?P<Account>\d{12}):transactional-id/(?P<ClusterName>.+?)/(?P<ClusterUuid>.+?)/(?P<TransactionalId>.+?)$"), ['transactional-id']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kafka:(?P<Region>[\w-]*):(?P<VpcOwnerAccount>.+?):vpc-connection/(?P<ClusterOwnerAccount>.+?)/(?P<ClusterName>.+?)/(?P<Uuid>.+?)$"), ['vpc-connection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kafka:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster/(?P<ClusterName>.+?)/(?P<ClusterUuid>.+?)$"), ['cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kafka:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster/(?P<ClusterName>.+?)/(?P<Uuid>.+?)$"), ['cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kafka:(?P<Region>[\w-]*):(?P<Account>\d{12}):configuration/(?P<ConfigurationName>.+?)/(?P<Uuid>.+?)$"), ['configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kafka:(?P<Region>[\w-]*):(?P<Account>\d{12}):replicator/(?P<ReplicatorName>.+?)/(?P<Uuid>.+?)$"), ['replicator']),
    ],
    'kafkaconnect': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kafkaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):connector-operation/(?P<ConnectorName>.+?)/(?P<ConnectorUUID>.+?)/(?P<UUID>.+?)$"), ['connector operation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kafkaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):connector/(?P<ConnectorName>.+?)/(?P<UUID>.+?)$"), ['connector']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kafkaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):custom-plugin/(?P<CustomPluginName>.+?)/(?P<UUID>.+?)$"), ['custom plugin']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kafkaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):worker-configuration/(?P<WorkerConfigurationName>.+?)/(?P<UUID>.+?)$"), ['worker configuration']),
    ],
    'kendra': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kendra:(?P<Region>[\w-]*):(?P<Account>\d{12}):index/(?P<IndexId>.+?)/access-control-configuration/(?P<AccessControlConfigurationId>.+?)$"), ['access-control-configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kendra:(?P<Region>[\w-]*):(?P<Account>\d{12}):index/(?P<IndexId>.+?)/data-source/(?P<DataSourceId>.+?)$"), ['data-source']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kendra:(?P<Region>[\w-]*):(?P<Account>\d{12}):index/(?P<IndexId>.+?)/experience/(?P<ExperienceId>.+?)$"), ['experience']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kendra:(?P<Region>[\w-]*):(?P<Account>\d{12}):index/(?P<IndexId>.+?)/faq/(?P<FaqId>.+?)$"), ['faq']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kendra:(?P<Region>[\w-]*):(?P<Account>\d{12}):index/(?P<IndexId>.+?)/featured-results-set/(?P<FeaturedResultsSetId>.+?)$"), ['featured-results-set']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kendra:(?P<Region>[\w-]*):(?P<Account>\d{12}):index/(?P<IndexId>.+?)/query-suggestions-block-list/(?P<QuerySuggestionsBlockListId>.+?)$"), ['query-suggestions-block-list']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kendra:(?P<Region>[\w-]*):(?P<Account>\d{12}):index/(?P<IndexId>.+?)/thesaurus/(?P<ThesaurusId>.+?)$"), ['thesaurus']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kendra:(?P<Region>[\w-]*):(?P<Account>\d{12}):index/(?P<IndexId>.+?)$"), ['index']),
    ],
    'kendra-ranking': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kendra-ranking:(?P<Region>[\w-]*):(?P<Account>\d{12}):rescore-execution-plan/(?P<RescoreExecutionPlanId>.+?)$"), ['rescore-execution-plan']),
    ],
    'kinesis': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kinesis:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<StreamType>.+?)/(?P<StreamName>.+?)/consumer/(?P<ConsumerName>.+?):(?P<ConsumerCreationTimpstamp>.+?)$"), ['consumer']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kinesis:(?P<Region>[\w-]*):(?P<Account>\d{12}):stream/(?P<StreamName>.+?)$"), ['stream']),
    ],
    'kinesisanalytics': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kinesisanalytics:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationName>.+?)$"), ['application']),
    ],
    'kinesisvideo': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kinesisvideo:(?P<Region>[\w-]*):(?P<Account>\d{12}):channel/(?P<ChannelName>.+?)/(?P<CreationTime>.+?)$"), ['channel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kinesisvideo:(?P<Region>[\w-]*):(?P<Account>\d{12}):stream/(?P<StreamName>.+?)/(?P<CreationTime>.+?)$"), ['stream']),
    ],
    'kms': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kms:(?P<Region>[\w-]*):(?P<Account>\d{12}):alias/(?P<Alias>.+?)$"), ['alias']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):kms:(?P<Region>[\w-]*):(?P<Account>\d{12}):key/(?P<KeyId>.+?)$"), ['key']),
    ],
    'lambda': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lambda:(?P<Region>[\w-]*):(?P<Account>\d{12}):function:(?P<FunctionName>.+?):(?P<Version>.+?)/durable-execution/(?P<ExecutionName>.+?)/(?P<ExecutionId>.+?)$"), ['durable execution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lambda:(?P<Region>[\w-]*):(?P<Account>\d{12}):function:(?P<FunctionName>.+?):(?P<Alias>.+?)$"), ['function alias']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lambda:(?P<Region>[\w-]*):(?P<Account>\d{12}):function:(?P<FunctionName>.+?):(?P<Version>.+?)$"), ['function version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lambda:(?P<Region>[\w-]*):(?P<Account>\d{12}):layer:(?P<LayerName>.+?):(?P<LayerVersion>.+?)$"), ['layerVersion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lambda:(?P<Region>[\w-]*):(?P<Account>\d{12}):capacity-provider:(?P<CapacityProviderName>.+?)$"), ['capacityProvider']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lambda:(?P<Region>[\w-]*):(?P<Account>\d{12}):code-signing-config:(?P<CodeSigningConfigId>.+?)$"), ['code signing config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lambda:(?P<Region>[\w-]*):(?P<Account>\d{12}):event-source-mapping:(?P<UUID>.+?)$"), ['eventSourceMapping']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lambda:(?P<Region>[\w-]*):(?P<Account>\d{12}):function:(?P<FunctionName>.+?)$"), ['function']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lambda:(?P<Region>[\w-]*):(?P<Account>\d{12}):layer:(?P<LayerName>.+?)$"), ['layer']),
    ],
    'launchwizard': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):launchwizard:(?P<Region>[\w-]*):(?P<Account>\d{12}):deployment/(?P<DeploymentId>.+?)$"), ['deployment']),
    ],
    'lex': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lex:(?P<Region>[\w-]*):(?P<Account>\d{12}):bot-channel:(?P<BotName>.+?):(?P<BotAlias>.+?):(?P<ChannelName>.+?)$"), ['channel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lex:(?P<Region>[\w-]*):(?P<Account>\d{12}):bot:(?P<BotName>.+?):(?P<BotVersion>.+?)$"), ['bot version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lex:(?P<Region>[\w-]*):(?P<Account>\d{12}):bot:(?P<BotName>.+?):(?P<BotAlias>.+?)$"), ['bot alias']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lex:(?P<Region>[\w-]*):(?P<Account>\d{12}):bot-alias/(?P<BotId>.+?)/(?P<BotAliasId>.+?)$"), ['bot alias']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lex:(?P<Region>[\w-]*):(?P<Account>\d{12}):intent:(?P<IntentName>.+?):(?P<IntentVersion>.+?)$"), ['intent version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lex:(?P<Region>[\w-]*):(?P<Account>\d{12}):slottype:(?P<SlotName>.+?):(?P<SlotVersion>.+?)$"), ['slottype version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lex:(?P<Region>[\w-]*):(?P<Account>\d{12}):bot:(?P<BotName>.+?)$"), ['bot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lex:(?P<Region>[\w-]*):(?P<Account>\d{12}):bot/(?P<BotId>.+?)$"), ['bot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lex:(?P<Region>[\w-]*):(?P<Account>\d{12}):test-set/(?P<TestSetId>.+?)$"), ['test set']),
    ],
    'license-manager': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):license-manager::(?P<Account>\d{12}):grant:(?P<GrantId>.+?)$"), ['grant']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):license-manager::(?P<Account>\d{12}):license:(?P<LicenseId>.+?)$"), ['license']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):license-manager:(?P<Region>[\w-]*):(?P<Account>\d{12}):license-asset-group:(?P<LicenseAssetGroupId>.+?)$"), ['license-asset-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):license-manager:(?P<Region>[\w-]*):(?P<Account>\d{12}):license-asset-ruleset:(?P<LicenseAssetRulesetId>.+?)$"), ['license-asset-ruleset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):license-manager:(?P<Region>[\w-]*):(?P<Account>\d{12}):license-configuration:(?P<LicenseConfigurationId>.+?)$"), ['license-configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):license-manager:(?P<Region>[\w-]*):(?P<Account>\d{12}):report-generator:(?P<ReportGeneratorId>.+?)$"), ['report-generator']),
    ],
    'license-manager-linux-subscriptions': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):license-manager-linux-subscriptions:(?P<Region>[\w-]*):(?P<Account>\d{12}):subscription-provider/(?P<SubscriptionProviderId>.+?)$"), ['subscription-provider']),
    ],
    'license-manager-user-subscriptions': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):license-manager-user-subscriptions:(?P<Region>[\w-]*):(?P<Account>\d{12}):identity-provider/(?P<IdentityProviderId>.+?)$"), ['identity-provider']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):license-manager-user-subscriptions:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance-user/(?P<InstanceUserId>.+?)$"), ['instance-user']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):license-manager-user-subscriptions:(?P<Region>[\w-]*):(?P<Account>\d{12}):license-server-endpoint/(?P<LicenseServerEndpointId>.+?)$"), ['license-server-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):license-manager-user-subscriptions:(?P<Region>[\w-]*):(?P<Account>\d{12}):product-subscription/(?P<ProductSubscriptionId>.+?)$"), ['product-subscription']),
    ],
    'lightsail': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):Alarm/(?P<Id>.+?)$"), ['Alarm']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):Bucket/(?P<Id>.+?)$"), ['Bucket']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):Certificate/(?P<Id>.+?)$"), ['Certificate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):CloudFormationStackRecord/(?P<Id>.+?)$"), ['CloudFormationStackRecord']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):ContactMethod/(?P<Id>.+?)$"), ['ContactMethod']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):ContainerService/(?P<Id>.+?)$"), ['ContainerService']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):Disk/(?P<Id>.+?)$"), ['Disk']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):DiskSnapshot/(?P<Id>.+?)$"), ['DiskSnapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):Distribution/(?P<Id>.+?)$"), ['Distribution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):Domain/(?P<Id>.+?)$"), ['Domain']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):ExportSnapshotRecord/(?P<Id>.+?)$"), ['ExportSnapshotRecord']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):Instance/(?P<Id>.+?)$"), ['Instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):InstanceSnapshot/(?P<Id>.+?)$"), ['InstanceSnapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):KeyPair/(?P<Id>.+?)$"), ['KeyPair']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):LoadBalancer/(?P<Id>.+?)$"), ['LoadBalancer']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):LoadBalancerTlsCertificate/(?P<Id>.+?)$"), ['LoadBalancerTlsCertificate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):RelationalDatabase/(?P<Id>.+?)$"), ['RelationalDatabase']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):RelationalDatabaseSnapshot/(?P<Id>.+?)$"), ['RelationalDatabaseSnapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lightsail:(?P<Region>[\w-]*):(?P<Account>\d{12}):StaticIp/(?P<Id>.+?)$"), ['StaticIp']),
    ],
    'logs': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):logs:(?P<Region>[\w-]*):(?P<Account>\d{12}):log-group:(?P<LogGroupName>.+?):log-stream:(?P<LogStreamName>.+?)$"), ['log-stream']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):logs:(?P<Region>[\w-]*):(?P<Account>\d{12}):anomaly-detector:(?P<DetectorId>.+?)$"), ['anomaly-detector']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):logs:(?P<Region>[\w-]*):(?P<Account>\d{12}):delivery:(?P<DeliveryName>.+?)$"), ['delivery']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):logs:(?P<Region>[\w-]*):(?P<Account>\d{12}):delivery-destination:(?P<DeliveryDestinationName>.+?)$"), ['delivery-destination']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):logs:(?P<Region>[\w-]*):(?P<Account>\d{12}):delivery-source:(?P<DeliverySourceName>.+?)$"), ['delivery-source']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):logs:(?P<Region>[\w-]*):(?P<Account>\d{12}):destination:(?P<DestinationName>.+?)$"), ['destination']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):logs:(?P<Region>[\w-]*):(?P<Account>\d{12}):log-group:(?P<LogGroupName>.+?)$"), ['log-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):logs:(?P<Region>[\w-]*):(?P<Account>\d{12}):scheduled-query:(?P<ScheduledQueryId>.+?)$"), ['scheduled-query']),
    ],
    'lookoutequipment': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lookoutequipment:(?P<Region>[\w-]*):(?P<Account>\d{12}):model/(?P<ModelName>.+?)/(?P<ModelId>.+?)/model-version/(?P<ModelVersionNumber>.+?)$"), ['model-version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lookoutequipment:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataset/(?P<DatasetName>.+?)/(?P<DatasetId>.+?)$"), ['dataset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lookoutequipment:(?P<Region>[\w-]*):(?P<Account>\d{12}):inference-scheduler/(?P<InferenceSchedulerName>.+?)/(?P<InferenceSchedulerId>.+?)$"), ['inference-scheduler']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lookoutequipment:(?P<Region>[\w-]*):(?P<Account>\d{12}):label-group/(?P<LabelGroupName>.+?)/(?P<LabelGroupId>.+?)$"), ['label-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lookoutequipment:(?P<Region>[\w-]*):(?P<Account>\d{12}):model/(?P<ModelName>.+?)/(?P<ModelId>.+?)$"), ['model']),
    ],
    'lookoutmetrics': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lookoutmetrics:(?P<Region>[\w-]*):(?P<Account>\d{12}):MetricSet/(?P<AnomalyDetectorName>.+?)/(?P<MetricSetName>.+?)$"), ['MetricSet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lookoutmetrics:(?P<Region>[\w-]*):(?P<Account>\d{12}):Alert:(?P<AlertName>.+?)$"), ['Alert']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lookoutmetrics:(?P<Region>[\w-]*):(?P<Account>\d{12}):AnomalyDetector:(?P<AnomalyDetectorName>.+?)$"), ['AnomalyDetector']),
    ],
    'lookoutvision': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lookoutvision:(?P<Region>[\w-]*):(?P<Account>\d{12}):model/(?P<ProjectName>.+?)/(?P<ModelVersion>.+?)$"), ['model']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):lookoutvision:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ProjectName>.+?)$"), ['project']),
    ],
    'm2': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):m2:(?P<Region>[\w-]*):(?P<Account>\d{12}):app/(?P<ApplicationId>.+?)$"), ['Application']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):m2:(?P<Region>[\w-]*):(?P<Account>\d{12}):env/(?P<EnvironmentId>.+?)$"), ['Environment']),
    ],
    'machinelearning': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):machinelearning:(?P<Region>[\w-]*):(?P<Account>\d{12}):batchprediction/(?P<BatchPredictionId>.+?)$"), ['batchprediction']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):machinelearning:(?P<Region>[\w-]*):(?P<Account>\d{12}):datasource/(?P<DatasourceId>.+?)$"), ['datasource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):machinelearning:(?P<Region>[\w-]*):(?P<Account>\d{12}):evaluation/(?P<EvaluationId>.+?)$"), ['evaluation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):machinelearning:(?P<Region>[\w-]*):(?P<Account>\d{12}):mlmodel/(?P<MlModelId>.+?)$"), ['mlmodel']),
    ],
    'macie2': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):macie2:(?P<Region>[\w-]*):(?P<Account>\d{12}):allow-list/(?P<ResourceId>.+?)$"), ['AllowList']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):macie2:(?P<Region>[\w-]*):(?P<Account>\d{12}):classification-job/(?P<ResourceId>.+?)$"), ['ClassificationJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):macie2:(?P<Region>[\w-]*):(?P<Account>\d{12}):custom-data-identifier/(?P<ResourceId>.+?)$"), ['CustomDataIdentifier']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):macie2:(?P<Region>[\w-]*):(?P<Account>\d{12}):findings-filter/(?P<ResourceId>.+?)$"), ['FindingsFilter']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):macie2:(?P<Region>[\w-]*):(?P<Account>\d{12}):member/(?P<ResourceId>.+?)$"), ['Member']),
    ],
    'managedblockchain': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):managedblockchain:(?P<Region>[\w-]*)::networks/(?P<NetworkId>.+?)$"), ['network']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):managedblockchain:(?P<Region>[\w-]*)::proposals/(?P<ProposalId>.+?)$"), ['proposal']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):managedblockchain:(?P<Region>[\w-]*):(?P<Account>\d{12}):accessors/(?P<AccessorId>.+?)$"), ['accessor']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):managedblockchain:(?P<Region>[\w-]*):(?P<Account>\d{12}):invitations/(?P<InvitationId>.+?)$"), ['invitation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):managedblockchain:(?P<Region>[\w-]*):(?P<Account>\d{12}):members/(?P<MemberId>.+?)$"), ['member']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):managedblockchain:(?P<Region>[\w-]*):(?P<Account>\d{12}):nodes/(?P<NodeId>.+?)$"), ['node']),
    ],
    'mapcredits': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mapcredits:::(?P<Agreement>.+?)/(?P<AgreementId>.+?)$"), ['agreement']),
    ],
    'mediaconnect': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconnect:(?P<Region>[\w-]*):offering:(?P<OfferingId>.+?)$"), ['Offering']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):gateway:(?P<GatewayId>.+?):(?P<GatewayName>.+?):instance:(?P<InstanceId>.+?)$"), ['GatewayInstance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):bridge:(?P<BridgeId>.+?):(?P<BridgeName>.+?)$"), ['Bridge']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):entitlement:(?P<FlowId>.+?):(?P<EntitlementName>.+?)$"), ['Entitlement']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):flow:(?P<FlowId>.+?):(?P<FlowName>.+?)$"), ['Flow']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):gateway:(?P<GatewayId>.+?):(?P<GatewayName>.+?)$"), ['Gateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):output:(?P<OutputId>.+?):(?P<OutputName>.+?)$"), ['Output']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):reservation:(?P<ReservationId>.+?):(?P<ReservationName>.+?)$"), ['Reservation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):source:(?P<SourceId>.+?):(?P<SourceName>.+?)$"), ['Source']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):routerInput:(?P<RouterInputId>.+?)$"), ['RouterInput']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):routerNetworkInterface:(?P<RouterNetworkInterfaceId>.+?)$"), ['RouterNetworkInterface']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconnect:(?P<Region>[\w-]*):(?P<Account>\d{12}):routerOutput:(?P<RouterOutputId>.+?)$"), ['RouterOutput']),
    ],
    'mediaconvert': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconvert:(?P<Region>[\w-]*):(?P<Account>\d{12}):certificates/(?P<CertificateArn>.+?)$"), ['CertificateAssociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconvert:(?P<Region>[\w-]*):(?P<Account>\d{12}):jobTemplates/(?P<JobTemplateName>.+?)$"), ['JobTemplate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconvert:(?P<Region>[\w-]*):(?P<Account>\d{12}):jobs/(?P<JobId>.+?)$"), ['Job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconvert:(?P<Region>[\w-]*):(?P<Account>\d{12}):presets/(?P<PresetName>.+?)$"), ['Preset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediaconvert:(?P<Region>[\w-]*):(?P<Account>\d{12}):queues/(?P<QueueName>.+?)$"), ['Queue']),
    ],
    'medialive': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):channelPlacementGroup:(?P<ClusterId>.+?)/(?P<ChannelPlacementGroupId>.+?)$"), ['channel-placement-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):node:(?P<ClusterId>.+?)/(?P<NodeId>.+?)$"), ['node']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):channel:(?P<ChannelId>.+?)$"), ['channel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):cloudwatch-alarm-template:(?P<CloudWatchAlarmTemplateId>.+?)$"), ['cloudwatch-alarm-template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):cloudwatch-alarm-template-group:(?P<CloudWatchAlarmTemplateGroupId>.+?)$"), ['cloudwatch-alarm-template-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster:(?P<ClusterId>.+?)$"), ['cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):eventbridge-rule-template:(?P<EventBridgeRuleTemplateId>.+?)$"), ['eventbridge-rule-template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):eventbridge-rule-template-group:(?P<EventBridgeRuleTemplateGroupId>.+?)$"), ['eventbridge-rule-template-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):input:(?P<InputId>.+?)$"), ['input']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):inputDevice:(?P<DeviceId>.+?)$"), ['input-device']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):inputSecurityGroup:(?P<InputSecurityGroupId>.+?)$"), ['input-security-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):multiplex:(?P<MultiplexId>.+?)$"), ['multiplex']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):network:(?P<NetworkId>.+?)$"), ['network']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):offering:(?P<OfferingId>.+?)$"), ['offering']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):reservation:(?P<ReservationId>.+?)$"), ['reservation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):sdiSource:(?P<SdiSourceId>.+?)$"), ['sdi-source']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medialive:(?P<Region>[\w-]*):(?P<Account>\d{12}):signal-map:(?P<SignalMapId>.+?)$"), ['signal-map']),
    ],
    'mediapackage': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediapackage:(?P<Region>[\w-]*):(?P<Account>\d{12}):channels/(?P<ChannelIdentifier>.+?)$"), ['channels']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediapackage:(?P<Region>[\w-]*):(?P<Account>\d{12}):harvest_jobs/(?P<HarvestJobIdentifier>.+?)$"), ['harvest_jobs']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediapackage:(?P<Region>[\w-]*):(?P<Account>\d{12}):origin_endpoints/(?P<OriginEndpointIdentifier>.+?)$"), ['origin_endpoints']),
    ],
    'mediapackage-vod': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediapackage-vod:(?P<Region>[\w-]*):(?P<Account>\d{12}):assets/(?P<AssetIdentifier>.+?)$"), ['assets']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediapackage-vod:(?P<Region>[\w-]*):(?P<Account>\d{12}):packaging-configurations/(?P<PackagingConfigurationIdentifier>.+?)$"), ['packaging-configurations']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediapackage-vod:(?P<Region>[\w-]*):(?P<Account>\d{12}):packaging-groups/(?P<PackagingGroupIdentifier>.+?)$"), ['packaging-groups']),
    ],
    'mediapackagev2': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediapackagev2:(?P<Region>[\w-]*):(?P<Account>\d{12}):channelGroup/(?P<ChannelGroupName>.+?)/channel/(?P<ChannelName>.+?)/originEndpoint/(?P<OriginEndpointName>.+?)/harvestJob/(?P<HarvestJobName>.+?)$"), ['HarvestJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediapackagev2:(?P<Region>[\w-]*):(?P<Account>\d{12}):channelGroup/(?P<ChannelGroupName>.+?)$"), ['ChannelGroup']),
    ],
    'mediastore': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediastore:(?P<Region>[\w-]*):(?P<Account>\d{12}):container/(?P<ContainerName>.+?)/(?P<ObjectPath>.+?)$"), ['object']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediastore:(?P<Region>[\w-]*):(?P<Account>\d{12}):container/(?P<ContainerName>.+?)/(?P<FolderPath>.+?)$"), ['folder']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediastore:(?P<Region>[\w-]*):(?P<Account>\d{12}):container/(?P<ContainerName>.+?)$"), ['container']),
    ],
    'mediatailor': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediatailor:(?P<Region>[\w-]*):(?P<Account>\d{12}):liveSource/(?P<SourceLocationName>.+?)/(?P<LiveSourceName>.+?)$"), ['liveSource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediatailor:(?P<Region>[\w-]*):(?P<Account>\d{12}):program/(?P<ChannelName>.+?)/(?P<ProgramName>.+?)$"), ['program']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediatailor:(?P<Region>[\w-]*):(?P<Account>\d{12}):vodSource/(?P<SourceLocationName>.+?)/(?P<VodSourceName>.+?)$"), ['vodSource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediatailor:(?P<Region>[\w-]*):(?P<Account>\d{12}):channel/(?P<ChannelName>.+?)$"), ['channel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediatailor:(?P<Region>[\w-]*):(?P<Account>\d{12}):playbackConfiguration/(?P<ResourceId>.+?)$"), ['playbackConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediatailor:(?P<Region>[\w-]*):(?P<Account>\d{12}):prefetchSchedule/(?P<ResourceId>.+?)$"), ['prefetchSchedule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mediatailor:(?P<Region>[\w-]*):(?P<Account>\d{12}):sourceLocation/(?P<SourceLocationName>.+?)$"), ['sourceLocation']),
    ],
    'medical-imaging': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medical-imaging:(?P<Region>[\w-]*):(?P<Account>\d{12}):datastore/(?P<DatastoreId>.+?)/imageset/(?P<ImageSetId>.+?)$"), ['imageset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):medical-imaging:(?P<Region>[\w-]*):(?P<Account>\d{12}):datastore/(?P<DatastoreId>.+?)$"), ['datastore']),
    ],
    'memorydb': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):memorydb::(?P<Account>\d{12}):multiregioncluster/(?P<ClusterName>.+?)$"), ['multiregioncluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):memorydb::(?P<Account>\d{12}):multiregionparametergroup/(?P<MultiRegionParameterGroupName>.+?)$"), ['multiregionparametergroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):memorydb:(?P<Region>[\w-]*):(?P<Account>\d{12}):acl/(?P<AclName>.+?)$"), ['acl']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):memorydb:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster/(?P<ClusterName>.+?)$"), ['cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):memorydb:(?P<Region>[\w-]*):(?P<Account>\d{12}):parametergroup/(?P<ParameterGroupName>.+?)$"), ['parametergroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):memorydb:(?P<Region>[\w-]*):(?P<Account>\d{12}):reservednode/(?P<ReservationID>.+?)$"), ['reservednode']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):memorydb:(?P<Region>[\w-]*):(?P<Account>\d{12}):snapshot/(?P<SnapshotName>.+?)$"), ['snapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):memorydb:(?P<Region>[\w-]*):(?P<Account>\d{12}):subnetgroup/(?P<SubnetGroupName>.+?)$"), ['subnetgroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):memorydb:(?P<Region>[\w-]*):(?P<Account>\d{12}):user/(?P<UserName>.+?)$"), ['user']),
    ],
    'mgh': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgh:(?P<Region>[\w-]*):(?P<Account>\d{12}):progressUpdateStream/(?P<Stream>.+?)/migrationTask/(?P<Task>.+?)$"), ['migrationTask']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgh:(?P<Region>[\w-]*):(?P<Account>\d{12}):automation-run/(?P<RunID>.+?)$"), ['AutomationRunResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgh:(?P<Region>[\w-]*):(?P<Account>\d{12}):automation-unit/(?P<AutomationUnitId>.+?)$"), ['AutomationUnitResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgh:(?P<Region>[\w-]*):(?P<Account>\d{12}):progressUpdateStream/(?P<Stream>.+?)$"), ['progressUpdateStream']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgh:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<ConnectionArn>.+?)$"), ['ConnectionResource']),
    ],
    'mgn': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgn:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationID>.+?)$"), ['ApplicationResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgn:(?P<Region>[\w-]*):(?P<Account>\d{12}):connector/(?P<ConnectorID>.+?)$"), ['ConnectorResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgn:(?P<Region>[\w-]*):(?P<Account>\d{12}):export/(?P<ExportID>.+?)$"), ['ExportResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgn:(?P<Region>[\w-]*):(?P<Account>\d{12}):import/(?P<ImportID>.+?)$"), ['ImportResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgn:(?P<Region>[\w-]*):(?P<Account>\d{12}):job/(?P<JobID>.+?)$"), ['JobResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgn:(?P<Region>[\w-]*):(?P<Account>\d{12}):launch-configuration-template/(?P<LaunchConfigurationTemplateID>.+?)$"), ['LaunchConfigurationTemplateResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgn:(?P<Region>[\w-]*):(?P<Account>\d{12}):network-migration-definition/(?P<NetworkMigrationDefinitionID>.+?)$"), ['NetworkMigrationDefinitionResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgn:(?P<Region>[\w-]*):(?P<Account>\d{12}):replication-configuration-template/(?P<ReplicationConfigurationTemplateID>.+?)$"), ['ReplicationConfigurationTemplateResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgn:(?P<Region>[\w-]*):(?P<Account>\d{12}):source-server/(?P<SourceServerID>.+?)$"), ['SourceServerResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgn:(?P<Region>[\w-]*):(?P<Account>\d{12}):vcenter-client/(?P<VcenterClientID>.+?)$"), ['VcenterClientResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mgn:(?P<Region>[\w-]*):(?P<Account>\d{12}):wave/(?P<WaveID>.+?)$"), ['WaveResource']),
    ],
    'migrationhub-orchestrator': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):migrationhub-orchestrator:(?P<Region>[\w-]*):(?P<Account>\d{12}):template/(?P<ResourceId>.+?)$"), ['template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):migrationhub-orchestrator:(?P<Region>[\w-]*):(?P<Account>\d{12}):workflow/(?P<ResourceId>.+?)$"), ['workflow']),
    ],
    'mobiletargeting': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/campaigns/(?P<CampaignId>.+?)/kpis/daterange/(?P<KpiName>.+?)$"), ['campaign-metrics']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/journeys/(?P<JourneyId>.+?)/activities/(?P<JourneyActivityId>.+?)/execution-metrics$"), ['journey-execution-activity-metrics']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/journeys/(?P<JourneyId>.+?)/kpis/daterange/(?P<KpiName>.+?)$"), ['journey-metrics']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/jobs/export/(?P<JobId>.+?)$"), ['export-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/jobs/import/(?P<JobId>.+?)$"), ['import-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/journeys/(?P<JourneyId>.+?)/execution-metrics$"), ['journey-execution-metrics']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/kpis/daterange/(?P<KpiName>.+?)$"), ['application-metrics']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/attributes/(?P<AttributeType>.+?)$"), ['attribute']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/campaigns/(?P<CampaignId>.+?)$"), ['campaign']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/channels/(?P<ChannelType>.+?)$"), ['channel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/endpoints/(?P<EndpointId>.+?)$"), ['endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/journeys/(?P<JourneyId>.+?)$"), ['journey']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/segments/(?P<SegmentId>.+?)$"), ['segment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/users/(?P<UserId>.+?)$"), ['user']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/channels$"), ['channels']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/events$"), ['events']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/eventstream$"), ['event-stream']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/journeys$"), ['journeys']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/messages$"), ['messages']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/otp$"), ['otp']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)/verify-otp$"), ['verify-otp']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):phone/number/validate$"), ['phone-number-validate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):templates/(?P<TemplateName>.+?)/(?P<TemplateType>.+?)$"), ['template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):apps/(?P<AppId>.+?)$"), ['app']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):recommenders/(?P<RecommenderId>.+?)$"), ['recommender']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):reports$"), ['reports']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mobiletargeting:(?P<Region>[\w-]*):(?P<Account>\d{12}):templates$"), ['templates']),
    ],
    'monitron': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):monitron:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ResourceId>.+?)$"), ['project']),
    ],
    'mpa': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mpa:(?P<Region>[\w-]*):(?P<Account>\d{12}):approval-team/(?P<ApprovalTeamId>.+?)$"), ['approval-team']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mpa:(?P<Region>[\w-]*):(?P<Account>\d{12}):identity-source/(?P<IdentitySourceId>.+?)$"), ['identity-source']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mpa:(?P<Region>[\w-]*):(?P<Account>\d{12}):session/(?P<SessionId>.+?)$"), ['session']),
    ],
    'mq': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mq:(?P<Region>[\w-]*):(?P<Account>\d{12}):broker:(?P<BrokerName>.+?):(?P<BrokerId>.+?)$"), ['brokers']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):mq:(?P<Region>[\w-]*):(?P<Account>\d{12}):configuration:(?P<ConfigurationId>.+?)$"), ['configurations', 'configuration']),
    ],
    'neptune-db': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):neptune-db:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<ClusterResourceId>.+?)/(?P<DatabaseId>.+?)$"), ['database']),
    ],
    'neptune-graph': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):neptune-graph:(?P<Region>[\w-]*):(?P<Account>\d{12}):export-task/(?P<ResourceId>.+?)$"), ['export-task']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):neptune-graph:(?P<Region>[\w-]*):(?P<Account>\d{12}):graph/(?P<ResourceId>.+?)$"), ['graph']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):neptune-graph:(?P<Region>[\w-]*):(?P<Account>\d{12}):graph-snapshot/(?P<ResourceId>.+?)$"), ['graph-snapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):neptune-graph:(?P<Region>[\w-]*):(?P<Account>\d{12}):import-task/(?P<ResourceId>.+?)$"), ['import-task']),
    ],
    'network-firewall': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):network-firewall:(?P<Region>[\w-]*):(?P<Account>\d{12}):firewall/(?P<Name>.+?)$"), ['Firewall']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):network-firewall:(?P<Region>[\w-]*):(?P<Account>\d{12}):firewall-policy/(?P<Name>.+?)$"), ['FirewallPolicy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):network-firewall:(?P<Region>[\w-]*):(?P<Account>\d{12}):proxy/(?P<Name>.+?)$"), ['Proxy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):network-firewall:(?P<Region>[\w-]*):(?P<Account>\d{12}):proxy-configuration/(?P<Name>.+?)$"), ['ProxyConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):network-firewall:(?P<Region>[\w-]*):(?P<Account>\d{12}):proxy-rule-group/(?P<Name>.+?)$"), ['ProxyRuleGroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):network-firewall:(?P<Region>[\w-]*):(?P<Account>\d{12}):stateful-rulegroup/(?P<Name>.+?)$"), ['StatefulRuleGroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):network-firewall:(?P<Region>[\w-]*):(?P<Account>\d{12}):stateless-rulegroup/(?P<Name>.+?)$"), ['StatelessRuleGroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):network-firewall:(?P<Region>[\w-]*):(?P<Account>\d{12}):tls-configuration/(?P<Name>.+?)$"), ['TLSInspectionConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):network-firewall:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpc-endpoint-association/(?P<Name>.+?)$"), ['VpcEndpointAssociation']),
    ],
    'networkflowmonitor': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):networkflowmonitor:(?P<Region>[\w-]*):(?P<Account>\d{12}):monitor/(?P<MonitorName>.+?)$"), ['monitor']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):networkflowmonitor:(?P<Region>[\w-]*):(?P<Account>\d{12}):scope/(?P<ScopeId>.+?)$"), ['scope']),
    ],
    'networkmanager': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):networkmanager::(?P<Account>\d{12}):connection/(?P<GlobalNetworkId>.+?)/(?P<ResourceId>.+?)$"), ['connection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):networkmanager::(?P<Account>\d{12}):device/(?P<GlobalNetworkId>.+?)/(?P<ResourceId>.+?)$"), ['device']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):networkmanager::(?P<Account>\d{12}):link/(?P<GlobalNetworkId>.+?)/(?P<ResourceId>.+?)$"), ['link']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):networkmanager::(?P<Account>\d{12}):site/(?P<GlobalNetworkId>.+?)/(?P<ResourceId>.+?)$"), ['site']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):networkmanager::(?P<Account>\d{12}):attachment/(?P<ResourceId>.+?)$"), ['attachment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):networkmanager::(?P<Account>\d{12}):connect-peer/(?P<ResourceId>.+?)$"), ['connect-peer']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):networkmanager::(?P<Account>\d{12}):core-network/(?P<ResourceId>.+?)$"), ['core-network']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):networkmanager::(?P<Account>\d{12}):global-network/(?P<ResourceId>.+?)$"), ['global-network']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):networkmanager::(?P<Account>\d{12}):peering/(?P<ResourceId>.+?)$"), ['peering']),
    ],
    'networkmonitor': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):networkmonitor:(?P<Region>[\w-]*):(?P<Account>\d{12}):monitor/(?P<MonitorName>.+?)$"), ['monitor']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):networkmonitor:(?P<Region>[\w-]*):(?P<Account>\d{12}):probe/(?P<ProbeId>.+?)$"), ['probe']),
    ],
    'nimble': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):nimble:(?P<Region>[\w-]*):(?P<Account>\d{12}):eula/(?P<EulaId>.+?)$"), ['eula']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):nimble:(?P<Region>[\w-]*):(?P<Account>\d{12}):eula-acceptance/(?P<EulaAcceptanceId>.+?)$"), ['eula-acceptance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):nimble:(?P<Region>[\w-]*):(?P<Account>\d{12}):launch-profile/(?P<LaunchProfileId>.+?)$"), ['launch-profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):nimble:(?P<Region>[\w-]*):(?P<Account>\d{12}):streaming-image/(?P<StreamingImageId>.+?)$"), ['streaming-image']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):nimble:(?P<Region>[\w-]*):(?P<Account>\d{12}):streaming-session/(?P<StreamingSessionId>.+?)$"), ['streaming-session']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):nimble:(?P<Region>[\w-]*):(?P<Account>\d{12}):streaming-session-backup/(?P<StreamingSessionBackupId>.+?)$"), ['streaming-session-backup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):nimble:(?P<Region>[\w-]*):(?P<Account>\d{12}):studio/(?P<StudioId>.+?)$"), ['studio']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):nimble:(?P<Region>[\w-]*):(?P<Account>\d{12}):studio-component/(?P<StudioComponentId>.+?)$"), ['studio-component']),
    ],
    'notifications': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):notifications::(?P<Account>\d{12}):managed-notification-configuration/category/(?P<Category>.+?)/sub-category/(?P<Subcategory>.+?)/event/(?P<NotificationEventId>.+?)/child-event/(?P<NotificationChildEventId>.+?)$"), ['ManagedNotificationChildEvent']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):notifications::(?P<Account>\d{12}):managed-notification-configuration/category/(?P<Category>.+?)/sub-category/(?P<Subcategory>.+?)/event/(?P<NotificationEventId>.+?)$"), ['ManagedNotificationEvent']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):notifications::(?P<Account>\d{12}):managed-notification-configuration/category/(?P<Category>.+?)/sub-category/(?P<Subcategory>.+?)$"), ['ManagedNotificationConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):notifications::(?P<Account>\d{12}):configuration/(?P<NotificationConfigurationId>.+?)/rule/(?P<EventRuleId>.+?)$"), ['EventRule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):notifications::(?P<Account>\d{12}):configuration/(?P<NotificationConfigurationId>.+?)$"), ['NotificationConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):notifications:(?P<Region>[\w-]*):(?P<Account>\d{12}):configuration/(?P<NotificationConfigurationId>.+?)/event/(?P<NotificationEventId>.+?)$"), ['NotificationEvent']),
    ],
    'notifications-contacts': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):notifications-contacts::(?P<Account>\d{12}):emailcontact/(?P<EmailContactId>.+?)$"), ['EmailContactResource']),
    ],
    'nova-act': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):nova-act:(?P<Region>[\w-]*):(?P<Account>\d{12}):workflow-definition/(?P<WorkflowDefinitionName>.+?)/workflow-run/(?P<WorkflowRunId>.+?)$"), ['workflow-run']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):nova-act:(?P<Region>[\w-]*):(?P<Account>\d{12}):workflow-definition/(?P<WorkflowDefinitionName>.+?)$"), ['workflow-definition']),
    ],
    'oam': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):oam:(?P<Region>[\w-]*):(?P<Account>\d{12}):link/(?P<ResourceId>.+?)$"), ['Link']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):oam:(?P<Region>[\w-]*):(?P<Account>\d{12}):sink/(?P<ResourceId>.+?)$"), ['Sink']),
    ],
    'observabilityadmin': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):observabilityadmin:(?P<Region>[\w-]*):(?P<Account>\d{12}):organization-centralization-rule/(?P<CentralizationRuleName>.+?)$"), ['organization-centralization-rule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):observabilityadmin:(?P<Region>[\w-]*):(?P<Account>\d{12}):organization-telemetry-rule/(?P<TelemetryRuleName>.+?)$"), ['organization-telemetry-rule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):observabilityadmin:(?P<Region>[\w-]*):(?P<Account>\d{12}):s3tableintegration/(?P<S3TableIntegrationIdentifier>.+?)$"), ['s3tableintegration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):observabilityadmin:(?P<Region>[\w-]*):(?P<Account>\d{12}):telemetry-pipeline/(?P<TelemetryPipelineIdentifier>.+?)$"), ['telemetry-pipeline']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):observabilityadmin:(?P<Region>[\w-]*):(?P<Account>\d{12}):telemetry-rule/(?P<TelemetryRuleName>.+?)$"), ['telemetry-rule']),
    ],
    'odb': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):odb:(?P<Region>[\w-]*):(?P<Account>\d{12}):cloud-autonomous-vm-cluster/(?P<CloudAutonomousVmClusterId>.+?)$"), ['cloud-autonomous-vm-cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):odb:(?P<Region>[\w-]*):(?P<Account>\d{12}):cloud-exadata-infrastructure/(?P<CloudExadataInfrastructureId>.+?)$"), ['cloud-exadata-infrastructure']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):odb:(?P<Region>[\w-]*):(?P<Account>\d{12}):cloud-vm-cluster/(?P<CloudVmClusterId>.+?)$"), ['cloud-vm-cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):odb:(?P<Region>[\w-]*):(?P<Account>\d{12}):db-node/(?P<DbNodeId>.+?)$"), ['db-node']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):odb:(?P<Region>[\w-]*):(?P<Account>\d{12}):odb-network/(?P<OdbNetworkId>.+?)$"), ['odb-network']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):odb:(?P<Region>[\w-]*):(?P<Account>\d{12}):odb-peering-connection/(?P<OdbPeeringConnectionId>.+?)$"), ['odb-peering-connection']),
    ],
    'omics': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):omics:(?P<Region>[\w-]*):(?P<Account>\d{12}):annotationStore/(?P<AnnotationStoreName>.+?)/version/(?P<AnnotationStoreVersionName>.+?)$"), ['AnnotationStoreVersion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):omics:(?P<Region>[\w-]*):(?P<Account>\d{12}):referenceStore/(?P<ReferenceStoreId>.+?)/reference/(?P<ReferenceId>.+?)$"), ['reference']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):omics:(?P<Region>[\w-]*):(?P<Account>\d{12}):sequenceStore/(?P<SequenceStoreId>.+?)/readSet/(?P<ReadSetId>.+?)$"), ['readSet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):omics:(?P<Region>[\w-]*):(?P<Account>\d{12}):workflow/(?P<Id>.+?)/version/(?P<VersionName>.+?)$"), ['WorkflowVersion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):omics:(?P<Region>[\w-]*):(?P<Account>\d{12}):annotationStore/(?P<AnnotationStoreName>.+?)$"), ['AnnotationStore']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):omics:(?P<Region>[\w-]*):(?P<Account>\d{12}):referenceStore/(?P<ReferenceStoreId>.+?)$"), ['referenceStore']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):omics:(?P<Region>[\w-]*):(?P<Account>\d{12}):run/(?P<Id>.+?)$"), ['run']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):omics:(?P<Region>[\w-]*):(?P<Account>\d{12}):runCache/(?P<Id>.+?)$"), ['runCache']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):omics:(?P<Region>[\w-]*):(?P<Account>\d{12}):runGroup/(?P<Id>.+?)$"), ['runGroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):omics:(?P<Region>[\w-]*):(?P<Account>\d{12}):sequenceStore/(?P<SequenceStoreId>.+?)$"), ['sequenceStore']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):omics:(?P<Region>[\w-]*):(?P<Account>\d{12}):task/(?P<Id>.+?)$"), ['TaskResource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):omics:(?P<Region>[\w-]*):(?P<Account>\d{12}):variantStore/(?P<VariantStoreName>.+?)$"), ['VariantStore']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):omics:(?P<Region>[\w-]*):(?P<Account>\d{12}):workflow/(?P<Id>.+?)$"), ['workflow']),
    ],
    'one': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):one:(?P<Region>[\w-]*):(?P<Account>\d{12}):device-instance/(?P<DeviceInstanceId>.+?)/configuration/(?P<Version>.+?)$"), ['configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):one:(?P<Region>[\w-]*):(?P<Account>\d{12}):device-configuration-template/(?P<TemplateId>.+?)$"), ['device-configuration-template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):one:(?P<Region>[\w-]*):(?P<Account>\d{12}):device-instance/(?P<DeviceInstanceId>.+?)$"), ['device-instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):one:(?P<Region>[\w-]*):(?P<Account>\d{12}):site/(?P<SiteId>.+?)$"), ['site']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):one:(?P<Region>[\w-]*):(?P<Account>\d{12}):user/(?P<UserId>.+?)$"), ['user']),
    ],
    'opensearch': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):opensearch:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<AppId>.+?)$"), ['application']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):opensearch:(?P<Region>[\w-]*):(?P<Account>\d{12}):datasource/(?P<DataSourceName>.+?)$"), ['datasource']),
    ],
    'opsworks': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):opsworks:(?P<Region>[\w-]*):(?P<Account>\d{12}):stack/(?P<StackId>.+?)/$"), ['stack']),
    ],
    'opsworks-cm': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):opsworks-cm::(?P<Account>\d{12}):backup/(?P<ServerName>.+?)-\{Date-and-Time-Stamp-of-Backup\}$"), ['backup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):opsworks-cm::(?P<Account>\d{12}):server/(?P<ServerName>.+?)/(?P<UniqueId>.+?)$"), ['server']),
    ],
    'organizations': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):organizations::aws:policy/(?P<PolicyType>.+?)/p-(?P<PolicyId>.+?)$"), ['awspolicy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):organizations::(?P<Account>\d{12}):transfer/o-(?P<OrganizationId>.+?)/(?P<TransferType>.+?)/(?P<TransferDirection>.+?)/rt-(?P<ResponsibilityTransferId>.+?)$"), ['responsibilitytransfer']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):organizations::(?P<Account>\d{12}):handshake/o-(?P<OrganizationId>.+?)/(?P<HandshakeType>.+?)/h-(?P<HandshakeId>.+?)$"), ['handshake']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):organizations::(?P<Account>\d{12}):policy/o-(?P<OrganizationId>.+?)/(?P<PolicyType>.+?)/p-(?P<PolicyId>.+?)$"), ['policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):organizations::(?P<Account>\d{12}):ou/o-(?P<OrganizationId>.+?)/ou-(?P<OrganizationalUnitId>.+?)$"), ['organizationalunit']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):organizations::(?P<Account>\d{12}):resourcepolicy/o-(?P<OrganizationId>.+?)/rp-(?P<ResourcePolicyId>.+?)$"), ['resourcepolicy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):organizations::(?P<Account>\d{12}):root/o-(?P<OrganizationId>.+?)/r-(?P<RootId>.+?)$"), ['root']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):organizations::(?P<Account>\d{12}):account/o-(?P<OrganizationId>.+?)/(?P<AccountId>.+?)$"), ['account']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):organizations::(?P<Account>\d{12}):organization/o-(?P<OrganizationId>.+?)$"), ['organization']),
    ],
    'osis': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):osis:(?P<Region>[\w-]*):(?P<Account>\d{12}):blueprint/(?P<BlueprintName>.+?)$"), ['pipeline-blueprint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):osis:(?P<Region>[\w-]*):(?P<Account>\d{12}):endpoint/(?P<EndpointId>.+?)$"), ['pipeline-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):osis:(?P<Region>[\w-]*):(?P<Account>\d{12}):pipeline/(?P<PipelineName>.+?)$"), ['pipeline']),
    ],
    'outposts': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):outposts:(?P<Region>[\w-]*):(?P<Account>\d{12}):outpost/(?P<OutpostId>.+?)$"), ['outpost']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):outposts:(?P<Region>[\w-]*):(?P<Account>\d{12}):site/(?P<SiteId>.+?)$"), ['site']),
    ],
    'panorama': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):panorama:(?P<Region>[\w-]*):(?P<Account>\d{12}):applicationInstance/(?P<ApplicationInstanceId>.+?)$"), ['applicationInstance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):panorama:(?P<Region>[\w-]*):(?P<Account>\d{12}):device/(?P<DeviceId>.+?)$"), ['device']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):panorama:(?P<Region>[\w-]*):(?P<Account>\d{12}):package/(?P<PackageId>.+?)$"), ['package']),
    ],
    'partnercentral': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral::(?P<Account>\d{12}):catalog/(?P<Catalog>.+?)/ReportingData/(?P<TableId>.+?)/Dashboard/(?P<DashboardId>.+?)$"), ['Dashboard']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*)::catalog/(?P<Catalog>.+?)/benefit/(?P<Identifier>.+?)$"), ['Benefit']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*)::catalog/(?P<Catalog>.+?)/connection/(?P<Identifier>.+?)$"), ['Connection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*)::catalog/(?P<Catalog>.+?)/connection-invitation/(?P<Identifier>.+?)$"), ['ConnectionInvitation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*)::catalog/(?P<Catalog>.+?)/engagement/(?P<Identifier>.+?)$"), ['Engagement']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*)::catalog/(?P<Catalog>.+?)/engagement-by-accepting-invitation-task/(?P<TaskId>.+?)$"), ['engagement-by-accepting-invitation-task']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*)::catalog/(?P<Catalog>.+?)/engagement-from-opportunity-task/(?P<TaskId>.+?)$"), ['engagement-from-opportunity-task']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*)::catalog/(?P<Catalog>.+?)/engagement-invitation/(?P<Identifier>.+?)$"), ['engagement-invitation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*)::catalog/(?P<Catalog>.+?)/opportunity-from-engagement-task/(?P<TaskId>.+?)$"), ['OpportunityFromEngagementTask']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*):(?P<Account>\d{12}):catalog/(?P<Catalog>.+?)/engagement/(?P<EngagementIdentifier>.+?)/resource/(?P<ResourceType>.+?)/(?P<ResourceIdentifier>.+?)/template/(?P<TemplateIdentifier>.+?)/resource-snapshot/(?P<SnapshotRevision>.+?)$"), ['ResourceSnapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*):(?P<Account>\d{12}):catalog/(?P<Catalog>.+?)/program-management-account/(?P<ProgramManagementAccountId>.+?)/relationship/(?P<RelationshipId>.+?)$"), ['Relationship']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*):(?P<Account>\d{12}):catalog/(?P<Catalog>.+?)/benefit-allocation/(?P<Identifier>.+?)$"), ['BenefitAllocation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*):(?P<Account>\d{12}):catalog/(?P<Catalog>.+?)/benefit-application/(?P<Identifier>.+?)$"), ['BenefitApplication']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*):(?P<Account>\d{12}):catalog/(?P<Catalog>.+?)/channel-handshake/(?P<Identifier>.+?)$"), ['ChannelHandshake']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*):(?P<Account>\d{12}):catalog/(?P<Catalog>.+?)/opportunity/(?P<Identifier>.+?)$"), ['Opportunity']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*):(?P<Account>\d{12}):catalog/(?P<Catalog>.+?)/partner/(?P<Identifier>.+?)$"), ['Partner']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*):(?P<Account>\d{12}):catalog/(?P<Catalog>.+?)/program-management-account/(?P<Identifier>.+?)$"), ['ProgramManagementAccount']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*):(?P<Account>\d{12}):catalog/(?P<Catalog>.+?)/resource-snapshot-job/(?P<Identifier>.+?)$"), ['resource-snapshot-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*):(?P<Account>\d{12}):catalog/(?P<Catalog>.+?)/solution/(?P<Identifier>.+?)$"), ['Solution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):partnercentral:(?P<Region>[\w-]*):(?P<Account>\d{12}):catalog/(?P<Catalog>.+?)/connection-preferences$"), ['ConnectionPreferences']),
    ],
    'payment-cryptography': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):payment-cryptography:(?P<Region>[\w-]*):(?P<Account>\d{12}):alias/(?P<Alias>.+?)$"), ['alias']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):payment-cryptography:(?P<Region>[\w-]*):(?P<Account>\d{12}):key/(?P<KeyId>.+?)$"), ['key']),
    ],
    'payments': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):payments::(?P<Account>\d{12}):payment-instrument:(?P<ResourceId>.+?)$"), ['payment-instrument']),
    ],
    'pca-connector-ad': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):pca-connector-ad:(?P<Region>[\w-]*):(?P<Account>\d{12}):connector/(?P<ConnectorId>.+?)/template/(?P<TemplateId>.+?)$"), ['Template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):pca-connector-ad:(?P<Region>[\w-]*):(?P<Account>\d{12}):connector/(?P<ConnectorId>.+?)$"), ['Connector']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):pca-connector-ad:(?P<Region>[\w-]*):(?P<Account>\d{12}):directory-registration/(?P<DirectoryId>.+?)$"), ['DirectoryRegistration']),
    ],
    'pca-connector-scep': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):pca-connector-scep:(?P<Region>[\w-]*):(?P<Account>\d{12}):connector/(?P<ConnectorId>.+?)/challenge/(?P<ChallengeId>.+?)$"), ['Challenge']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):pca-connector-scep:(?P<Region>[\w-]*):(?P<Account>\d{12}):connector/(?P<ConnectorId>.+?)$"), ['Connector']),
    ],
    'pcs': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):pcs:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster/(?P<ClusterIdentifier>.+?)/computenodegroup/(?P<ComputeNodeGroupIdentifier>.+?)$"), ['computenodegroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):pcs:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster/(?P<ClusterIdentifier>.+?)/queue/(?P<QueueIdentifier>.+?)$"), ['queue']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):pcs:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster/(?P<ClusterIdentifier>.+?)$"), ['cluster']),
    ],
    'personalize': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:::algorithm/(?P<ResourceId>.+?)$"), ['algorithm']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:::feature-transformation/(?P<ResourceId>.+?)$"), ['featureTransformation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:::recipe/(?P<ResourceId>.+?)$"), ['recipe']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):batch-inference-job/(?P<ResourceId>.+?)$"), ['batchInferenceJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):batch-segment-job/(?P<ResourceId>.+?)$"), ['batchSegmentJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):campaign/(?P<ResourceId>.+?)$"), ['campaign']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-deletion-job/(?P<ResourceId>.+?)$"), ['dataDeletionJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-insights-job/(?P<ResourceId>.+?)$"), ['dataInsightsJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataset/(?P<ResourceId>.+?)$"), ['dataset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataset-export-job/(?P<ResourceId>.+?)$"), ['datasetExportJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataset-group/(?P<ResourceId>.+?)$"), ['datasetGroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataset-import-job/(?P<ResourceId>.+?)$"), ['datasetImportJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):event-tracker/(?P<ResourceId>.+?)$"), ['eventTracker']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):filter/(?P<ResourceId>.+?)$"), ['filter']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):metric-attribution/(?P<ResourceId>.+?)$"), ['metricAttribution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):recommender/(?P<ResourceId>.+?)$"), ['recommender']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):schema/(?P<ResourceId>.+?)$"), ['schema']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):personalize:(?P<Region>[\w-]*):(?P<Account>\d{12}):solution/(?P<ResourceId>.+?)$"), ['solution']),
    ],
    'pi': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):pi:(?P<Region>[\w-]*):(?P<Account>\d{12}):perf-reports/(?P<ServiceType>.+?)/(?P<Identifier>.+?)/(?P<ReportId>.+?)$"), ['perf-reports-resource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):pi:(?P<Region>[\w-]*):(?P<Account>\d{12}):metrics/(?P<ServiceType>.+?)/(?P<Identifier>.+?)$"), ['metric-resource']),
    ],
    'pipes': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):pipes:(?P<Region>[\w-]*):(?P<Account>\d{12}):pipe/(?P<Name>.+?)$"), ['pipe']),
    ],
    'polly': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):polly:(?P<Region>[\w-]*):(?P<Account>\d{12}):lexicon/(?P<LexiconName>.+?)$"), ['lexicon']),
    ],
    'pricingplanmanager': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):pricingplanmanager::(?P<Account>\d{12}):subscription/(?P<SubscriptionId>.+?)$"), ['subscription']),
    ],
    'private-networks': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):private-networks:(?P<Region>[\w-]*):(?P<Account>\d{12}):device-identifier/(?P<NetworkName>.+?)/(?P<DeviceId>.+?)$"), ['device-identifier']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):private-networks:(?P<Region>[\w-]*):(?P<Account>\d{12}):network-resource/(?P<NetworkName>.+?)/(?P<ResourceId>.+?)$"), ['network-resource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):private-networks:(?P<Region>[\w-]*):(?P<Account>\d{12}):network-site/(?P<NetworkName>.+?)/(?P<NetworkSiteName>.+?)$"), ['network-site']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):private-networks:(?P<Region>[\w-]*):(?P<Account>\d{12}):order/(?P<NetworkName>.+?)/(?P<OrderId>.+?)$"), ['order']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):private-networks:(?P<Region>[\w-]*):(?P<Account>\d{12}):network/(?P<NetworkName>.+?)$"), ['network']),
    ],
    'profile': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):profile:(?P<Region>[\w-]*):(?P<Account>\d{12}):domains/(?P<DomainName>.+?)/calculated-attributes/(?P<CalculatedAttributeName>.+?)$"), ['calculated-attributes']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):profile:(?P<Region>[\w-]*):(?P<Account>\d{12}):domains/(?P<DomainName>.+?)/domain-object-types/(?P<ObjectTypeName>.+?)$"), ['domain-object-types']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):profile:(?P<Region>[\w-]*):(?P<Account>\d{12}):domains/(?P<DomainName>.+?)/event-streams/(?P<EventStreamName>.+?)$"), ['event-streams']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):profile:(?P<Region>[\w-]*):(?P<Account>\d{12}):domains/(?P<DomainName>.+?)/event-triggers/(?P<EventTriggerName>.+?)$"), ['event-triggers']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):profile:(?P<Region>[\w-]*):(?P<Account>\d{12}):domains/(?P<DomainName>.+?)/integrations/(?P<Uri>.+?)$"), ['integrations']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):profile:(?P<Region>[\w-]*):(?P<Account>\d{12}):domains/(?P<DomainName>.+?)/layouts/(?P<LayoutDefinitionName>.+?)$"), ['layouts']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):profile:(?P<Region>[\w-]*):(?P<Account>\d{12}):domains/(?P<DomainName>.+?)/object-types/(?P<ObjectTypeName>.+?)$"), ['object-types']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):profile:(?P<Region>[\w-]*):(?P<Account>\d{12}):domains/(?P<DomainName>.+?)/recommenders/(?P<RecommenderTypeName>.+?)$"), ['recommenders']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):profile:(?P<Region>[\w-]*):(?P<Account>\d{12}):domains/(?P<DomainName>.+?)/segment-definitions/(?P<SegmentDefinitionName>.+?)$"), ['segment-definitions']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):profile:(?P<Region>[\w-]*):(?P<Account>\d{12}):domains/(?P<DomainName>.+?)$"), ['domains']),
    ],
    'proton': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment-template/(?P<TemplateName>.+?):(?P<MajorVersion>.+?)\.(?P<MinorVersion>.+?)$"), ['environment-template-version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment-template/(?P<TemplateName>.+?):(?P<MajorVersionId>.+?)\.(?P<MinorVersionId>.+?)$"), ['environment-template-minor-version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):service-template/(?P<TemplateName>.+?):(?P<MajorVersion>.+?)\.(?P<MinorVersion>.+?)$"), ['service-template-version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):service-template/(?P<TemplateName>.+?):(?P<MajorVersionId>.+?)\.(?P<MinorVersionId>.+?)$"), ['service-template-minor-version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):service/(?P<ServiceName>.+?)/service-instance/(?P<Name>.+?)$"), ['service-instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment-template/(?P<TemplateName>.+?):(?P<MajorVersionId>.+?)$"), ['environment-template-major-version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):repository/(?P<Provider>.+?):(?P<Name>.+?)$"), ['repository']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):service-template/(?P<TemplateName>.+?):(?P<MajorVersionId>.+?)$"), ['service-template-major-version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):component/(?P<Id>.+?)$"), ['component']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):deployment/(?P<Id>.+?)$"), ['deployment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment/(?P<Name>.+?)$"), ['environment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment-account-connection/(?P<Id>.+?)$"), ['environment-account-connection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment-template/(?P<Name>.+?)$"), ['environment-template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):service/(?P<Name>.+?)$"), ['service']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):proton:(?P<Region>[\w-]*):(?P<Account>\d{12}):service-template/(?P<Name>.+?)$"), ['service-template']),
    ],
    'purchase-orders': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):purchase-orders::(?P<Account>\d{12}):purchase-order/(?P<ResourceName>.+?)$"), ['purchase-order']),
    ],
    'qapps': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qapps:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/qapp/(?P<AppId>.+?)/session/(?P<SessionId>.+?)$"), ['qapp-session']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qapps:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/qapp/(?P<AppId>.+?)$"), ['qapp']),
    ],
    'qbusiness': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qbusiness:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/index/(?P<IndexId>.+?)/data-source/(?P<DataSourceId>.+?)$"), ['data-source']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qbusiness:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/chat-response-configuration/(?P<ChatResponseConfigurationId>.+?)$"), ['chat-response-configuration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qbusiness:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/data-accessor/(?P<DataAccessorId>.+?)$"), ['data-accessor']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qbusiness:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/index/(?P<IndexId>.+?)$"), ['index']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qbusiness:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/integration/(?P<IntegrationId>.+?)$"), ['integration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qbusiness:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/plugin/(?P<PluginId>.+?)$"), ['plugin']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qbusiness:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/retriever/(?P<RetrieverId>.+?)$"), ['retriever']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qbusiness:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/subscription/(?P<SubscriptionId>.+?)$"), ['subscription']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qbusiness:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)/web-experience/(?P<WebExperienceId>.+?)$"), ['web-experience']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qbusiness:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)$"), ['application']),
    ],
    'qdeveloper': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qdeveloper:(?P<Region>[\w-]*):(?P<Account>\d{12}):codeTransformation/(?P<Identifier>.+?)$"), ['codeTransformation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qdeveloper:(?P<Region>[\w-]*):(?P<Account>\d{12}):plugin/(?P<Identifier>.+?)$"), ['plugin']),
    ],
    'qldb': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qldb:(?P<Region>[\w-]*):(?P<Account>\d{12}):ledger/(?P<LedgerName>.+?)/information_schema/user_tables$"), ['catalog']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qldb:(?P<Region>[\w-]*):(?P<Account>\d{12}):ledger/(?P<LedgerName>.+?)/table/(?P<TableId>.+?)$"), ['table']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qldb:(?P<Region>[\w-]*):(?P<Account>\d{12}):stream/(?P<LedgerName>.+?)/(?P<StreamId>.+?)$"), ['stream']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):qldb:(?P<Region>[\w-]*):(?P<Account>\d{12}):ledger/(?P<LedgerName>.+?)$"), ['ledger']),
    ],
    'quicksight': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight::(?P<Account>\d{12}):assignment/(?P<ResourceId>.+?)$"), ['assignment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):dashboard/(?P<DashboardId>.+?)/snapshot-job/(?P<ResourceId>.+?)$"), ['dashboardSnapshotJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataset/(?P<DatasetId>.+?)/ingestion/(?P<ResourceId>.+?)$"), ['ingestion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataset/(?P<DatasetId>.+?)/refresh-schedule/(?P<ResourceId>.+?)$"), ['refreshschedule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):account/(?P<ResourceId>.+?)$"), ['account']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):action-connector/(?P<ResourceId>.+?)$"), ['actionconnector']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):agent/(?P<ResourceId>.+?)$"), ['agent']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):analysis/(?P<ResourceId>.+?)$"), ['analysis']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):asset-bundle-export-job/(?P<ResourceId>.+?)$"), ['assetBundleExportJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):asset-bundle-import-job/(?P<ResourceId>.+?)$"), ['assetBundleImportJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):brand/(?P<ResourceId>.+?)$"), ['brand']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):customization/(?P<ResourceId>.+?)$"), ['customization']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):custompermissions/(?P<ResourceId>.+?)$"), ['custompermissions']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):dashboard/(?P<ResourceId>.+?)$"), ['dashboard']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataset/(?P<ResourceId>.+?)$"), ['dataset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):datasource/(?P<ResourceId>.+?)$"), ['datasource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):email-customization-template/(?P<ResourceId>.+?)$"), ['emailCustomizationTemplate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):extension-access/(?P<ResourceId>.+?)$"), ['extensionaccess']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):flow/(?P<ResourceId>.+?)$"), ['flow']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):folder/(?P<ResourceId>.+?)$"), ['folder']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):group/(?P<ResourceId>.+?)$"), ['group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):namespace/(?P<ResourceId>.+?)$"), ['namespace']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):template/(?P<ResourceId>.+?)$"), ['template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):theme/(?P<ResourceId>.+?)$"), ['theme']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):topic/(?P<ResourceId>.+?)$"), ['topic']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):user/(?P<ResourceId>.+?)$"), ['user']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):quicksight:(?P<Region>[\w-]*):(?P<Account>\d{12}):vpcConnection/(?P<ResourceId>.+?)$"), ['vpcconnection']),
    ],
    'ram': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ram::(?P<Account>\d{12}):permission/(?P<ResourcePath>.+?)$"), ['permission']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ram:(?P<Region>[\w-]*):(?P<Account>\d{12}):permission/(?P<ResourcePath>.+?)$"), ['customer-managed-permission']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ram:(?P<Region>[\w-]*):(?P<Account>\d{12}):resource-share/(?P<ResourcePath>.+?)$"), ['resource-share']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ram:(?P<Region>[\w-]*):(?P<Account>\d{12}):resource-share-invitation/(?P<ResourcePath>.+?)$"), ['resource-share-invitation']),
    ],
    'rbin': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rbin:(?P<Region>[\w-]*):(?P<Account>\d{12}):rule/(?P<ResourceName>.+?)$"), ['rule']),
    ],
    'rds': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds::(?P<Account>\d{12}):global-cluster:(?P<GlobalCluster>.+?)$"), ['global-cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):cev:(?P<Engine>.+?)/(?P<EngineVersion>.+?)/(?P<CustomDbEngineVersionId>.+?)$"), ['cev']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):snapshot-tenant-database:(?P<SnapshotName>.+?):(?P<TenantResourceId>.+?)$"), ['snapshot-tenant-database']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):auto-backup:(?P<DbInstanceAutomatedBackupId>.+?)$"), ['auto-backup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster:(?P<DbClusterInstanceName>.+?)$"), ['cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster-auto-backup:(?P<DbClusterAutomatedBackupId>.+?)$"), ['cluster-auto-backup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster-endpoint:(?P<DbClusterEndpoint>.+?)$"), ['cluster-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster-pg:(?P<ClusterParameterGroupName>.+?)$"), ['cluster-pg']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster-snapshot:(?P<ClusterSnapshotName>.+?)$"), ['cluster-snapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):db:(?P<DbInstanceName>.+?)$"), ['db']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):db-proxy:(?P<DbProxyId>.+?)$"), ['proxy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):db-proxy-endpoint:(?P<DbProxyEndpointId>.+?)$"), ['proxy-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):deployment:(?P<BlueGreenDeploymentIdentifier>.+?)$"), ['deployment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):es:(?P<SubscriptionName>.+?)$"), ['es']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):integration:(?P<IntegrationIdentifier>.+?)$"), ['integration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):og:(?P<OptionGroupName>.+?)$"), ['og']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):pg:(?P<ParameterGroupName>.+?)$"), ['pg']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):ri:(?P<ReservedDbInstanceName>.+?)$"), ['ri']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):secgrp:(?P<SecurityGroupName>.+?)$"), ['secgrp']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):shard-group:(?P<DbShardGroupResourceId>.+?)$"), ['shardgrp']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):snapshot:(?P<SnapshotName>.+?)$"), ['snapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):subgrp:(?P<SubnetGroupName>.+?)$"), ['subgrp']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):target-group:(?P<TargetGroupId>.+?)$"), ['target-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds:(?P<Region>[\w-]*):(?P<Account>\d{12}):tenant-database:(?P<TenantResourceId>.+?)$"), ['tenant-database']),
    ],
    'rds-db': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rds-db:(?P<Region>[\w-]*):(?P<Account>\d{12}):dbuser:(?P<DbiResourceId>.+?)/(?P<DbUserName>.+?)$"), ['db-user']),
    ],
    'redshift': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):securitygroup:(?P<SecurityGroupName>.+?)/ec2securitygroup/(?P<Owner>.+?)/(?P<Ec2SecurityGroupId>.+?)$"), ['securitygroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):securitygroupingress:(?P<SecurityGroupName>.+?)/ec2securitygroup/(?P<Owner>.+?)/(?P<Ece2SecuritygroupId>.+?)$"), ['securitygroupingress-ec2securitygroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):securitygroupingress:(?P<SecurityGroupName>.+?)/cidrip/(?P<IpRange>.+?)$"), ['securitygroupingress-cidr']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):datashare:(?P<ProducerClusterNamespace>.+?)/(?P<DataShareName>.+?)$"), ['datashare']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):dbgroup:(?P<ClusterName>.+?)/(?P<DbGroup>.+?)$"), ['dbgroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):dbname:(?P<ClusterName>.+?)/(?P<DbName>.+?)$"), ['dbname']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):dbuser:(?P<ClusterName>.+?)/(?P<DbUser>.+?)$"), ['dbuser']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):snapshot:(?P<ClusterName>.+?)/(?P<SnapshotName>.+?)$"), ['snapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster:(?P<ClusterName>.+?)$"), ['cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):eventsubscription:(?P<EventSubscriptionName>.+?)$"), ['eventsubscription']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):hsmclientcertificate:(?P<HSMClientCertificateId>.+?)$"), ['hsmclientcertificate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):hsmconfiguration:(?P<HSMConfigurationId>.+?)$"), ['hsmconfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):integration:(?P<IntegrationIdentifier>.+?)$"), ['integration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):namespace:(?P<ClusterNamespace>.+?)$"), ['namespace']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):parametergroup:(?P<ParameterGroupName>.+?)$"), ['parametergroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):qev2idcapplication:(?P<Qev2IdcApplicationId>.+?)$"), ['qev2idcapplication']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):redshiftidcapplication:(?P<RedshiftIdcApplicationId>.+?)$"), ['redshiftidcapplication']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):snapshotcopygrant:(?P<SnapshotCopyGrantName>.+?)$"), ['snapshotcopygrant']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):snapshotschedule:(?P<ScheduleIdentifier>.+?)$"), ['snapshotschedule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):subnetgroup:(?P<SubnetGroupName>.+?)$"), ['subnetgroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift:(?P<Region>[\w-]*):(?P<Account>\d{12}):usagelimit:(?P<UsageLimitId>.+?)$"), ['usagelimit']),
    ],
    'redshift-serverless': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift-serverless:(?P<Region>[\w-]*):(?P<Account>\d{12}):managed-workgroup/(?P<ManagedWorkgroupId>.+?)$"), ['managed-workgroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift-serverless:(?P<Region>[\w-]*):(?P<Account>\d{12}):managed-workgroup/(?P<ManagedWorkgroupName>.+?)$"), ['managed-workgroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift-serverless:(?P<Region>[\w-]*):(?P<Account>\d{12}):managedvpcendpoint/(?P<EndpointAccessId>.+?)$"), ['endpointAccess']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift-serverless:(?P<Region>[\w-]*):(?P<Account>\d{12}):namespace/(?P<NamespaceId>.+?)$"), ['namespace']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift-serverless:(?P<Region>[\w-]*):(?P<Account>\d{12}):recoverypoint/(?P<RecoveryPointId>.+?)$"), ['recoveryPoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift-serverless:(?P<Region>[\w-]*):(?P<Account>\d{12}):snapshot/(?P<SnapshotId>.+?)$"), ['snapshot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):redshift-serverless:(?P<Region>[\w-]*):(?P<Account>\d{12}):workgroup/(?P<WorkgroupId>.+?)$"), ['workgroup']),
    ],
    'refactor-spaces': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):refactor-spaces:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment/(?P<EnvironmentId>.+?)/application/(?P<ApplicationId>.+?)/route/(?P<RouteId>.+?)$"), ['route']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):refactor-spaces:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment/(?P<EnvironmentId>.+?)/application/(?P<ApplicationId>.+?)/service/(?P<ServiceId>.+?)$"), ['service']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):refactor-spaces:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment/(?P<EnvironmentId>.+?)/application/(?P<ApplicationId>.+?)$"), ['application']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):refactor-spaces:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment/(?P<EnvironmentId>.+?)$"), ['environment']),
    ],
    'rekognition': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rekognition:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ProjectName>.+?)/dataset/(?P<DatasetType>.+?)/(?P<CreationTimestamp>.+?)$"), ['dataset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rekognition:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ProjectName>.+?)/version/(?P<VersionName>.+?)/(?P<CreationTimestamp>.+?)$"), ['projectversion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rekognition:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ProjectName>.+?)/(?P<CreationTimestamp>.+?)$"), ['project']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rekognition:(?P<Region>[\w-]*):(?P<Account>\d{12}):collection/(?P<CollectionId>.+?)$"), ['collection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rekognition:(?P<Region>[\w-]*):(?P<Account>\d{12}):streamprocessor/(?P<StreamprocessorId>.+?)$"), ['streamprocessor']),
    ],
    'repostspace': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):repostspace:(?P<Region>[\w-]*):(?P<Account>\d{12}):space/(?P<ResourceId>.+?)$"), ['space']),
    ],
    'resiliencehub': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):resiliencehub:(?P<Region>[\w-]*):(?P<Account>\d{12}):app/(?P<AppId>.+?)$"), ['application']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):resiliencehub:(?P<Region>[\w-]*):(?P<Account>\d{12}):app-assessment/(?P<AppAssessmentId>.+?)$"), ['app-assessment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):resiliencehub:(?P<Region>[\w-]*):(?P<Account>\d{12}):recommendation-template/(?P<RecommendationTemplateId>.+?)$"), ['recommendation-template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):resiliencehub:(?P<Region>[\w-]*):(?P<Account>\d{12}):resiliency-policy/(?P<ResiliencyPolicyId>.+?)$"), ['resiliency-policy']),
    ],
    'resource-explorer-2': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):resource-explorer-2:(?P<Region>[\w-]*):(?P<Account>\d{12}):managed-view/(?P<ManagedViewName>.+?)/(?P<ManagedViewUuid>.+?)$"), ['managed-view']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):resource-explorer-2:(?P<Region>[\w-]*):(?P<Account>\d{12}):view/(?P<ViewName>.+?)/(?P<ViewUuid>.+?)$"), ['view']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):resource-explorer-2:(?P<Region>[\w-]*):(?P<Account>\d{12}):index/(?P<IndexUuid>.+?)$"), ['index']),
    ],
    'resource-groups': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):resource-groups:(?P<Region>[\w-]*):(?P<Account>\d{12}):group/(?P<GroupName>.+?)/tag-sync-task/(?P<TaskId>.+?)$"), ['tagSyncTask']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):resource-groups:(?P<Region>[\w-]*):(?P<Account>\d{12}):group/(?P<GroupName>.+?)$"), ['group']),
    ],
    'robomaker': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):robomaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):deployment-fleet/(?P<FleetName>.+?)/(?P<CreatedOnEpoch>.+?)$"), ['deploymentFleet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):robomaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):robot/(?P<RobotName>.+?)/(?P<CreatedOnEpoch>.+?)$"), ['robot']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):robomaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):robot-application/(?P<ApplicationName>.+?)/(?P<CreatedOnEpoch>.+?)$"), ['robotApplication']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):robomaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):simulation-application/(?P<ApplicationName>.+?)/(?P<CreatedOnEpoch>.+?)$"), ['simulationApplication']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):robomaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):deployment-job/(?P<DeploymentJobId>.+?)$"), ['deploymentJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):robomaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):simulation-job/(?P<SimulationJobId>.+?)$"), ['simulationJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):robomaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):simulation-job-batch/(?P<SimulationJobBatchId>.+?)$"), ['simulationJobBatch']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):robomaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):world/(?P<WorldId>.+?)$"), ['world']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):robomaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):world-export-job/(?P<WorldExportJobId>.+?)$"), ['worldExportJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):robomaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):world-generation-job/(?P<WorldGenerationJobId>.+?)$"), ['worldGenerationJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):robomaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):world-template/(?P<WorldTemplateJobId>.+?)$"), ['worldTemplate']),
    ],
    'rolesanywhere': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rolesanywhere:(?P<Region>[\w-]*):(?P<Account>\d{12}):crl/(?P<CrlId>.+?)$"), ['crl']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rolesanywhere:(?P<Region>[\w-]*):(?P<Account>\d{12}):profile/(?P<ProfileId>.+?)$"), ['profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rolesanywhere:(?P<Region>[\w-]*):(?P<Account>\d{12}):subject/(?P<SubjectId>.+?)$"), ['subject']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rolesanywhere:(?P<Region>[\w-]*):(?P<Account>\d{12}):trust-anchor/(?P<TrustAnchorId>.+?)$"), ['trust-anchor']),
    ],
    'route53': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53:::change/(?P<Id>.+?)$"), ['change']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53:::cidrcollection/(?P<Id>.+?)$"), ['cidrcollection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53:::delegationset/(?P<Id>.+?)$"), ['delegationset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53:::healthcheck/(?P<Id>.+?)$"), ['healthcheck']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53:::hostedzone/(?P<Id>.+?)$"), ['hostedzone']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53:::queryloggingconfig/(?P<Id>.+?)$"), ['queryloggingconfig']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53:::trafficpolicy/(?P<Id>.+?)$"), ['trafficpolicy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53:::trafficpolicyinstance/(?P<Id>.+?)$"), ['trafficpolicyinstance']),
    ],
    'route53-recovery-control': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53-recovery-control::(?P<Account>\d{12}):controlpanel/(?P<ControlPanelId>.+?)/routingcontrol/(?P<RoutingControlId>.+?)$"), ['routingcontrol']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53-recovery-control::(?P<Account>\d{12}):controlpanel/(?P<ControlPanelId>.+?)/safetyrule/(?P<SafetyRuleId>.+?)$"), ['safetyrule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53-recovery-control::(?P<Account>\d{12}):cluster/(?P<ResourceId>.+?)$"), ['cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53-recovery-control::(?P<Account>\d{12}):controlpanel/(?P<ControlPanelId>.+?)$"), ['controlpanel']),
    ],
    'route53-recovery-readiness': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53-recovery-readiness::(?P<Account>\d{12}):cell/(?P<ResourceId>.+?)$"), ['cell']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53-recovery-readiness::(?P<Account>\d{12}):readiness-check/(?P<ResourceId>.+?)$"), ['readinesscheck']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53-recovery-readiness::(?P<Account>\d{12}):recovery-group/(?P<ResourceId>.+?)$"), ['recoverygroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53-recovery-readiness::(?P<Account>\d{12}):resource-set/(?P<ResourceId>.+?)$"), ['resourceset']),
    ],
    'route53globalresolver': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53globalresolver::(?P<Account>\d{12}):access-source/(?P<Id>.+?)$"), ['access-source']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53globalresolver::(?P<Account>\d{12}):access-token/(?P<Id>.+?)$"), ['access-token']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53globalresolver::(?P<Account>\d{12}):dns-view/(?P<Id>.+?)$"), ['dns-view']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53globalresolver::(?P<Account>\d{12}):firewall-domain-list/(?P<Id>.+?)$"), ['firewall-domain-list']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53globalresolver::(?P<Account>\d{12}):global-resolver/(?P<Id>.+?)$"), ['global-resolver']),
    ],
    'route53profiles': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53profiles:(?P<Region>[\w-]*):(?P<Account>\d{12}):profile/(?P<ResourceId>.+?)$"), ['profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53profiles:(?P<Region>[\w-]*):(?P<Account>\d{12}):profile-association/(?P<ResourceId>.+?)$"), ['profile-association']),
    ],
    'route53resolver': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53resolver:(?P<Region>[\w-]*):(?P<Account>\d{12}):autodefined-rule/(?P<ResourceId>.+?)$"), ['autodefined-rule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53resolver:(?P<Region>[\w-]*):(?P<Account>\d{12}):firewall-config/(?P<ResourceId>.+?)$"), ['firewall-config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53resolver:(?P<Region>[\w-]*):(?P<Account>\d{12}):firewall-domain-list/(?P<ResourceId>.+?)$"), ['firewall-domain-list']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53resolver:(?P<Region>[\w-]*):(?P<Account>\d{12}):firewall-rule-group/(?P<ResourceId>.+?)$"), ['firewall-rule-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53resolver:(?P<Region>[\w-]*):(?P<Account>\d{12}):firewall-rule-group-association/(?P<ResourceId>.+?)$"), ['firewall-rule-group-association']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53resolver:(?P<Region>[\w-]*):(?P<Account>\d{12}):outpost-resolver/(?P<ResourceId>.+?)$"), ['outpost-resolver']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53resolver:(?P<Region>[\w-]*):(?P<Account>\d{12}):resolver-config/(?P<ResourceId>.+?)$"), ['resolver-config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53resolver:(?P<Region>[\w-]*):(?P<Account>\d{12}):resolver-dnssec-config/(?P<ResourceId>.+?)$"), ['resolver-dnssec-config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53resolver:(?P<Region>[\w-]*):(?P<Account>\d{12}):resolver-endpoint/(?P<ResourceId>.+?)$"), ['resolver-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53resolver:(?P<Region>[\w-]*):(?P<Account>\d{12}):resolver-query-log-config/(?P<ResourceId>.+?)$"), ['resolver-query-log-config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):route53resolver:(?P<Region>[\w-]*):(?P<Account>\d{12}):resolver-rule/(?P<ResourceId>.+?)$"), ['resolver-rule']),
    ],
    'rum': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):rum:(?P<Region>[\w-]*):(?P<Account>\d{12}):appmonitor/(?P<Name>.+?)$"), ['AppMonitorResource']),
    ],
    's3': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3:::(?P<BucketName>.+?)/(?P<ObjectName>.+?)$"), ['object']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3:::(?P<BucketName>.+?)$"), ['bucket']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3::(?P<Account>\d{12}):accesspoint/(?P<AccessPointAlias>.+?)$"), ['multiregionaccesspoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3:us-west-2:(?P<Account>\d{12}):async-request/mrap/(?P<Operation>.+?)/(?P<Token>.+?)$"), ['multiregionaccesspointrequestarn']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3:(?P<Region>[\w-]*):(?P<Account>\d{12}):access-grants/default/grant/(?P<Token>.+?)$"), ['accessgrant']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3:(?P<Region>[\w-]*):(?P<Account>\d{12}):access-grants/default/location/(?P<Token>.+?)$"), ['accessgrantslocation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3:(?P<Region>[\w-]*):(?P<Account>\d{12}):accesspoint/(?P<AccessPointName>.+?)/object/(?P<ObjectName>.+?)$"), ['accesspointobject']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3:(?P<Region>[\w-]*):(?P<Account>\d{12}):access-grants/default$"), ['accessgrantsinstance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3:(?P<Region>[\w-]*):(?P<Account>\d{12}):accesspoint/(?P<AccessPointName>.+?)$"), ['accesspoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3:(?P<Region>[\w-]*):(?P<Account>\d{12}):job/(?P<JobId>.+?)$"), ['job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3:(?P<Region>[\w-]*):(?P<Account>\d{12}):storage-lens/(?P<ConfigId>.+?)$"), ['storagelensconfiguration', 'storage-lens']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3:(?P<Region>[\w-]*):(?P<Account>\d{12}):storage-lens-group/(?P<Name>.+?)$"), ['storagelensgroup']),
    ],
    's3-object-lambda': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3-object-lambda:(?P<Region>[\w-]*):(?P<Account>\d{12}):accesspoint/(?P<AccessPointName>.+?)$"), ['objectlambdaaccesspoint']),
    ],
    's3-outposts': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3-outposts:(?P<Region>[\w-]*):(?P<Account>\d{12}):outpost/(?P<OutpostId>.+?)/bucket/(?P<BucketName>.+?)/object/(?P<ObjectName>.+?)$"), ['object']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3-outposts:(?P<Region>[\w-]*):(?P<Account>\d{12}):outpost/(?P<OutpostId>.+?)/accesspoint/(?P<AccessPointName>.+?)$"), ['accesspoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3-outposts:(?P<Region>[\w-]*):(?P<Account>\d{12}):outpost/(?P<OutpostId>.+?)/bucket/(?P<BucketName>.+?)$"), ['bucket']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3-outposts:(?P<Region>[\w-]*):(?P<Account>\d{12}):outpost/(?P<OutpostId>.+?)/endpoint/(?P<EndpointId>.+?)$"), ['endpoint']),
    ],
    's3express': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3express:(?P<Region>[\w-]*):(?P<Account>\d{12}):accesspoint/(?P<AccessPointName>.+?)$"), ['accesspoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3express:(?P<Region>[\w-]*):(?P<Account>\d{12}):bucket/(?P<BucketName>.+?)$"), ['bucket']),
    ],
    's3tables': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3tables:(?P<Region>[\w-]*):(?P<Account>\d{12}):bucket/(?P<TableBucketName>.+?)/table/(?P<TableID>.+?)$"), ['Table']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3tables:(?P<Region>[\w-]*):(?P<Account>\d{12}):bucket/(?P<TableBucketName>.+?)$"), ['TableBucket']),
    ],
    's3vectors': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3vectors:(?P<Region>[\w-]*):(?P<Account>\d{12}):bucket/(?P<BucketName>.+?)/index/(?P<IndexName>.+?)$"), ['Index']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):s3vectors:(?P<Region>[\w-]*):(?P<Account>\d{12}):bucket/(?P<BucketName>.+?)$"), ['VectorBucket']),
    ],
    'sagemaker': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):app/(?P<DomainId>.+?)/(?P<UserProfileName>.+?)/(?P<AppType>.+?)/(?P<AppName>.+?)$"), ['app']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):device-fleet/(?P<DeviceFleetName>.+?)/device/(?P<DeviceName>.+?)$"), ['device']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):hub-content/(?P<HubName>.+?)/(?P<HubContentType>.+?)/(?P<HubContentName>.+?)$"), ['hub-content']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):model-card/(?P<ModelCardName>.+?)/export-job/(?P<ExportJobName>.+?)$"), ['model-card-export-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):monitoring-schedule/(?P<MonitoringScheduleName>.+?)/alert/(?P<MonitoringScheduleAlertName>.+?)$"), ['monitoring-schedule-alert']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):pipeline/(?P<PipelineName>.+?)/execution/(?P<RandomString>.+?)$"), ['pipeline-execution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):image-version/(?P<ImageName>.+?)/(?P<Version>.+?)$"), ['image-version']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):space/(?P<DomainId>.+?)/(?P<SpaceName>.+?)$"), ['space']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):user-profile/(?P<DomainId>.+?)/(?P<UserProfileName>.+?)$"), ['user-profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):action/(?P<ActionName>.+?)$"), ['action']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):algorithm/(?P<AlgorithmName>.+?)$"), ['algorithm']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):app-image-config/(?P<AppImageConfigName>.+?)$"), ['app-image-config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):artifact/(?P<HashOfArtifactSource>.+?)$"), ['artifact']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):automl-job/(?P<AutoMLJobJobName>.+?)$"), ['automl-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster/(?P<ClusterId>.+?)$"), ['cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):cluster-scheduler-config/(?P<ClusterSchedulerConfigId>.+?)$"), ['cluster-scheduler-config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):code-repository/(?P<CodeRepositoryName>.+?)$"), ['code-repository']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):compilation-job/(?P<CompilationJobName>.+?)$"), ['compilation-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):compute-quota/(?P<ComputeQuotaId>.+?)$"), ['compute-quota']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):context/(?P<ContextName>.+?)$"), ['context']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-quality-job-definition/(?P<DataQualityJobDefinitionName>.+?)$"), ['data-quality-job-definition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):device-fleet/(?P<DeviceFleetName>.+?)$"), ['device-fleet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):domain/(?P<DomainId>.+?)$"), ['domain']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):edge-deployment/(?P<EdgeDeploymentPlanName>.+?)$"), ['edge-deployment-plan']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):edge-packaging-job/(?P<EdgePackagingJobName>.+?)$"), ['edge-packaging-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):endpoint/(?P<EndpointName>.+?)$"), ['endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):endpoint-config/(?P<EndpointConfigName>.+?)$"), ['endpoint-config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):experiment/(?P<ExperimentName>.+?)$"), ['experiment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):experiment-trial/(?P<TrialName>.+?)$"), ['experiment-trial']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):experiment-trial-component/(?P<TrialComponentName>.+?)$"), ['experiment-trial-component']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):feature-group/(?P<FeatureGroupName>.+?)$"), ['feature-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):flow-definition/(?P<FlowDefinitionName>.+?)$"), ['flow-definition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):hub/(?P<HubName>.+?)$"), ['hub']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):human-loop/(?P<HumanLoopName>.+?)$"), ['human-loop']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):human-task-ui/(?P<HumanTaskUiName>.+?)$"), ['human-task-ui']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):hyper-parameter-tuning-job/(?P<HyperParameterTuningJobName>.+?)$"), ['hyper-parameter-tuning-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):image/(?P<ImageName>.+?)$"), ['image']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):inference-component/(?P<InferenceComponentName>.+?)$"), ['inference-component']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):inference-experiment/(?P<InferenceExperimentName>.+?)$"), ['inference-experiment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):inference-recommendations-job/(?P<InferenceRecommendationsJobName>.+?)$"), ['inference-recommendations-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):labeling-job/(?P<LabelingJobName>.+?)$"), ['labeling-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):lineage-group/(?P<LineageGroupName>.+?)$"), ['lineage-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):mlflow-app/(?P<MLflowAppId>.+?)$"), ['mlflow-app']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):mlflow-tracking-server/(?P<MlflowTrackingServerName>.+?)$"), ['mlflow-tracking-server']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):model/(?P<ModelName>.+?)$"), ['model']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):model-bias-job-definition/(?P<ModelBiasJobDefinitionName>.+?)$"), ['model-bias-job-definition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):model-card/(?P<ModelCardName>.+?)$"), ['model-card']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):model-explainability-job-definition/(?P<ModelExplainabilityJobDefinitionName>.+?)$"), ['model-explainability-job-definition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):model-package/(?P<ModelPackageName>.+?)$"), ['model-package']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):model-package-group/(?P<ModelPackageGroupName>.+?)$"), ['model-package-group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):model-quality-job-definition/(?P<ModelQualityJobDefinitionName>.+?)$"), ['model-quality-job-definition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):monitoring-schedule/(?P<MonitoringScheduleName>.+?)$"), ['monitoring-schedule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):notebook-instance/(?P<NotebookInstanceName>.+?)$"), ['notebook-instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):notebook-instance-lifecycle-config/(?P<NotebookInstanceLifecycleConfigName>.+?)$"), ['notebook-instance-lifecycle-config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):optimization-job/(?P<OptimizationJobName>.+?)$"), ['optimization-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):partner-app/(?P<AppId>.+?)$"), ['partner-app']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):pipeline/(?P<PipelineName>.+?)$"), ['pipeline']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):processing-job/(?P<ProcessingJobName>.+?)$"), ['processing-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):project/(?P<ProjectName>.+?)$"), ['project']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):reserved-capacity/(?P<RandomString>.+?)$"), ['reserved-capacity']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):sagemaker-catalog/(?P<ResourceCatalogName>.+?)$"), ['sagemaker-catalog']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):shared-model/(?P<SharedModelId>.+?)$"), ['shared-model']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):shared-model-event/(?P<EventId>.+?)$"), ['shared-model-event']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):studio-lifecycle-config/(?P<StudioLifecycleConfigName>.+?)$"), ['studio-lifecycle-config']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):training-job/(?P<TrainingJobName>.+?)$"), ['training-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):training-plan/(?P<TrainingPlanName>.+?)$"), ['training-plan']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):transform-job/(?P<TransformJobName>.+?)$"), ['transform-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):workforce/(?P<WorkforceName>.+?)$"), ['workforce']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker:(?P<Region>[\w-]*):(?P<Account>\d{12}):workteam/(?P<WorkteamName>.+?)$"), ['workteam']),
    ],
    'sagemaker-geospatial': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker-geospatial:(?P<Region>[\w-]*):(?P<Account>\d{12}):earth-observation-job/(?P<JobID>.+?)$"), ['EarthObservationJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker-geospatial:(?P<Region>[\w-]*):(?P<Account>\d{12}):raster-data-collection/(?P<CollectionID>.+?)$"), ['RasterDataCollection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sagemaker-geospatial:(?P<Region>[\w-]*):(?P<Account>\d{12}):vector-enrichment-job/(?P<JobID>.+?)$"), ['VectorEnrichmentJob']),
    ],
    'savingsplans': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):savingsplans::(?P<Account>\d{12}):savingsplan/(?P<ResourceId>.+?)$"), ['savingsplan']),
    ],
    'scheduler': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):scheduler:(?P<Region>[\w-]*):(?P<Account>\d{12}):schedule/(?P<GroupName>.+?)/(?P<ScheduleName>.+?)$"), ['schedule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):scheduler:(?P<Region>[\w-]*):(?P<Account>\d{12}):schedule-group/(?P<GroupName>.+?)$"), ['schedule-group']),
    ],
    'schemas': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):schemas:(?P<Region>[\w-]*):(?P<Account>\d{12}):schema/(?P<RegistryName>.+?)/(?P<SchemaName>.+?)$"), ['schema']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):schemas:(?P<Region>[\w-]*):(?P<Account>\d{12}):discoverer/(?P<DiscovererId>.+?)$"), ['discoverer']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):schemas:(?P<Region>[\w-]*):(?P<Account>\d{12}):registry/(?P<RegistryName>.+?)$"), ['registry']),
    ],
    'scn': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):scn:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/namespaces/(?P<Namespace>.+?)/datasets/(?P<DatasetName>.+?)$"), ['dataset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):scn:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/bill-of-materials-import-job/(?P<JobId>.+?)$"), ['bill-of-materials-import-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):scn:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/data-integration-flows/(?P<FlowName>.+?)$"), ['data-integration-flow']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):scn:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)/namespaces/(?P<Namespace>.+?)$"), ['namespace']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):scn:(?P<Region>[\w-]*):(?P<Account>\d{12}):instance/(?P<InstanceId>.+?)$"), ['instance']),
    ],
    'sdb': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sdb:(?P<Region>[\w-]*):(?P<Account>\d{12}):domain/(?P<DomainName>.+?)$"), ['domain']),
    ],
    'secretsmanager': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):secretsmanager:(?P<Region>[\w-]*):(?P<Account>\d{12}):secret:(?P<SecretId>.+?)$"), ['Secret', 'secret']),
    ],
    'security-ir': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):security-ir:(?P<Region>[\w-]*):(?P<Account>\d{12}):case/(?P<CaseId>.+?)$"), ['case']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):security-ir:(?P<Region>[\w-]*):(?P<Account>\d{12}):membership/(?P<MembershipId>.+?)$"), ['membership']),
    ],
    'securityagent': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityagent:(?P<Region>[\w-]*):(?P<Account>\d{12}):agent-space/(?P<AgentId>.+?)/artifact/(?P<ArtifactId>.+?)$"), ['Artifact']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityagent:(?P<Region>[\w-]*):(?P<Account>\d{12}):agent-space/(?P<AgentId>.+?)/finding/(?P<FindingId>.+?)$"), ['Finding']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityagent:(?P<Region>[\w-]*):(?P<Account>\d{12}):agent-space/(?P<AgentId>.+?)/pentest/(?P<PentestId>.+?)$"), ['Pentest']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityagent:(?P<Region>[\w-]*):(?P<Account>\d{12}):agent-space/(?P<AgentId>.+?)/pentest-job/(?P<JobId>.+?)$"), ['PentestJob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityagent:(?P<Region>[\w-]*):(?P<Account>\d{12}):agent-space/(?P<AgentId>.+?)/pentest-task/(?P<TaskId>.+?)$"), ['PentestTask']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityagent:(?P<Region>[\w-]*):(?P<Account>\d{12}):agent-instance/(?P<AgentId>.+?)$"), ['AgentInstance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityagent:(?P<Region>[\w-]*):(?P<Account>\d{12}):agent-space/(?P<AgentId>.+?)$"), ['AgentSpace']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityagent:(?P<Region>[\w-]*):(?P<Account>\d{12}):application/(?P<ApplicationId>.+?)$"), ['Application']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityagent:(?P<Region>[\w-]*):(?P<Account>\d{12}):control/(?P<ControlId>.+?)$"), ['Control']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityagent:(?P<Region>[\w-]*):(?P<Account>\d{12}):integration/(?P<IntegrationId>.+?)$"), ['Integration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityagent:(?P<Region>[\w-]*):(?P<Account>\d{12}):security-requirement/(?P<SecurityRequirementId>.+?)$"), ['SecurityRequirement']),
    ],
    'securityhub': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityhub:(?P<Region>[\w-]*):(?P<Account>\d{12}):product/(?P<Company>.+?)/(?P<ProductId>.+?)$"), ['product']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityhub:(?P<Region>[\w-]*):(?P<Account>\d{12}):aggregatorv2/(?P<AggregatorV2Id>.+?)$"), ['aggregatorv2']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityhub:(?P<Region>[\w-]*):(?P<Account>\d{12}):automation-rule/(?P<AutomationRuleId>.+?)$"), ['automation-rule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityhub:(?P<Region>[\w-]*):(?P<Account>\d{12}):automation-rulev2/(?P<AutomationRuleV2Id>.+?)$"), ['automation-rulev2']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityhub:(?P<Region>[\w-]*):(?P<Account>\d{12}):configuration-policy/(?P<ConfigurationPolicyId>.+?)$"), ['configuration-policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityhub:(?P<Region>[\w-]*):(?P<Account>\d{12}):connectorv2/(?P<ConnectorV2Id>.+?)$"), ['connectorv2']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityhub:(?P<Region>[\w-]*):(?P<Account>\d{12}):finding-aggregator/(?P<FindingAggregatorId>.+?)$"), ['finding-aggregator']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityhub:(?P<Region>[\w-]*):(?P<Account>\d{12}):hub/default$"), ['hub']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securityhub:(?P<Region>[\w-]*):(?P<Account>\d{12}):hubv2/(?P<HubV2Id>.+?)$"), ['hubv2']),
    ],
    'securitylake': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securitylake:(?P<Region>[\w-]*):(?P<Account>\d{12}):data-lake/default$"), ['data-lake']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):securitylake:(?P<Region>[\w-]*):(?P<Account>\d{12}):subscriber/(?P<SubscriberId>.+?)$"), ['subscriber']),
    ],
    'serverlessrepo': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):serverlessrepo:(?P<Region>[\w-]*):(?P<Account>\d{12}):applications/(?P<ResourceId>.+?)$"), ['applications']),
    ],
    'servicecatalog': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):servicecatalog:(?P<Region>[\w-]*):(?P<Account>\d{12}):/applications/(?P<ApplicationId>.+?)$"), ['Application']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):servicecatalog:(?P<Region>[\w-]*):(?P<Account>\d{12}):/attribute-groups/(?P<AttributeGroupId>.+?)$"), ['AttributeGroup']),
    ],
    'servicediscovery': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):servicediscovery:(?P<Region>[\w-]*):(?P<Account>\d{12}):namespace/(?P<NamespaceId>.+?)$"), ['namespace']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):servicediscovery:(?P<Region>[\w-]*):(?P<Account>\d{12}):service/(?P<ServiceId>.+?)$"), ['service']),
    ],
    'servicequotas': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):servicequotas:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<ServiceCode>.+?)/(?P<QuotaCode>.+?)$"), ['quota']),
    ],
    'ses': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):aws:reputation-policy/(?P<ReputationPolicyName>.+?)$"), ['reputation-policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):tenant/(?P<TenantName>.+?)/(?P<TenantId>.+?)$"), ['tenant']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):addon-instance/(?P<AddonInstanceId>.+?)$"), ['addon-instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):addon-subscription/(?P<AddonSubscriptionId>.+?)$"), ['addon-subscription']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):configuration-set/(?P<ConfigurationSetName>.+?)$"), ['configuration-set']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):contact-list/(?P<ContactListName>.+?)$"), ['contact-list']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):custom-verification-email-template/(?P<TemplateName>.+?)$"), ['custom-verification-email-template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):dedicated-ip-pool/(?P<DedicatedIPPool>.+?)$"), ['dedicated-ip-pool']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):deliverability-test-report/(?P<ReportId>.+?)$"), ['deliverability-test-report']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):export-job/(?P<ExportJobId>.+?)$"), ['export-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):identity/(?P<IdentityName>.+?)$"), ['identity']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):import-job/(?P<ImportJobId>.+?)$"), ['import-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):mailmanager-address-list/(?P<AddressListId>.+?)$"), ['mailmanager-address-list']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):mailmanager-archive/(?P<ArchiveId>.+?)$"), ['mailmanager-archive']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):mailmanager-ingress-point/(?P<IngressPointId>.+?)$"), ['mailmanager-ingress-point']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):mailmanager-rule-set/(?P<RuleSetId>.+?)$"), ['mailmanager-rule-set']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):mailmanager-smtp-relay/(?P<RelayId>.+?)$"), ['mailmanager-smtp-relay']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):mailmanager-traffic-policy/(?P<TrafficPolicyId>.+?)$"), ['mailmanager-traffic-policy']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):multi-region-endpoint/(?P<EndpointName>.+?)$"), ['multi-region-endpoint']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ses:(?P<Region>[\w-]*):(?P<Account>\d{12}):template/(?P<TemplateName>.+?)$"), ['template']),
    ],
    'shield': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):shield::(?P<Account>\d{12}):attack/(?P<Id>.+?)$"), ['attack']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):shield::(?P<Account>\d{12}):protection/(?P<Id>.+?)$"), ['protection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):shield::(?P<Account>\d{12}):protection-group/(?P<Id>.+?)$"), ['protection-group']),
    ],
    'signer': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):signer:(?P<Region>[\w-]*):(?P<Account>\d{12}):/signing-jobs/(?P<JobId>.+?)$"), ['signing-job']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):signer:(?P<Region>[\w-]*):(?P<Account>\d{12}):/signing-profiles/(?P<ProfileName>.+?)$"), ['signing-profile']),
    ],
    'simspaceweaver': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):simspaceweaver:(?P<Region>[\w-]*):(?P<Account>\d{12}):simulation/(?P<SimulationName>.+?)$"), ['Simulation']),
    ],
    'sms-voice': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sms-voice:(?P<Region>[\w-]*):(?P<Account>\d{12}):sender-id/(?P<SenderId>.+?)/(?P<IsoCountryCode>.+?)$"), ['SenderId']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sms-voice:(?P<Region>[\w-]*):(?P<Account>\d{12}):configuration-set/(?P<ConfigurationSetName>.+?)$"), ['ConfigurationSet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sms-voice:(?P<Region>[\w-]*):(?P<Account>\d{12}):message/(?P<MessageId>.+?)$"), ['Message']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sms-voice:(?P<Region>[\w-]*):(?P<Account>\d{12}):opt-out-list/(?P<OptOutListName>.+?)$"), ['OptOutList']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sms-voice:(?P<Region>[\w-]*):(?P<Account>\d{12}):phone-number/(?P<PhoneNumberId>.+?)$"), ['PhoneNumber']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sms-voice:(?P<Region>[\w-]*):(?P<Account>\d{12}):pool/(?P<PoolId>.+?)$"), ['Pool']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sms-voice:(?P<Region>[\w-]*):(?P<Account>\d{12}):protect-configuration/(?P<ProtectConfigurationId>.+?)$"), ['ProtectConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sms-voice:(?P<Region>[\w-]*):(?P<Account>\d{12}):registration/(?P<RegistrationId>.+?)$"), ['Registration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sms-voice:(?P<Region>[\w-]*):(?P<Account>\d{12}):registration-attachment/(?P<RegistrationAttachmentId>.+?)$"), ['RegistrationAttachment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sms-voice:(?P<Region>[\w-]*):(?P<Account>\d{12}):verified-destination-number/(?P<VerifiedDestinationNumberId>.+?)$"), ['VerifiedDestinationNumber']),
    ],
    'snow-device-management': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):snow-device-management:(?P<Region>[\w-]*):(?P<Account>\d{12}):managed-device/(?P<ResourceId>.+?)$"), ['managed-device']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):snow-device-management:(?P<Region>[\w-]*):(?P<Account>\d{12}):task/(?P<ResourceId>.+?)$"), ['task']),
    ],
    'sns': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sns:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<TopicName>.+?)$"), ['topic']),
    ],
    'social-messaging': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):social-messaging:(?P<Region>[\w-]*):(?P<Account>\d{12}):phone-number-id/(?P<OriginationPhoneNumberId>.+?)$"), ['phone-number-id']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):social-messaging:(?P<Region>[\w-]*):(?P<Account>\d{12}):waba/(?P<WabaId>.+?)$"), ['waba']),
    ],
    'sqlworkbench': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sqlworkbench:(?P<Region>[\w-]*):(?P<Account>\d{12}):chart/(?P<ResourceId>.+?)$"), ['chart']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sqlworkbench:(?P<Region>[\w-]*):(?P<Account>\d{12}):connection/(?P<ResourceId>.+?)$"), ['connection']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sqlworkbench:(?P<Region>[\w-]*):(?P<Account>\d{12}):notebook/(?P<ResourceId>.+?)$"), ['notebook']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sqlworkbench:(?P<Region>[\w-]*):(?P<Account>\d{12}):query/(?P<ResourceId>.+?)$"), ['query']),
    ],
    'sqs': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sqs:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<QueueName>.+?)$"), ['queue']),
    ],
    'ssm': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):automation-definition/(?P<AutomationDefinitionName>.+?):(?P<VersionId>.+?)$"), ['automation-definition']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):association/(?P<AssociationId>.+?)$"), ['association']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):automation-execution/(?P<AutomationExecutionId>.+?)$"), ['automation-execution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):document/(?P<DocumentName>.+?)$"), ['document']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):maintenancewindow/(?P<ResourceId>.+?)$"), ['maintenancewindow']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):managed-instance/(?P<InstanceId>.+?)$"), ['managed-instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):managed-instance-inventory/(?P<InstanceId>.+?)$"), ['managed-instance-inventory']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):opsitem/(?P<ResourceId>.+?)$"), ['opsitem']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):opsitemgroup/default$"), ['opsitemgroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):opsmetadata/(?P<ResourceId>.+?)$"), ['opsmetadata']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):parameter/(?P<ParameterNameWithoutLeadingSlash>.+?)$"), ['parameter']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):patchbaseline/(?P<PatchBaselineIdResourceId>.+?)$"), ['patchbaseline']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):resource-data-sync/(?P<SyncName>.+?)$"), ['resourcedatasync', 'resource-data-sync']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):servicesetting/(?P<ResourceId>.+?)$"), ['servicesetting']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):session/(?P<SessionId>.+?)$"), ['session']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):windowtarget/(?P<WindowTargetId>.+?)$"), ['windowtarget']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm:(?P<Region>[\w-]*):(?P<Account>\d{12}):windowtask/(?P<WindowTaskId>.+?)$"), ['windowtask']),
    ],
    'ssm-contacts': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm-contacts:(?P<Region>[\w-]*):(?P<Account>\d{12}):contactchannel/(?P<ContactAlias>.+?)/(?P<ContactChannelId>.+?)$"), ['contactchannel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm-contacts:(?P<Region>[\w-]*):(?P<Account>\d{12}):engagement/(?P<ContactAlias>.+?)/(?P<EngagementId>.+?)$"), ['engagement']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm-contacts:(?P<Region>[\w-]*):(?P<Account>\d{12}):page/(?P<ContactAlias>.+?)/(?P<PageId>.+?)$"), ['page']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm-contacts:(?P<Region>[\w-]*):(?P<Account>\d{12}):contact/(?P<ContactAlias>.+?)$"), ['contact']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm-contacts:(?P<Region>[\w-]*):(?P<Account>\d{12}):rotation/(?P<RotationId>.+?)$"), ['rotation']),
    ],
    'ssm-incidents': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm-incidents::(?P<Account>\d{12}):incident-record/(?P<ResponsePlan>.+?)/(?P<IncidentRecord>.+?)$"), ['incident-record']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm-incidents::(?P<Account>\d{12}):replication-set/(?P<ReplicationSet>.+?)$"), ['replication-set']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm-incidents::(?P<Account>\d{12}):response-plan/(?P<ResponsePlan>.+?)$"), ['response-plan']),
    ],
    'ssm-quicksetup': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm-quicksetup:(?P<Region>[\w-]*):(?P<Account>\d{12}):configuration-manager/(?P<ConfigurationManagerId>.+?)$"), ['configuration-manager']),
    ],
    'ssm-sap': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm-sap:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<ApplicationType>.+?)/(?P<ApplicationId>.+?)/COMPONENT/(?P<ComponentId>.+?)$"), ['component']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm-sap:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<ApplicationType>.+?)/(?P<ApplicationId>.+?)/DB/(?P<DatabaseId>.+?)$"), ['database']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ssm-sap:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<ApplicationType>.+?)/(?P<ApplicationId>.+?)$"), ['application']),
    ],
    'sso': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sso:::permissionSet/(?P<InstanceId>.+?)/(?P<PermissionSetId>.+?)$"), ['PermissionSet']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sso:::account/(?P<AccountId>.+?)$"), ['Account']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sso:::instance/(?P<InstanceId>.+?)$"), ['Instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sso::aws:applicationProvider/(?P<ApplicationProviderId>.+?)$"), ['ApplicationProvider']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sso::(?P<AccountId>.+?):application/(?P<InstanceId>.+?)/(?P<ApplicationId>.+?)$"), ['Application']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sso::(?P<AccountId>.+?):trustedTokenIssuer/(?P<InstanceId>.+?)/(?P<TrustedTokenIssuerId>.+?)$"), ['TrustedTokenIssuer']),
    ],
    'states': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):states:(?P<Region>[\w-]*):(?P<Account>\d{12}):express:(?P<StateMachineName>.+?)/(?P<MapRunLabel>.+?):(?P<ExecutionId>.+?):(?P<ExpressId>.+?)$"), ['labelled express']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):states:(?P<Region>[\w-]*):(?P<Account>\d{12}):execution:(?P<StateMachineName>.+?)/(?P<MapRunLabel>.+?):(?P<ExecutionId>.+?)$"), ['labelled execution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):states:(?P<Region>[\w-]*):(?P<Account>\d{12}):express:(?P<StateMachineName>.+?):(?P<ExecutionId>.+?):(?P<ExpressId>.+?)$"), ['express']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):states:(?P<Region>[\w-]*):(?P<Account>\d{12}):mapRun:(?P<StateMachineName>.+?)/(?P<MapRunLabel>.+?):(?P<MapRunId>.+?)$"), ['maprun']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):states:(?P<Region>[\w-]*):(?P<Account>\d{12}):execution:(?P<StateMachineName>.+?):(?P<ExecutionId>.+?)$"), ['execution']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):states:(?P<Region>[\w-]*):(?P<Account>\d{12}):stateMachine:(?P<StateMachineName>.+?):(?P<StateMachineVersionId>.+?)$"), ['statemachineversion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):states:(?P<Region>[\w-]*):(?P<Account>\d{12}):stateMachine:(?P<StateMachineName>.+?):(?P<StateMachineAliasName>.+?)$"), ['statemachinealias']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):states:(?P<Region>[\w-]*):(?P<Account>\d{12}):activity:(?P<ActivityName>.+?)$"), ['activity']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):states:(?P<Region>[\w-]*):(?P<Account>\d{12}):stateMachine:(?P<StateMachineName>.+?)$"), ['statemachine']),
    ],
    'storagegateway': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):storagegateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):gateway/(?P<GatewayId>.+?)/device/(?P<Vtldevice>.+?)$"), ['device']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):storagegateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):gateway/(?P<GatewayId>.+?)/target/(?P<IscsiTarget>.+?)$"), ['target']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):storagegateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):gateway/(?P<GatewayId>.+?)/volume/(?P<VolumeId>.+?)$"), ['volume']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):storagegateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):share/(?P<ShareId>.+?)/cache-report/(?P<CacheReportId>.+?)$"), ['cache-report']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):storagegateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):fs-association/(?P<FsaId>.+?)$"), ['fs-association']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):storagegateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):gateway/(?P<GatewayId>.+?)$"), ['gateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):storagegateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):share/(?P<ShareId>.+?)$"), ['share']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):storagegateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):tape/(?P<TapeBarcode>.+?)$"), ['tape']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):storagegateway:(?P<Region>[\w-]*):(?P<Account>\d{12}):tapepool/(?P<PoolId>.+?)$"), ['tapepool']),
    ],
    'sts': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sts::(?P<Account>\d{12}):federated-user/(?P<FederatedUserName>.+?)$"), ['federated-user']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):sts::(?P<Account>\d{12}):self$"), ['self-session']),
    ],
    'swf': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):swf::(?P<Account>\d{12}):/domain/(?P<DomainName>.+?)$"), ['domain']),
    ],
    'synthetics': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):synthetics:(?P<Region>[\w-]*):(?P<Account>\d{12}):canary:(?P<CanaryName>.+?)$"), ['canary']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):synthetics:(?P<Region>[\w-]*):(?P<Account>\d{12}):group:(?P<GroupId>.+?)$"), ['group']),
    ],
    'textract': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):textract:(?P<Region>[\w-]*):(?P<Account>\d{12}):/adapters/(?P<AdapterId>.+?)/versions/(?P<AdapterVersion>.+?)$"), ['adapterversion']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):textract:(?P<Region>[\w-]*):(?P<Account>\d{12}):/adapters/(?P<AdapterId>.+?)$"), ['adapter']),
    ],
    'thinclient': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):thinclient:(?P<Region>[\w-]*):(?P<Account>\d{12}):device/(?P<DeviceId>.+?)$"), ['device']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):thinclient:(?P<Region>[\w-]*):(?P<Account>\d{12}):environment/(?P<EnvironmentId>.+?)$"), ['environment']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):thinclient:(?P<Region>[\w-]*):(?P<Account>\d{12}):softwareset/(?P<SoftwareSetId>.+?)$"), ['softwareset']),
    ],
    'timestream': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):timestream:(?P<Region>[\w-]*):(?P<Account>\d{12}):database/(?P<DatabaseName>.+?)/table/(?P<TableName>.+?)$"), ['table']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):timestream:(?P<Region>[\w-]*):(?P<Account>\d{12}):database/(?P<DatabaseName>.+?)$"), ['database']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):timestream:(?P<Region>[\w-]*):(?P<Account>\d{12}):scheduled-query/(?P<ScheduledQueryName>.+?)$"), ['scheduled-query']),
    ],
    'timestream-influxdb': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):timestream-influxdb:(?P<Region>[\w-]*):(?P<Account>\d{12}):db-cluster/(?P<DbClusterId>.+?)$"), ['db-cluster']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):timestream-influxdb:(?P<Region>[\w-]*):(?P<Account>\d{12}):db-instance/(?P<DbInstanceIdentifier>.+?)$"), ['db-instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):timestream-influxdb:(?P<Region>[\w-]*):(?P<Account>\d{12}):db-parameter-group/(?P<DbParameterGroupIdentifier>.+?)$"), ['db-parameter-group']),
    ],
    'tnb': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):tnb:(?P<Region>[\w-]*):(?P<Account>\d{12}):function-instance/(?P<FunctionInstanceId>.+?)$"), ['function-instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):tnb:(?P<Region>[\w-]*):(?P<Account>\d{12}):function-package/(?P<FunctionPackageId>.+?)$"), ['function-package']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):tnb:(?P<Region>[\w-]*):(?P<Account>\d{12}):network-instance/(?P<NetworkInstanceId>.+?)$"), ['network-instance']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):tnb:(?P<Region>[\w-]*):(?P<Account>\d{12}):network-operation/(?P<NetworkOperationId>.+?)$"), ['network-operation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):tnb:(?P<Region>[\w-]*):(?P<Account>\d{12}):network-package/(?P<NetworkPackageId>.+?)$"), ['network-package']),
    ],
    'transcribe': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transcribe:(?P<Region>[\w-]*):(?P<Account>\d{12}):analytics/(?P<JobName>.+?)$"), ['callanalyticsjob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transcribe:(?P<Region>[\w-]*):(?P<Account>\d{12}):analytics-category/(?P<CategoryName>.+?)$"), ['callanalyticscategory']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transcribe:(?P<Region>[\w-]*):(?P<Account>\d{12}):language-model/(?P<ModelName>.+?)$"), ['languagemodel']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transcribe:(?P<Region>[\w-]*):(?P<Account>\d{12}):medical-scribe-job/(?P<JobName>.+?)$"), ['medicalscribejob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transcribe:(?P<Region>[\w-]*):(?P<Account>\d{12}):medical-transcription-job/(?P<JobName>.+?)$"), ['medicaltranscriptionjob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transcribe:(?P<Region>[\w-]*):(?P<Account>\d{12}):medical-vocabulary/(?P<VocabularyName>.+?)$"), ['medicalvocabulary']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transcribe:(?P<Region>[\w-]*):(?P<Account>\d{12}):transcription-job/(?P<JobName>.+?)$"), ['transcriptionjob']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transcribe:(?P<Region>[\w-]*):(?P<Account>\d{12}):vocabulary/(?P<VocabularyName>.+?)$"), ['vocabulary']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transcribe:(?P<Region>[\w-]*):(?P<Account>\d{12}):vocabulary-filter/(?P<VocabularyFilterName>.+?)$"), ['vocabularyfilter']),
    ],
    'transfer': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transfer:(?P<Region>[\w-]*):(?P<Account>\d{12}):agreement/(?P<ServerId>.+?)/(?P<AgreementId>.+?)$"), ['agreement']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transfer:(?P<Region>[\w-]*):(?P<Account>\d{12}):host-key/(?P<ServerId>.+?)/(?P<HostKeyId>.+?)$"), ['host-key']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transfer:(?P<Region>[\w-]*):(?P<Account>\d{12}):user/(?P<ServerId>.+?)/(?P<UserName>.+?)$"), ['user']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transfer:(?P<Region>[\w-]*):(?P<Account>\d{12}):certificate/(?P<CertificateId>.+?)$"), ['certificate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transfer:(?P<Region>[\w-]*):(?P<Account>\d{12}):connector/(?P<ConnectorId>.+?)$"), ['connector']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transfer:(?P<Region>[\w-]*):(?P<Account>\d{12}):profile/(?P<ProfileId>.+?)$"), ['profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transfer:(?P<Region>[\w-]*):(?P<Account>\d{12}):server/(?P<ServerId>.+?)$"), ['server']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transfer:(?P<Region>[\w-]*):(?P<Account>\d{12}):webapp/(?P<WebAppId>.+?)$"), ['webapp']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transfer:(?P<Region>[\w-]*):(?P<Account>\d{12}):workflow/(?P<WorkflowId>.+?)$"), ['workflow']),
    ],
    'transform': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transform:(?P<Region>[\w-]*):(?P<Account>\d{12}):connector/(?P<WorkspaceId>.+?)/(?P<ConnectorId>.+?)$"), ['connector']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transform:(?P<Region>[\w-]*):(?P<Account>\d{12}):profile/(?P<Identifier>.+?)$"), ['profile']),
    ],
    'transform-custom': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transform-custom:(?P<Region>[\w-]*):(?P<Account>\d{12}):package/(?P<TransformationPackageName>.+?)/knowledge-item/(?P<Id>.+?)$"), ['knowledge-item']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transform-custom:(?P<Region>[\w-]*):(?P<Account>\d{12}):campaign/(?P<Name>.+?)$"), ['campaign']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):transform-custom:(?P<Region>[\w-]*):(?P<Account>\d{12}):package/(?P<Name>.+?)$"), ['package']),
    ],
    'translate': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):translate:(?P<Region>[\w-]*):(?P<Account>\d{12}):parallel-data/(?P<ResourceName>.+?)$"), ['parallel-data']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):translate:(?P<Region>[\w-]*):(?P<Account>\d{12}):terminology/(?P<ResourceName>.+?)$"), ['terminology']),
    ],
    'trustedadvisor': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):trustedadvisor:(?P<Region>[\w-]*):(?P<Account>\d{12}):checks/(?P<CategoryCode>.+?)/(?P<CheckId>.+?)$"), ['checks']),
    ],
    'ts': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ts::aws:tool/(?P<ToolId>.+?)$"), ['tool']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):ts::(?P<Account>\d{12}):execution/(?P<UserId>.+?)/(?P<ToolId>.+?)/(?P<ExecutionId>.+?)$"), ['execution']),
    ],
    'vendor-insights': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vendor-insights:::data-source:(?P<ResourceId>.+?)$"), ['DataSource']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vendor-insights:::security-profile:(?P<ResourceId>.+?)$"), ['SecurityProfile']),
    ],
    'verifiedpermissions': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):verifiedpermissions::(?P<Account>\d{12}):policy-store/(?P<PolicyStoreId>.+?)$"), ['policy-store']),
    ],
    'voiceid': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):voiceid:(?P<Region>[\w-]*):(?P<Account>\d{12}):domain/(?P<DomainId>.+?)$"), ['domain']),
    ],
    'vpc-lattice': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vpc-lattice:(?P<Region>[\w-]*):(?P<Account>\d{12}):service/(?P<ServiceId>.+?)/listener/(?P<ListenerId>.+?)/rule/(?P<RuleId>.+?)$"), ['Rule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vpc-lattice:(?P<Region>[\w-]*):(?P<Account>\d{12}):service/(?P<ServiceId>.+?)/listener/(?P<ListenerId>.+?)$"), ['Listener']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vpc-lattice:(?P<Region>[\w-]*):(?P<Account>\d{12}):service/(?P<ServiceId>.+?)/(?P<RequestPath>.+?)$"), ['Service']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vpc-lattice:(?P<Region>[\w-]*):(?P<Account>\d{12}):accesslogsubscription/(?P<AccessLogSubscriptionId>.+?)$"), ['AccessLogSubscription']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vpc-lattice:(?P<Region>[\w-]*):(?P<Account>\d{12}):domainverification/(?P<DomainVerificationId>.+?)$"), ['DomainVerification']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vpc-lattice:(?P<Region>[\w-]*):(?P<Account>\d{12}):resourceconfiguration/(?P<ResourceConfigurationId>.+?)$"), ['ResourceConfiguration']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vpc-lattice:(?P<Region>[\w-]*):(?P<Account>\d{12}):resourceendpointassociation/(?P<ResourceEndpointAssociationId>.+?)$"), ['ResourceEndpointAssociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vpc-lattice:(?P<Region>[\w-]*):(?P<Account>\d{12}):resourcegateway/(?P<ResourceGatewayId>.+?)$"), ['ResourceGateway']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vpc-lattice:(?P<Region>[\w-]*):(?P<Account>\d{12}):service/(?P<ServiceId>.+?)$"), ['Service']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vpc-lattice:(?P<Region>[\w-]*):(?P<Account>\d{12}):servicenetwork/(?P<ServiceNetworkId>.+?)$"), ['ServiceNetwork']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vpc-lattice:(?P<Region>[\w-]*):(?P<Account>\d{12}):servicenetworkresourceassociation/(?P<ServiceNetworkResourceAssociationId>.+?)$"), ['ServiceNetworkResourceAssociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vpc-lattice:(?P<Region>[\w-]*):(?P<Account>\d{12}):servicenetworkserviceassociation/(?P<ServiceNetworkServiceAssociationId>.+?)$"), ['ServiceNetworkServiceAssociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vpc-lattice:(?P<Region>[\w-]*):(?P<Account>\d{12}):servicenetworkvpcassociation/(?P<ServiceNetworkVpcAssociationId>.+?)$"), ['ServiceNetworkVpcAssociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):vpc-lattice:(?P<Region>[\w-]*):(?P<Account>\d{12}):targetgroup/(?P<TargetGroupId>.+?)$"), ['TargetGroup']),
    ],
    'waf': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf::(?P<Account>\d{12}):bytematchset/(?P<Id>.+?)$"), ['bytematchset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf::(?P<Account>\d{12}):geomatchset/(?P<Id>.+?)$"), ['geomatchset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf::(?P<Account>\d{12}):ipset/(?P<Id>.+?)$"), ['ipset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf::(?P<Account>\d{12}):ratebasedrule/(?P<Id>.+?)$"), ['ratebasedrule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf::(?P<Account>\d{12}):regexmatch/(?P<Id>.+?)$"), ['regexmatchset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf::(?P<Account>\d{12}):regexpatternset/(?P<Id>.+?)$"), ['regexpatternset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf::(?P<Account>\d{12}):rule/(?P<Id>.+?)$"), ['rule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf::(?P<Account>\d{12}):rulegroup/(?P<Id>.+?)$"), ['rulegroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf::(?P<Account>\d{12}):sizeconstraintset/(?P<Id>.+?)$"), ['sizeconstraintset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf::(?P<Account>\d{12}):sqlinjectionset/(?P<Id>.+?)$"), ['sqlinjectionmatchset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf::(?P<Account>\d{12}):webacl/(?P<Id>.+?)$"), ['webacl']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf::(?P<Account>\d{12}):xssmatchset/(?P<Id>.+?)$"), ['xssmatchset']),
    ],
    'waf-regional': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf-regional:(?P<Region>[\w-]*):(?P<Account>\d{12}):bytematchset/(?P<Id>.+?)$"), ['bytematchset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf-regional:(?P<Region>[\w-]*):(?P<Account>\d{12}):geomatchset/(?P<Id>.+?)$"), ['geomatchset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf-regional:(?P<Region>[\w-]*):(?P<Account>\d{12}):ipset/(?P<Id>.+?)$"), ['ipset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf-regional:(?P<Region>[\w-]*):(?P<Account>\d{12}):ratebasedrule/(?P<Id>.+?)$"), ['ratebasedrule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf-regional:(?P<Region>[\w-]*):(?P<Account>\d{12}):regexmatch/(?P<Id>.+?)$"), ['regexmatchset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf-regional:(?P<Region>[\w-]*):(?P<Account>\d{12}):regexpatternset/(?P<Id>.+?)$"), ['regexpatternset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf-regional:(?P<Region>[\w-]*):(?P<Account>\d{12}):rule/(?P<Id>.+?)$"), ['rule']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf-regional:(?P<Region>[\w-]*):(?P<Account>\d{12}):rulegroup/(?P<Id>.+?)$"), ['rulegroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf-regional:(?P<Region>[\w-]*):(?P<Account>\d{12}):sizeconstraintset/(?P<Id>.+?)$"), ['sizeconstraintset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf-regional:(?P<Region>[\w-]*):(?P<Account>\d{12}):sqlinjectionset/(?P<Id>.+?)$"), ['sqlinjectionmatchset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf-regional:(?P<Region>[\w-]*):(?P<Account>\d{12}):webacl/(?P<Id>.+?)$"), ['webacl']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):waf-regional:(?P<Region>[\w-]*):(?P<Account>\d{12}):xssmatchset/(?P<Id>.+?)$"), ['xssmatchset']),
    ],
    'wafv2': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wafv2:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<Scope>.+?)/ipset/(?P<Name>.+?)/(?P<Id>.+?)$"), ['ipset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wafv2:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<Scope>.+?)/managedruleset/(?P<Name>.+?)/(?P<Id>.+?)$"), ['managedruleset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wafv2:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<Scope>.+?)/regexpatternset/(?P<Name>.+?)/(?P<Id>.+?)$"), ['regexpatternset']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wafv2:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<Scope>.+?)/rulegroup/(?P<Name>.+?)/(?P<Id>.+?)$"), ['rulegroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wafv2:(?P<Region>[\w-]*):(?P<Account>\d{12}):(?P<Scope>.+?)/webacl/(?P<Name>.+?)/(?P<Id>.+?)$"), ['webacl']),
    ],
    'wellarchitected': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wellarchitected:(?P<Region>[\w-]*):(?P<Account>\d{12}):lens/(?P<ResourceId>.+?)$"), ['lens']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wellarchitected:(?P<Region>[\w-]*):(?P<Account>\d{12}):profile/(?P<ResourceId>.+?)$"), ['profile']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wellarchitected:(?P<Region>[\w-]*):(?P<Account>\d{12}):review-template/(?P<ResourceId>.+?)$"), ['review-template']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wellarchitected:(?P<Region>[\w-]*):(?P<Account>\d{12}):workload/(?P<ResourceId>.+?)$"), ['workload']),
    ],
    'wickr': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wickr:(?P<Region>[\w-]*):(?P<Account>\d{12}):network/(?P<NetworkId>.+?)$"), ['network']),
    ],
    'wisdom': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wisdom:(?P<Region>[\w-]*):(?P<Account>\d{12}):ai-agent/(?P<AssistantId>.+?)/(?P<AIAgentId>.+?):(?P<Version>.+?)$"), ['ai-agent']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wisdom:(?P<Region>[\w-]*):(?P<Account>\d{12}):content-association/(?P<KnowledgeBaseId>.+?)/(?P<ContentId>.+?)/(?P<ContentAssociationId>.+?)$"), ['ContentAssociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wisdom:(?P<Region>[\w-]*):(?P<Account>\d{12}):ai-agent/(?P<AssistantId>.+?)/(?P<AIAgentId>.+?)$"), ['AIAgent']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wisdom:(?P<Region>[\w-]*):(?P<Account>\d{12}):ai-guardrail/(?P<AssistantId>.+?)/(?P<AIGuardrailId>.+?)$"), ['AIGuardrail']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wisdom:(?P<Region>[\w-]*):(?P<Account>\d{12}):ai-prompt/(?P<AssistantId>.+?)/(?P<AIPromptId>.+?)$"), ['AIPrompt']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wisdom:(?P<Region>[\w-]*):(?P<Account>\d{12}):association/(?P<AssistantId>.+?)/(?P<AssistantAssociationId>.+?)$"), ['AssistantAssociation']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wisdom:(?P<Region>[\w-]*):(?P<Account>\d{12}):content/(?P<KnowledgeBaseId>.+?)/(?P<ContentId>.+?)$"), ['Content']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wisdom:(?P<Region>[\w-]*):(?P<Account>\d{12}):message-template/(?P<KnowledgeBaseId>.+?)/(?P<MessageTemplateId>.+?)$"), ['MessageTemplate']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wisdom:(?P<Region>[\w-]*):(?P<Account>\d{12}):quick-response/(?P<KnowledgeBaseId>.+?)/(?P<QuickResponseId>.+?)$"), ['QuickResponse']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wisdom:(?P<Region>[\w-]*):(?P<Account>\d{12}):session/(?P<AssistantId>.+?)/(?P<SessionId>.+?)$"), ['Session']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wisdom:(?P<Region>[\w-]*):(?P<Account>\d{12}):assistant/(?P<AssistantId>.+?)$"), ['Assistant']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):wisdom:(?P<Region>[\w-]*):(?P<Account>\d{12}):knowledge-base/(?P<KnowledgeBaseId>.+?)$"), ['KnowledgeBase']),
    ],
    'workdocs': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workdocs:(?P<Region>[\w-]*):(?P<Account>\d{12}):organization/(?P<ResourceId>.+?)$"), ['organization']),
    ],
    'worklink': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):worklink::(?P<Account>\d{12}):fleet/(?P<FleetName>.+?)$"), ['fleet']),
    ],
    'workmail': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workmail:(?P<Region>[\w-]*):(?P<Account>\d{12}):organization/(?P<ResourceId>.+?)$"), ['organization']),
    ],
    'workmailmessageflow': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workmailmessageflow:(?P<Region>[\w-]*):(?P<Account>\d{12}):message/(?P<OrganizationId>.+?)/(?P<Context>.+?)/(?P<MessageId>.+?)$"), ['RawMessage']),
    ],
    'workspaces': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces:(?P<Region>[\w-]*):(?P<Account>\d{12}):connectionalias/(?P<ConnectionAliasId>.+?)$"), ['connectionalias']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces:(?P<Region>[\w-]*):(?P<Account>\d{12}):directory/(?P<DirectoryId>.+?)$"), ['directoryid']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces:(?P<Region>[\w-]*):(?P<Account>\d{12}):workspace/(?P<WorkspaceId>.+?)$"), ['workspaceid']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces:(?P<Region>[\w-]*):(?P<Account>\d{12}):workspaceapplication/(?P<WorkSpaceApplicationId>.+?)$"), ['workspaceapplication']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces:(?P<Region>[\w-]*):(?P<Account>\d{12}):workspacebundle/(?P<BundleId>.+?)$"), ['workspacebundle']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces:(?P<Region>[\w-]*):(?P<Account>\d{12}):workspacecertificate/(?P<CertificateId>.+?)$"), ['certificateid']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces:(?P<Region>[\w-]*):(?P<Account>\d{12}):workspaceimage/(?P<ImageId>.+?)$"), ['workspaceimage']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces:(?P<Region>[\w-]*):(?P<Account>\d{12}):workspaceipgroup/(?P<GroupId>.+?)$"), ['workspaceipgroup']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces:(?P<Region>[\w-]*):(?P<Account>\d{12}):workspacespool/(?P<PoolId>.+?)$"), ['workspacespoolid']),
    ],
    'workspaces-instances': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces-instances:(?P<Region>[\w-]*):(?P<Account>\d{12}):workspaceinstance/(?P<WorkspaceInstanceId>.+?)$"), ['WorkspaceInstanceId']),
    ],
    'workspaces-web': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces-web:(?P<Region>[\w-]*):(?P<Account>\d{12}):identityProvider/(?P<PortalId>.+?)/(?P<IdentityProviderId>.+?)$"), ['identityProvider']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces-web:(?P<Region>[\w-]*):(?P<Account>\d{12}):browserSettings/(?P<BrowserSettingsId>.+?)$"), ['browserSettings']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces-web:(?P<Region>[\w-]*):(?P<Account>\d{12}):dataProtectionSettings/(?P<DataProtectionSettingsId>.+?)$"), ['dataProtectionSettings']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces-web:(?P<Region>[\w-]*):(?P<Account>\d{12}):ipAccessSettings/(?P<IpAccessSettingsId>.+?)$"), ['ipAccessSettings']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces-web:(?P<Region>[\w-]*):(?P<Account>\d{12}):networkSettings/(?P<NetworkSettingsId>.+?)$"), ['networkSettings']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces-web:(?P<Region>[\w-]*):(?P<Account>\d{12}):portal/(?P<PortalId>.+?)$"), ['portal']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces-web:(?P<Region>[\w-]*):(?P<Account>\d{12}):sessionLogger/(?P<SessionLoggerId>.+?)$"), ['sessionLogger']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces-web:(?P<Region>[\w-]*):(?P<Account>\d{12}):trustStore/(?P<TrustStoreId>.+?)$"), ['trustStore']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces-web:(?P<Region>[\w-]*):(?P<Account>\d{12}):userAccessLoggingSettings/(?P<UserAccessLoggingSettingsId>.+?)$"), ['userAccessLoggingSettings']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):workspaces-web:(?P<Region>[\w-]*):(?P<Account>\d{12}):userSettings/(?P<UserSettingsId>.+?)$"), ['userSettings']),
    ],
    'xray': [
        (re.compile(r"^arn:(?P<Partition>[\w-]+):xray:(?P<Region>[\w-]*):(?P<Account>\d{12}):group/(?P<GroupName>.+?)/(?P<Id>.+?)$"), ['group']),
        (re.compile(r"^arn:(?P<Partition>[\w-]+):xray:(?P<Region>[\w-]*):(?P<Account>\d{12}):sampling-rule/(?P<SamplingRuleName>.+?)$"), ['sampling-rule']),
    ],
}

# Auto-generated mapping: ARN service -> AWS SDK client names
AWS_SDK_SERVICES = {
    'a4b': [],
    'access-analyzer': ['accessanalyzer'],
    'account': ['account'],
    'acm': ['acm'],
    'acm-pca': ['acm-pca'],
    'aidevops': ['aiops'],
    'aiops': ['aiops'],
    'airflow': ['mwaa'],
    'airflow-serverless': ['mwaa-serverless'],
    'amplify': ['amplify'],
    'amplifybackend': ['amplifybackend'],
    'amplifyuibuilder': ['amplifyuibuilder'],
    'aoss': ['opensearchserverless'],
    'apigateway': ['apigateway', 'apigatewayv2'],
    'app-integrations': ['appintegrations'],
    'appconfig': ['appconfig', 'appconfigdata'],
    'appfabric': ['appfabric'],
    'appflow': ['appflow'],
    'application-autoscaling': ['application-autoscaling'],
    'application-signals': ['application-signals'],
    'appmesh': ['appmesh'],
    'appmesh-preview': ['appmesh'],
    'apprunner': ['apprunner'],
    'appstream': ['appstream'],
    'appstudio': [],
    'appsync': ['appsync'],
    'apptest': [],
    'aps': ['amp'],
    'arc-region-switch': ['arc-region-switch'],
    'artifact': ['artifact'],
    'athena': ['athena'],
    'auditmanager': ['auditmanager'],
    'autoscaling': ['autoscaling'],
    'aws-marketplace': ['marketplace-agreement', 'marketplace-catalog', 'marketplace-deployment', 'marketplace-entitlement', 'marketplace-reporting', 'meteringmarketplace'],
    'b2bi': ['b2bi'],
    'backup': ['backup'],
    'backup-gateway': ['backup-gateway'],
    'backup-search': ['backupsearch'],
    'batch': ['batch'],
    'bcm-data-exports': ['bcm-data-exports'],
    'bcm-pricing-calculator': ['bcm-pricing-calculator'],
    'bedrock': ['bedrock', 'bedrock-agent', 'bedrock-agent-runtime', 'bedrock-data-automation', 'bedrock-data-automation-runtime', 'bedrock-runtime'],
    'bedrock-agentcore': ['bedrock-agentcore', 'bedrock-agentcore-control'],
    'bedrock-mantle': [],
    'billing': ['billing'],
    'billingconductor': ['billingconductor'],
    'braket': ['braket'],
    'budgets': ['budgets'],
    'bugbust': [],
    'cases': ['connectcases'],
    'cassandra': ['keyspaces', 'keyspacesstreams'],
    'catalog': ['servicecatalog'],
    'ce': ['ce'],
    'chatbot': ['chatbot'],
    'chime': ['chime', 'chime-sdk-identity', 'chime-sdk-media-pipelines', 'chime-sdk-meetings', 'chime-sdk-messaging', 'chime-sdk-voice'],
    'cleanrooms': ['cleanrooms'],
    'cleanrooms-ml': ['cleanroomsml'],
    'cloud9': ['cloud9'],
    'clouddirectory': ['clouddirectory'],
    'cloudformation': ['cloudformation'],
    'cloudfront': ['cloudfront'],
    'cloudhsm': ['cloudhsm', 'cloudhsmv2'],
    'cloudsearch': ['cloudsearch', 'cloudsearchdomain'],
    'cloudshell': [],
    'cloudtrail': ['cloudtrail'],
    'cloudwatch': ['cloudwatch'],
    'codeartifact': ['codeartifact'],
    'codebuild': ['codebuild'],
    'codecatalyst': ['codecatalyst'],
    'codecommit': ['codecommit'],
    'codeconnections': ['codeconnections'],
    'codedeploy': ['codedeploy'],
    'codeguru-profiler': ['codeguruprofiler'],
    'codeguru-reviewer': ['codeguru-reviewer'],
    'codeguru-security': ['codeguru-security'],
    'codepipeline': ['codepipeline'],
    'codestar': [],
    'codestar-connections': ['codestar-connections'],
    'codestar-notifications': ['codestar-notifications'],
    'codewhisperer': [],
    'cognito-identity': ['cognito-identity'],
    'cognito-idp': ['cognito-idp'],
    'cognito-sync': ['cognito-sync'],
    'comprehend': ['comprehend'],
    'compute-optimizer': ['compute-optimizer'],
    'config': ['config'],
    'connect': ['connect', 'connect-contact-lens'],
    'connect-campaigns': ['connectcampaigns', 'connectcampaignsv2'],
    'consoleapp': [],
    'controlcatalog': ['controlcatalog'],
    'controltower': ['controltower'],
    'cur': ['cur'],
    'databrew': ['databrew'],
    'dataexchange': ['dataexchange'],
    'datapipeline': ['datapipeline'],
    'datasync': ['datasync'],
    'datazone': ['datazone'],
    'dax': ['dax'],
    'deadline': ['deadline'],
    'detective': ['detective'],
    'devicefarm': ['devicefarm'],
    'directconnect': ['directconnect'],
    'dlm': ['dlm'],
    'dms': ['dms'],
    'docdb-elastic': ['docdb-elastic'],
    'drs': ['drs'],
    'ds': ['ds'],
    'dsql': ['dsql'],
    'dynamodb': ['dynamodb', 'dynamodbstreams'],
    'ec2': ['ec2'],
    'ecr': ['ecr'],
    'ecr-public': ['ecr-public'],
    'ecs': ['ecs'],
    'eks': ['eks'],
    'elastic-inference': [],
    'elasticache': ['elasticache'],
    'elasticbeanstalk': ['elasticbeanstalk'],
    'elasticfilesystem': ['efs'],
    'elasticloadbalancing': ['elb', 'elbv2'],
    'elasticmapreduce': ['emr'],
    'elastictranscoder': [],
    'elemental-appliances-software': [],
    'elemental-support-cases': [],
    'emr-containers': ['emr-containers'],
    'emr-serverless': ['emr-serverless'],
    'entityresolution': ['entityresolution'],
    'es': ['es', 'opensearch'],
    'events': ['events'],
    'evidently': ['evidently'],
    'evs': ['evs'],
    'execute-api': ['apigatewaymanagementapi', 'connectparticipant'],
    'finspace': ['finspace'],
    'finspace-api': ['finspace-data'],
    'firehose': ['firehose'],
    'fis': ['fis'],
    'fms': ['fms'],
    'forecast': ['forecast', 'forecastquery'],
    'frauddetector': ['frauddetector'],
    'freertos': [],
    'fsx': ['fsx'],
    'gamelift': ['gamelift'],
    'gameliftstreams': ['gameliftstreams'],
    'geo': ['location'],
    'geo-maps': ['geo-maps'],
    'geo-places': ['geo-places'],
    'geo-routes': ['geo-routes'],
    'glacier': ['glacier'],
    'globalaccelerator': ['globalaccelerator'],
    'glue': ['glue'],
    'grafana': ['grafana'],
    'greengrass': ['greengrass', 'greengrassv2'],
    'groundstation': ['groundstation'],
    'guardduty': ['guardduty'],
    'health': ['health'],
    'healthlake': ['healthlake'],
    'honeycode': [],
    'iam': ['iam'],
    'identity-sync': [],
    'identitystore': ['identitystore'],
    'imagebuilder': ['imagebuilder'],
    'inspector': ['inspector'],
    'inspector2': ['inspector2'],
    'internetmonitor': ['internetmonitor'],
    'invoicing': ['invoicing'],
    'iot': ['iot'],
    'iotanalytics': ['iotanalytics'],
    'iotdeviceadvisor': ['iotdeviceadvisor'],
    'iotevents': ['iotevents'],
    'iotfleethub': [],
    'iotfleetwise': ['iotfleetwise'],
    'iotmanagedintegrations': ['iot-managed-integrations'],
    'iotsitewise': ['iotsitewise'],
    'iottwinmaker': ['iottwinmaker'],
    'iotwireless': ['iotwireless'],
    'iq': [],
    'iq-permission': [],
    'ivs': ['ivs', 'ivs-realtime'],
    'ivschat': ['ivschat'],
    'kafka': ['kafka'],
    'kafkaconnect': ['kafkaconnect'],
    'kendra': ['kendra'],
    'kendra-ranking': ['kendra-ranking'],
    'kinesis': ['kinesis'],
    'kinesisanalytics': ['kinesisanalytics', 'kinesisanalyticsv2'],
    'kinesisvideo': ['kinesisvideo', 'kinesis-video-archived-media', 'kinesis-video-media', 'kinesis-video-signaling', 'kinesis-video-webrtc-storage'],
    'kms': ['kms'],
    'lambda': ['lambda'],
    'launchwizard': ['launch-wizard'],
    'lex': ['lex-models', 'lex-runtime', 'lexv2-models', 'lexv2-runtime'],
    'license-manager': ['license-manager'],
    'license-manager-linux-subscriptions': ['license-manager-linux-subscriptions'],
    'license-manager-user-subscriptions': ['license-manager-user-subscriptions'],
    'lightsail': ['lightsail'],
    'logs': ['logs'],
    'lookoutequipment': ['lookoutequipment'],
    'lookoutmetrics': [],
    'lookoutvision': [],
    'm2': ['m2'],
    'machinelearning': ['machinelearning'],
    'macie2': ['macie2'],
    'managedblockchain': ['managedblockchain'],
    'mapcredits': [],
    'mediaconnect': ['mediaconnect'],
    'mediaconvert': ['mediaconvert'],
    'medialive': ['medialive'],
    'mediapackage': ['mediapackage'],
    'mediapackage-vod': ['mediapackage-vod'],
    'mediapackagev2': ['mediapackagev2'],
    'mediastore': ['mediastore', 'mediastore-data'],
    'mediatailor': ['mediatailor'],
    'medical-imaging': ['medical-imaging'],
    'memorydb': ['memorydb'],
    'mgh': ['mgh', 'migrationhub-config'],
    'mgn': ['mgn'],
    'migrationhub-orchestrator': ['migrationhuborchestrator'],
    'mobiletargeting': ['pinpoint'],
    'monitron': [],
    'mpa': ['mpa'],
    'mq': ['mq'],
    'neptune-db': ['neptunedata'],
    'neptune-graph': ['neptune-graph'],
    'network-firewall': ['network-firewall'],
    'networkflowmonitor': ['networkflowmonitor'],
    'networkmanager': ['networkmanager'],
    'networkmonitor': ['networkmonitor'],
    'nimble': [],
    'notifications': ['notifications'],
    'notifications-contacts': ['notificationscontacts'],
    'nova-act': ['nova-act'],
    'oam': ['oam'],
    'observabilityadmin': ['observabilityadmin'],
    'odb': ['odb'],
    'omics': ['omics'],
    'one': [],
    'opensearch': ['opensearch'],
    'opsworks': [],
    'opsworks-cm': [],
    'organizations': ['organizations'],
    'osis': ['osis'],
    'outposts': ['outposts'],
    'panorama': ['panorama'],
    'partnercentral': ['partnercentral-account', 'partnercentral-benefits', 'partnercentral-channel', 'partnercentral-selling'],
    'payment-cryptography': ['payment-cryptography', 'payment-cryptography-data'],
    'payments': [],
    'pca-connector-ad': ['pca-connector-ad'],
    'pca-connector-scep': ['pca-connector-scep'],
    'pcs': ['pcs'],
    'personalize': ['personalize', 'personalize-events', 'personalize-runtime'],
    'pi': ['pi'],
    'pipes': ['pipes'],
    'polly': ['polly'],
    'pricingplanmanager': [],
    'private-networks': ['privatenetworks'],
    'profile': ['customer-profiles'],
    'proton': ['proton'],
    'purchase-orders': [],
    'qapps': ['qapps'],
    'qbusiness': ['qbusiness'],
    'qdeveloper': [],
    'qldb': [],
    'quicksight': ['quicksight'],
    'ram': ['ram'],
    'rbin': ['rbin'],
    'rds': ['rds', 'docdb', 'neptune'],
    'rds-db': ['rds'],
    'redshift': ['redshift'],
    'redshift-serverless': ['redshift-serverless'],
    'refactor-spaces': ['migration-hub-refactor-spaces'],
    'rekognition': ['rekognition'],
    'repostspace': ['repostspace'],
    'resiliencehub': ['resiliencehub'],
    'resource-explorer-2': ['resource-explorer-2'],
    'resource-groups': ['resource-groups'],
    'robomaker': [],
    'rolesanywhere': ['rolesanywhere'],
    'route53': ['route53'],
    'route53-recovery-control': ['route53-recovery-cluster', 'route53-recovery-control-config'],
    'route53-recovery-readiness': ['route53-recovery-readiness'],
    'route53globalresolver': ['route53globalresolver'],
    'route53profiles': ['route53profiles'],
    'route53resolver': ['route53resolver'],
    'rum': ['rum'],
    's3': ['s3', 's3control'],
    's3-object-lambda': ['s3'],
    's3-outposts': ['s3outposts'],
    's3express': ['s3'],
    's3tables': ['s3tables'],
    's3vectors': ['s3vectors'],
    'sagemaker': ['sagemaker', 'sagemaker-a2i-runtime', 'sagemaker-edge', 'sagemaker-featurestore-runtime', 'sagemaker-metrics', 'sagemaker-runtime'],
    'sagemaker-geospatial': ['sagemaker-geospatial'],
    'savingsplans': ['savingsplans'],
    'scheduler': ['scheduler'],
    'schemas': ['schemas'],
    'scn': ['supplychain'],
    'sdb': ['sdb'],
    'secretsmanager': ['secretsmanager'],
    'security-ir': ['security-ir'],
    'securityagent': [],
    'securityhub': ['securityhub'],
    'securitylake': ['securitylake'],
    'serverlessrepo': ['serverlessrepo'],
    'servicecatalog': ['servicecatalog', 'servicecatalog-appregistry'],
    'servicediscovery': ['servicediscovery'],
    'servicequotas': ['service-quotas'],
    'ses': ['ses', 'mailmanager', 'pinpoint-email', 'sesv2'],
    'shield': ['shield'],
    'signer': ['signer'],
    'simspaceweaver': ['simspaceweaver'],
    'sms-voice': ['sms-voice', 'pinpoint-sms-voice', 'pinpoint-sms-voice-v2'],
    'snow-device-management': ['snow-device-management'],
    'sns': ['sns'],
    'social-messaging': ['socialmessaging'],
    'sqlworkbench': [],
    'sqs': ['sqs'],
    'ssm': ['ssm'],
    'ssm-contacts': ['ssm-contacts'],
    'ssm-incidents': ['ssm-incidents'],
    'ssm-quicksetup': ['ssm-quicksetup'],
    'ssm-sap': ['ssm-sap'],
    'sso': ['sso', 'sso-admin'],
    'states': ['stepfunctions'],
    'storagegateway': ['storagegateway'],
    'sts': ['sts'],
    'swf': ['swf'],
    'synthetics': ['synthetics'],
    'textract': ['textract'],
    'thinclient': ['workspaces-thin-client'],
    'timestream': ['timestream-query', 'timestream-write'],
    'timestream-influxdb': ['timestream-influxdb'],
    'tnb': ['tnb'],
    'transcribe': ['transcribe'],
    'transfer': ['transfer'],
    'transform': [],
    'transform-custom': [],
    'translate': ['translate'],
    'trustedadvisor': ['trustedadvisor'],
    'ts': [],
    'vendor-insights': [],
    'verifiedpermissions': ['verifiedpermissions'],
    'voiceid': ['voice-id'],
    'vpc-lattice': ['vpc-lattice'],
    'waf': ['waf'],
    'waf-regional': ['waf-regional'],
    'wafv2': ['wafv2'],
    'wellarchitected': ['wellarchitected'],
    'wickr': ['wickr'],
    'wisdom': ['wisdom', 'qconnect'],
    'workdocs': ['workdocs'],
    'worklink': [],
    'workmail': ['workmail'],
    'workmailmessageflow': ['workmailmessageflow'],
    'workspaces': ['workspaces'],
    'workspaces-instances': ['workspaces-instances'],
    'workspaces-web': ['workspaces-web'],
    'xray': ['xray'],
}
